
!******************************************************************************
!  
!  Diagnostic Post-Processor (Diapost) <Kevin.Yeh@noaa.gov> 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
!  This program is a post-processor for WRF-NMM and its varieties (HWRF etc.)
!  with features for hurricane diagnostics, such as storm tracker, wind swath, 
!  and cylindrical grid. Starting from the model's NetCDF output on a rotated 
!  E-grid at sigma-p levels, variables are extracted & converted to an A-grid 
!  for analysis and display with GrADS, they are also vertically interpolated 
!  to pressure & height levels for more analysis. Dr. Frank Marks, Rob Rogers 
!  and many other colleagues at NOAA AOML Hurricane Research Division (HRD) 
!  have contributed to the development of Diapost with insightful comments 
!  and suggestions. See the script Diapost for usage. 
!  
!  0a. I/O preparation for data processing 
!  0b. Read original E-grid data & convert to A-grid 
!  0c. Coordinates & geometry of the rotated A-grid 
!  0d. Derive diagnostic variables on sigma-p levels 
!  1a. Calculate storm center and intensity (tracker) 
!  1b. Interpolate to static domain for wind & rain swath 
!  1c. Sigma-p-level variables on rotated latlon grid 
!  2a. 25mb-pressure-level variables on rotated latlon grid 
!  2b. 50mb-pressure-level variables on standard latlon grid 
!  3a. Height-level variables on rotated latlon grid (dp = ~25mb) 
!  3b. Height-level variables on cylindrical grid (dp = ~25mb) 
!  3c. Diagnostics with height-level variables on rotated grid 
!  
!______________________________________________________________________________


PROGRAM diapost !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

   USE  wrf_data
   IMPLICIT NONE
#include "wrf_status_codes.h"
#include "netcdf.inc"
   Integer, external :: iargc 
   Logical, external :: iveceq 
   Character(len=240):: filename 
   Character(len=20) :: argument 
   Character(len=1)  :: storm_bas 
   Character(len=3)  :: ordering, ord 
   Character(len=2)  :: storm_basin, storm_num 
   Character(len=4)  :: staggering, atcf_code 
   Character(len=80) :: SysInfo, dimnames(3) 
   Character(len=19) :: DateStr, VarName, variable(99) 
   Integer:: iorder  =  4        !* 2/4: order of interpolation 
   Integer:: ibdcut  =  3        !* cutoff bad bdry points in x-dir 
   Integer:: jbdcut  =  3        !* cutoff bad bdry points in y-dir 
   Integer:: itermax =  10       !* max # of iterations for tracker 
   Integer:: iterpot =  10       !* max # of iterations for eq.pot.temp 
   Integer:: iterlax =  20       !* total # of iterations for relaxation  
   Real(4):: overlax = 0.8       !* over-relaxation coefficient for MSLP 
   Real(4):: low_res = 0.1       !* min resol  (deg)  for grid-refining 
   Real(4):: r_slope = 20.       !* max slope (dr/dz) for storm-tilting 
   Real(4):: r_inner = 1.5       !* inner storm-searching radius (deg) 
   Real(4):: r_outer = 2.0       !* outer storm-searching radius (deg) 
   Real(4):: r_extra = 4.0       !* extra storm-searching radius (deg) 
   Real(4):: v_close = 4.0       !* min avg Vt (m/s) for closed circulation 
   Integer:: msearch = 5         !* # of radii for storm-searching 
   Real(4):: rsearch(5)          !* radii for storm-searching (deg) 
      DATA   rsearch / 1.0, 1.5, 2.0, 2.5, 3.0 / 
   Integer:: kphi = 11           !* # of characteristic azimuthal levels 
   Real(4):: philev(11)          !* characteristic azimuthal levels (deg) 
      DATA   philev / 9, 12, 15, 18, 24, 30, 36, 48, 60, 72, 90 / 
   Integer:: kmb  = 20           !* # of 50-mb pressure levels 
   Real(4):: pbasic(20)          !* 50-mb pressure levels (mb) 
      DATA   pbasic / 1000, 950, 900, 850, 800, 750, 700, 650, 600, 550, &
                       500, 450, 400, 350, 300, 250, 200, 150, 100,  55  / 
   Integer:: kmp  = 39           !* # of 25-mb pressure levels 
   Real(4):: plevel(39)          !* 25-mb pressure levels (mb) 
      DATA   plevel / 1000, 975, 950, 925, 900, 875, 850, 825, &
                       800, 775, 750, 725, 700, 675, 650, 625, &
                       600, 575, 550, 525, 500, 475, 450, 425, &
                       400, 375, 350, 325, 300, 275, 250, 225, &
                       200, 175, 150, 125, 100,  75,  55  / 
   Integer:: kmz  = 39           !* # of 25-mb-equiv height levels 
   Real(4):: zlevel(39)          !* 25-mb-equiv height levels (m) 
      DATA   zlevel /  35,   100,   200,   400,   600,   800,  1000,  1200, &
                     1500,  1800,  2100,  2400,  2700,  3000,  3400,  3800, &
                     4200,  4600,  5000,  5500,  6000,  6500,  7000,  7500, &
                     8000,  8500,  9000,  9750, 10500, 11250, 12000, 13000, &
                    14000, 15000, 16000, 17000, 18000, 19000, 20000   / 
   Real(4):: undef   = 1.E+25    !* undefined value for GrADS
   Real(4):: rd_air  = 287.053   !* dry-air gas  constant (J/kg*K) 
   Real(4):: cp_air  = 1005.65   !* dry-air heat capacity (J/kg*K)
   Real(4):: kd_air  = 0.28544   !* dry-air constant (Rd/Cp: N.D.)
   Real(4):: lapsr   = 0.0065    !* standard air temp lapse rate (K/m) 
   Real(4):: kk_0c   = 273.15    !* Kelvin temperature (K) at 0'C 
   Real(4):: ev_0c   = 6.1121    !* water vapor pressure (mb) at 0'C 
   Real(4):: Lv_0c   = 2.5E+6    !* w.vapor latent heat (J/kg) at 0'C 
   Real(4):: viscos  = 1.4E-5    !* air kinematic viscosity (m^2/s) 
   Real(4):: karman  = 0.4000    !* PBL von Karman constant (N.D.) 
   Real(4):: rho_h2o = 1000.0    !* water density (kg/m^3) at 0'C 
   Real(4):: r_earth = 6371220.  !* average earth radius   (m) 
   Real(4):: g_earth = 9.80616   !* average earth gravity  (m/s^2) 
   Real(4):: o_earth = 7.292E-5  !* average earth rotation (rad/s) 
   Real(4):: mps2kt  = 1.9438445 !* factor to convert m/s to knot 
   Real(4):: kt2mph  = 1.1517413 !* convert knot to mile/hour 
   Real(8):: pi_cnst = 3.141592653589793238462643383279502884 
   Real(8):: deg2rad = 3.141592653589793238462643383279502884/180. 
   Logical:: defined, fix_t0_u10, calcu_mslp, accum_precip 
   Logical:: derive_vars, press_vars,  basic_vars,  height_vars 
   Logical:: sigma_level, press_level, basic_level, height_level 
   Logical:: wind_swath, height_cylin, tilt_cylin,  refine_rot, refine_std 
   Logical:: debug_data, gfdl_surface, diagnostics, bad_cylin, valid_track 
   Integer:: msigma, mpress, mheight, mswath, mbasic, mbasic_cen, mrefine 
   Integer:: mdiagno, mzcylin, mtilt, mdebug, mgfdlsf, domain_num, ibad 
   Integer:: dh0, Status, Status_next_time, Status_next_var, start_date 
   Integer:: WrfType, start_index(4), end_index(4), ndim, icross, ierror 
   Integer:: ifcst_hour, ifcst_min, lat10, lon10, mfix_u10, mslpcal, mprecip 
   Integer:: mvars, imax, jmax, kmax, ide, jde, kde, im, jm, km, imin, jmin 
   Integer:: nvars, imx, jmx, ims, jms, imr, jmr, im3, jm3, imc, jmc, ntime
   Integer:: ibd, jbd, ibd1, jbd1, ibd9, jbd9, iold, jold, itmp, jtmp, koff 
   Integer:: imid, jmid, iloc, jloc, ioff, joff, itercen, iter_qt, mbins 
   Integer:: i34ne, i34se, i34sw, i34nw, i50ne, i50se, i50sw, i50nw, jvt 
   Integer:: i64ne, i64se, i64sw, i64nw, ipoci, iroci, ipmin, ivmax, irmw 
   Integer:: i, j, k, m, n,  ii, jj, i0, i1, i2, j0, j1, j2, k0, kk, iguess 
   Integer:: m0, m1, m2, m3, m4, n0, n1, n2, n3, n4, kout, k2d, k2e, jguess 
   Real(4):: ETIME, atime(2), btime(2), cpu_time_a, cpu_time_b, user_time 
   Real(4):: ctr_lat, ctr_lon, dlat, dlon, dtime, prtime, drlen, yrlen, xxlen 
   Real(4):: d02cut, d01cut, aspect, basic_dlon, r_cylin, dr_cylin, pguess 
   Real(4):: adv_time, dphi_deg, vtmax, rmvt, rcos, rsin, uur, vvr, yguess 
   Real(4):: fcst_time, stormlat, stormlon, vmax, pmax, pmin, pcut, xguess 
   Real(4):: olamda, otheta, phi, dphi, alfa, beta, gama, pi3_2, sqrt2_ 
   Real(4):: percent, binsize, vrange, xrs11, yrs11, xrs31,  y02, y10 
   Real(4):: xrr1, xrr2, xrr8, xrr9, yrr1, yrr2, yrr8, yrr9, dxo, dyo 
   Real(4):: xxr1, xxr_, yyr1, yyr_, xxs1, xxs_, yys1, yys_, dxx, dyy 
   Real(4):: eps, ykappam, ppp, qqq, ttt, tt0, ttx, tts, tts_error 
   Real(4):: ff(4,4), wtx(4), wty(4), aa1, aa2, bb1, bb2, ees, ee0 
   Real(4):: aaa, bbb, ccc, ddd, eee, xxx, yyy, zzz, www, rrr, rho 
   Real(4):: alon, alat, ptmp, stormrad, previous_time, rmw 
   
!* 2D variables on rotated A-grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Real(4), Allocatable::  zs(:,:),  sm(:,:),  p0(:,:), prc(:,:) 
   Real(4), Allocatable::  ps(:,:), ths(:,:),  ts(:,:),  qs(:,:) 
   Real(4), Allocatable:: u10(:,:), v10(:,:), t10(:,:), q10(:,:) 
   Real(4), Allocatable:: sst(:,:), ssq(:,:), t02(:,:), q02(:,:) 
   Real(4), Allocatable::  tw(:,:),  qw(:,:), cwh(:,:), qqh(:,:) 
   Real(4), Allocatable::  uf(:,:), z0m(:,:), z0t(:,:) 
   
!* 3D variables on sigma-p levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Real(4), Allocatable:: uu(:,:,:), vv(:,:,:), ww(:,:,:) 
   Real(4), Allocatable:: tt(:,:,:), td(:,:,:), qt(:,:,:) 
   Real(4), Allocatable:: pp(:,:,:), qq(:,:,:), cw(:,:,:) 
   Real(4), Allocatable:: gh(:,:,:), rh(:,:,:), dp(:,:,:) 
   
!* Coordinates and general workspace 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Real(4), Allocatable::   wk(:,:),   wk1(:,:),  wk2(:,:),  wk3(:,:)  
   Real(4), Allocatable::  xrr(:,:),   yrr(:,:),  xrs(:,:),  yrs(:,:) 
   Real(4), Allocatable::  xsr(:,:),   ysr(:,:),  xss(:,:),  yss(:,:) 
   Real(4), Allocatable::  xxr(:,:),   yyr(:,:),  xxs(:,:),  yys(:,:) 
   Real(4), Allocatable:: wcos(:,:),  wsin(:,:),  xrlen(:),  wnd(:,:)  
   Real(4), Allocatable:: u10x(:,:),  v10x(:,:),  p0x(:,:),  prx(:,:) 
   Real(4), Allocatable:: uus(:,:,:), vvs(:,:,:), urs(:,:),  vrs(:,:) 
   Real(4), Allocatable:: dat(:,:,:), wks(:,:,:), p02(:,:),  td2(:,:) 
   Integer, Allocatable::idat(:,:,:) 
   
!* 3D workspace for pressure and height levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Real(4), Allocatable:: uu_(:,:,:), vv_(:,:,:), ww_(:,:,:) 
   Real(4), Allocatable:: tt_(:,:,:), td_(:,:,:), qt_(:,:,:) 
   Real(4), Allocatable:: pp_(:,:,:), qq_(:,:,:), cw_(:,:,:) 
   Real(4), Allocatable:: gh_(:,:,:), rh_(:,:,:) 
   Real(4), Allocatable:: ux_(:,:,:), vx_(:,:,:) 
   
!* 3D workspace for triple grid refinement 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Real(4), Allocatable:: uu3(:,:,:), vv3(:,:,:), ww3(:,:,:) 
   Real(4), Allocatable:: tt3(:,:,:), td3(:,:,:), qt3(:,:,:) 
   Real(4), Allocatable:: pp3(:,:,:), qq3(:,:,:), cw3(:,:,:) 
   Real(4), Allocatable:: gh3(:,:,:), rh3(:,:,:) 
   Real(4), Allocatable:: ux3(:,:,:), vx3(:,:,:) 
   
   Real(4), Allocatable:: uu3_(:,:,:), vv3_(:,:,:), ww3_(:,:,:) 
   Real(4), Allocatable:: tt3_(:,:,:), td3_(:,:,:), qt3_(:,:,:) 
   Real(4), Allocatable:: pp3_(:,:,:), qq3_(:,:,:), cw3_(:,:,:) 
   Real(4), Allocatable:: gh3_(:,:,:), rh3_(:,:,:) 
   Real(4), Allocatable:: ux3_(:,:,:), vx3_(:,:,:) 
   
!* Workspace for variables on cylindrical grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Integer, Allocatable::  icen(:),    jcen(:) 
   Real(4), Allocatable:: ppc(:,:,:), qqc(:,:,:), cwc(:,:,:), rhc(:,:,:) 
   Real(4), Allocatable:: ttc(:,:,:), tdc(:,:,:), qtc(:,:,:) 
   Real(4), Allocatable:: uuc(:,:,:), vvc(:,:,:), wwc(:,:,:) 
   Real(4), Allocatable:: uxc(:,:,:), vxc(:,:,:), mwc(:,:,:) 
   
!* Diagnostic or statistical quantities 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Real(4), Allocatable:: wwfr(:,:) 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   cpu_time_a = ETIME(atime) 
   CALL set_wrf_debug_level( 2 )
   CALL ext_ncd_ioinit(SysInfo,Status)
   itercen = 0 ; iter_qt = 0 ; bad_cylin = .false. 

!******************************************! 
!  0a. I/O preparation for data processing ! 
!__________________________________________! 
   
!* Get arguments from script
!* ~~~~~~~~~~~~~~~~~~~~~~~~~ 
   CALL getarg( 1,filename)
   CALL getarg( 2,argument) ; READ(argument,*) msigma
   CALL getarg( 3,argument) ; READ(argument,*) mpress
   CALL getarg( 4,argument) ; READ(argument,*) mheight
   CALL getarg( 5,argument) ; READ(argument,*) mswath
   CALL getarg( 6,argument) ; READ(argument,*) mbasic
   CALL getarg( 7,argument) ; READ(argument,*) mzcylin
   CALL getarg( 8,argument) ; READ(argument,*) mtilt
   CALL getarg( 9,argument) ; READ(argument,*) mdiagno
   CALL getarg(10,argument) ; READ(argument,*) mdebug
   CALL getarg(11,argument) ; READ(argument,*) mrefine
   CALL getarg(12,argument) ; READ(argument,*) mgfdlsf
   CALL getarg(13,argument) ; READ(argument,*) domain_num
   CALL getarg(14,argument) ; READ(argument,*) storm_bas
   CALL getarg(15,argument) ; READ(argument,*) storm_num
   CALL getarg(16,argument) ; READ(argument,*) start_date
   CALL getarg(17,argument) ; READ(argument,*) d02cut
   CALL getarg(18,argument) ; READ(argument,*) d01cut
   CALL getarg(19,argument) ; READ(argument,*) aspect
   CALL getarg(20,argument) ; READ(argument,*) basic_dlon 
   CALL getarg(21,argument) ; READ(argument,*) mbasic_cen
   CALL getarg(22,argument) ; READ(argument,*) r_cylin
   CALL getarg(23,argument) ; READ(argument,*) dr_cylin
   CALL getarg(24,argument) ; READ(argument,*) dphi_deg
   CALL getarg(25,argument) ; READ(argument,*) atcf_code
   CALL getarg(26,argument) ; READ(argument,*) mfix_u10
   CALL getarg(27,argument) ; READ(argument,*) mslpcal
   CALL getarg(28,argument) ; READ(argument,*) mprecip
   CALL getarg(29,argument) ; READ(argument,*) ntime
   
   wind_swath   = ( mswath  == 1 ) 
   basic_level  = ( mbasic  == 1 ) 
   sigma_level  = ( msigma  == 1 ) 
   press_level  = ( mpress  == 1 ) 
   height_level = ( mheight == 1 ) 
   height_cylin = ( mzcylin == 1 ) 
   tilt_cylin   = ( mtilt   == 1 ) 
   diagnostics  = ( mdiagno == 1 ) 
   debug_data   = ( mdebug  == 1 ) 
   calcu_mslp   = ( mslpcal == 1 ) 
   accum_precip = ( mprecip == 1 ) 
   gfdl_surface = ( mgfdlsf == 1 ) 
   press_vars   = ( press_level  ) 
   basic_vars   = ( basic_level  ) 
   height_vars  = ( height_level ) .OR. ( height_cylin ) 
   derive_vars  = ( sigma_level  ) .OR. ( diagnostics  ) .OR. ( calcu_mslp  ) & 
          .OR.    ( basic_vars   ) .OR. ( press_vars   ) .OR. ( height_vars ) 
   
   storm_basin = 'XX' 
   IF ( storm_bas == 'L' ) storm_basin = 'AL' 
   IF ( storm_bas == 'E' ) storm_basin = 'EP' 
   IF ( aspect    ==  0. ) aspect = 0.8 
   
!* Variables to be extracted 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~ 
   variable( 1) = 'U10'
   variable( 2) = 'V10'
   variable( 3) = 'MSLP'
   variable( 4) = 'ACPREC'
   variable( 5) = 'NPREC'
   variable( 6) = 'XTIME'
   variable( 7) = 'HLAT'
   variable( 8) = 'HLON'
   variable( 9) = 'Z0'
   variable(10) = 'SM'
   variable(11) = 'FIS'
   variable(12) = 'SST'
   variable(13) = 'QS'
   variable(14) = 'THS'
   variable(15) = 'USTAR'
   variable(16) = 'TWBS'
   variable(17) = 'QWBS'
   variable(18) = 'U'
   variable(19) = 'V'
   variable(20) = 'W'
   variable(21) = 'T'
   variable(22) = 'Q'
   variable(23) = 'PINT'
   variable(24) = 'CWM'

   mvars = 10 ; IF (derive_vars) mvars = 24

!* Open the data files to write to 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   OPEN(51,file='fcst_track.msg',status='unknown')
   OPEN(52,file='fcst_track.txt',position='append')
   OPEN(53,file='atcf_track.msg',status='unknown')
   OPEN(54,file='atcf_track.txt',position='append')

   if ( wind_swath ) then 
   OPEN(61,file='swath_std.ctl',form='formatted')
   OPEN(62,file='swath_std.dat',form='unformatted',position='append')
   endif 

   if ( sigma_level ) then 
   OPEN(100,file='sigma_lev.dat',form='unformatted',position='append')
   OPEN(101,file='sigma_rot.ctl',form='formatted')
   OPEN(102,file='sigma_std.ctl',form='formatted')
   endif 

   if ( press_level ) then 
   OPEN(200,file='press_lev.dat',form='unformatted',position='append')
   OPEN(201,file='press_rot.ctl',form='formatted')
   OPEN(202,file='press_std.ctl',form='formatted')
   endif 

   if ( basic_level ) then 
   OPEN(251,file='basic_std.ctl',form='formatted')
   OPEN(252,file='basic_std.dat',form='unformatted',position='append')
   OPEN(253,file='basic_std.msg',status='unknown')
   OPEN(254,file='basic_std.txt',position='append')
   OPEN(261,file='bas2d_std.ctl',form='formatted')
   OPEN(262,file='bas2d_std.dat',form='unformatted',position='append')
   endif 
   
   if ( height_level ) then 
   OPEN(300,file='height_lev.dat',form='unformatted',position='append')
   OPEN(301,file='height_rot.ctl',form='formatted')
   OPEN(302,file='height_std.ctl',form='formatted')
   endif 

   if ( height_cylin ) then 
   OPEN(310,file='height_cyl.dat',form='unformatted',position='append')
   OPEN(311,file='height_cyl.ctl',form='formatted')
   OPEN(312,file='height_cpr.ctl',form='formatted')
   endif 

   if ( diagnostics ) then 
   OPEN(331,file='cfads_demo.ctl',form='formatted')
   OPEN(332,file='cfads_demo.dat',form='unformatted',position='append')
   endif 

!* Open the data file to read from 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   CALL ext_ncd_open_for_read( trim(filename), 0, 0, "", dh0, Status )

   PRINT* 
   PRINT*, 'Reading ', trim(filename) 
   IF ( Status /= 0 ) THEN
      PRINT*, 'Error opening', filename, ' Status = ', Status
      STOP
   ENDIF
   PRINT* 
   
!* Dimensions & resolutions of data grids 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   CALL ext_ncd_get_dom_ti_integer( dh0,'WEST-EAST_GRID_DIMENSION',   &
                                         imax, 1, icross, ierror      )
   CALL ext_ncd_get_dom_ti_integer( dh0,'SOUTH-NORTH_GRID_DIMENSION', &
                                         jmax, 1, icross, ierror      )
   CALL ext_ncd_get_dom_ti_integer( dh0,'BOTTOM-TOP_GRID_DIMENSION',  &
                                         kmax, 1, icross, ierror      )
   CALL ext_ncd_get_dom_ti_real( dh0, 'CEN_LAT', ctr_lat, 1, icross, ierror )
   CALL ext_ncd_get_dom_ti_real( dh0, 'CEN_LON', ctr_lon, 1, icross, ierror )
   CALL ext_ncd_get_dom_ti_real( dh0, 'DX', dlon, 1, icross, ierror )
   CALL ext_ncd_get_dom_ti_real( dh0, 'DY', dlat, 1, icross, ierror ) 
   CALL ext_ncd_get_dom_ti_real( dh0, 'DT',dtime, 1, icross, ierror ) 
   
   refine_std = ( dlon > low_res ) 
   refine_rot = ( dlon > low_res ) .AND. ( mrefine == 1 ) 
   ide = imax - 1 ; im = 2*ide - 1 !* X dimensions of E-grid & A-grid 
   jde = jmax - 1 ; jm =   jde     !* Y dimensions of E-grid & A-grid 
   kde = kmax - 1 ; km =   kde     !* Number of sigma levels (layers) 
   
!* Allocate memory for 2D & sigma-level variables 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate(  zs(im,jm),  sm(im,jm),  p0(im,jm), prc(im,jm) ) 
   Allocate(  ps(im,jm), ths(im,jm),  ts(im,jm),  qs(im,jm) ) 
   Allocate( u10(im,jm), v10(im,jm), t10(im,jm), q10(im,jm) ) 
   Allocate( sst(im,jm), ssq(im,jm), t02(im,jm), q02(im,jm) ) 
   Allocate(  tw(im,jm),  qw(im,jm), cwh(im,jm), qqh(im,jm) ) 
   Allocate(  uf(im,jm), z0m(im,jm), z0t(im,jm)  ) 
   Allocate( uu(im,jm,km), vv(im,jm,km), ww(im,jm,km) ) 
   Allocate( tt(im,jm,km), td(im,jm,km), qt(im,jm,km) ) 
   Allocate( pp(im,jm,km), qq(im,jm,km), cw(im,jm,km) ) 
   Allocate( gh(im,jm,km), rh(im,jm,km) ) 
   
!           *     *     *     *     *     *     *     *     *     * 

   CALL ext_ncd_get_next_time( dh0, DateStr, Status_next_time )
   
   DO WHILE ( Status_next_time .EQ. 0 ) !==================================> 

!****************************************************! 
!  0b. Read original E-grid data & convert to A-grid !  
!____________________________________________________! 
   
   Allocate( u10x(ide,jde), v10x(ide,jde), dp(im,jm,km) ) 

   CALL ext_ncd_get_next_var( dh0, VarName, Status_next_var )

   DO WHILE ( Status_next_var .EQ. 0 ) !-----------------------------------> 

!*    Get the variable information 
!*    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
      start_index= 1 ; end_index= 1
      call ext_ncd_get_var_info( dh0, VarName, ndim, ordering, staggering, &
                                 start_index, end_index, WrfType, ierror   )

      defined = (WrfType == WRF_REAL).OR.(WrfType == WRF_DOUBLE) 
      IF ( .not. defined .AND. WrfType /= WRF_INTEGER )  THEN
         CALL ext_ncd_get_next_var( dh0, VarName, Status_next_var )
         CYCLE
      ENDIF

      do m = 1, mvars 
      IF ( VarName == variable(m) ) THEN  !* - - - - - - - - - - - - - - 

      IF ( ndim == 3 )  ord = 'XYZ'
      IF ( ndim == 2 )  ord = 'XY'
      IF ( ndim == 1 )  ord = 'Z'
      IF ( ndim == 0 )  ord = '0'

      Allocate( idat( end_index(1),end_index(2),end_index(3)   ) )
      Allocate(  dat( end_index(1),end_index(2),end_index(3)   ) )
      Allocate(  wks( end_index(1),end_index(2),end_index(3)-1 ) )

!*    Read original data on rotated E-grid 
!*    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
      IF ( WrfType == WRF_INTEGER ) THEN !* ------------------------ 
      CALL ext_ncd_read_field( dh0, DateStr, trim(VarName), idat, WrfType ,  &
                        0,0,0,ord,staggering,dimnames,start_index,end_index, & 
                        start_index,end_index,start_index,end_index, ierror  ) 
      ELSE !* ( WRF_REAL .OR. WRF_DOUBLE )  ------------------------ 
      CALL ext_ncd_read_field( dh0, DateStr, trim(VarName),  dat, WrfType ,  &
                        0,0,0,ord,staggering,dimnames,start_index,end_index, & 
                        start_index,end_index,start_index,end_index, ierror  ) 
      ENDIF !* ----------------------------------------------------- 

      if ( ierror /= 0 ) then
         PRINT*, 'Error in reading data:'
         PRINT*, 'ndim, end_index(1:3) =', ndim, end_index(1:3) 
         STOP 
      endif

      WRITE(*,'(1x,A9,1x,I1,2(1x,I3),1x,I2,1x,A3,1x,A)')  VarName, ndim, &
      end_index(1), end_index(2), end_index(3), trim(ordering), trim(DateStr)

!*    Convert E-grid data to enhanced A-grid 
!*    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
      IF ( trim(VarName)== 'XTIME') fcst_time = dat(1,1,1) ! (min) 
      IF ( trim(VarName)== 'NPREC') prtime = idat(1,1,1)*dtime/60. 
      IF ( trim(VarName)== 'HLAT' ) yrs11  =  dat(1,1,1) 
      IF ( trim(VarName)== 'HLON' ) THEN
           xrs11 = dat(1,1,1) ; xrs31 = dat(2,1,1) 
      ENDIF 
   
      IF ( trim(VarName)== 'SM' ) & 
      CALL E2A_Grid_Scalar( sm(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'FIS' ) THEN 
      CALL E2A_Grid_Scalar( zs(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
      zs(:,:) = zs(:,:)/g_earth  !* (m) 
      ENDIF 
   
      IF ( trim(VarName)== 'MSLP' ) THEN 
      CALL E2A_Grid_Scalar( p0(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
      p0(:,:) = p0(:,:)*.01      !* (mb) 
      ENDIF 
   
      IF ( trim(VarName)== 'ACPREC' ) THEN 
      CALL E2A_Grid_Scalar( prc(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
      ENDIF 
   
      IF ( trim(VarName)== 'QS' ) & 
      CALL E2A_Grid_Scalar( qs(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'THS' ) & 
      CALL E2A_Grid_Scalar( ths(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'SST' ) & 
      CALL E2A_Grid_Scalar( sst(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'Z0' ) & 
      CALL E2A_Grid_Scalar( z0t(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'USTAR' ) & 
      CALL E2A_Grid_Scalar( uf(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'TWBS' ) & 
      CALL E2A_Grid_Scalar( tw(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'QWBS' ) & 
      CALL E2A_Grid_Scalar( qw(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
   
      IF ( trim(VarName)== 'U10' ) u10x(:,:) = dat(:,:,1) 
      IF ( trim(VarName)== 'V10' ) v10x(:,:) = dat(:,:,1) 
   
      IF ( trim(VarName)== 'U' ) & 
      CALL E2A_Grid_Vector( uu(1,1,1), im, jm, km, dat(1,1,1), ide, jde, kde ) 
   
      IF ( trim(VarName)== 'V' ) & 
      CALL E2A_Grid_Vector( vv(1,1,1), im, jm, km, dat(1,1,1), ide, jde, kde ) 
   
      IF ( trim(VarName)== 'W' ) THEN 
      dat(:,:,kde+1) = dat(:,:,kde) 
      DO k = 1, kde ; wks(:,:,k) = .5*(dat(:,:,k)+dat(:,:,k+1)) ; ENDDO 
      CALL E2A_Grid_Scalar( ww(1,1,1), im, jm, km, wks(1,1,1), ide, jde, kde ) 
      ENDIF 
   
      IF ( trim(VarName)== 'T' ) & 
      CALL E2A_Grid_Scalar( tt(1,1,1), im, jm, km, dat(1,1,1), ide, jde, kde ) 
   
      IF ( trim(VarName)== 'Q' ) & 
      CALL E2A_Grid_Scalar( qq(1,1,1), im, jm, km, dat(1,1,1), ide, jde, kde ) 
   
      IF ( trim(VarName)== 'PINT' ) THEN !* ( Pa -> mb ) 
      DO k = 1, kde ; wks(:,:,k) = .005*(dat(:,:,k)+dat(:,:,k+1)) ; ENDDO 
      CALL E2A_Grid_Scalar( pp(1,1,1), im, jm, km, wks(1,1,1), ide, jde, kde ) 
      DO k = 1, kde ; wks(:,:,k) =  .01*(dat(:,:,k)-dat(:,:,k+1)) ; ENDDO 
      CALL E2A_Grid_Scalar( dp(1,1,1), im, jm, km, wks(1,1,1), ide, jde, kde ) 
      CALL E2A_Grid_Scalar( ps(1,1), im, jm, 1, dat(1,1,1), ide, jde, 1 ) 
      ps(:,:) = ps(:,:)*.01 
      ENDIF 
   
      IF ( trim(VarName)== 'CWM' ) & 
      CALL E2A_Grid_Scalar( cw(1,1,1), im, jm, km, dat(1,1,1), ide, jde, kde ) 
   
      DeAllocate( idat, dat, wks ) 

      ENDIF !* for VarName - - - - - - - - - - - - - - - - - - - - - - - 
      enddo

      CALL ext_ncd_get_next_var( dh0, VarName, Status_next_var ) 

   ENDDO !<----------------------------------------------------------------- 

!* Accumulated precipitation or avg precip rate 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF ( accum_precip ) THEN  !* - - - - - - - - 
      prc(:,:) = prc(:,:)*1000.      !* (mm) 
   ELSE  !* average precip rate - - - - - - - - 
       m  = Mod( nint(fcst_time), nint(prtime) ) 
      ttt = m/60. ; IF ( m == 0 ) ttt = prtime/60. 
      prc(:,:) = prc(:,:)*1000./ttt  !* (mm/hr) 
   ENDIF !* - - - - - - - - - - - - - - - - - - 

!* Special E-to-A-grid conversion for 10m winds 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF    ( fcst_time == 0 ) THEN 
      CALL E2A_Grid_Vector( u10(1,1), im, jm, 1, u10x(1,1), ide, jde, 1 ) 
      CALL E2A_Grid_Vector( v10(1,1), im, jm, 1, v10x(1,1), ide, jde, 1 ) 
   ELSE !( fcst_time /= 0 )! 
      CALL E2A_Debug_Vector(u10(1,1), im, jm, 1, u10x(1,1), ide, jde, 1 ) 
      CALL E2A_Debug_Vector(v10(1,1), im, jm, 1, v10x(1,1), ide, jde, 1 ) 
   ENDIF 
   
   DeAllocate( u10x, v10x ) 
   
!***************************************************! 
!  0c. Coordinates & geometry of the rotated A-grid ! 
!___________________________________________________! 

!* Rotated grid's rotated & standard coordinates (degree) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate(  xrr(im,jm),  yrr(im,jm), xrs(im,jm), yrs(im,jm) ) 
   Allocate( wcos(im,jm), wsin(im,jm), urs(im,jm), vrs(im,jm), xrlen(jm) ) 
   
   IF ( xrs11 /= xrs31 ) THEN !* Earth latlon defined ------------------ 
      Call Convert_LatLon( 1, 1, 1, xrs11, yrs11, xxx, yyy, ctr_lon, ctr_lat ) 
      xrr(1,1) = nint(xxx/dlon)*dlon ; yrr(1,1) = nint(yyy/dlat)*dlat 
   ELSE  !* Earth latlon undefined, assume data on d01 ----------------- 
      xrr(1,1) = 0.5*(1-im)*dlon     ; yrr(1,1) = 0.5*(1-jm)*dlat 
   ENDIF !* ------------------------------------------------------------ 
   DO i = 1, im ; xrr(i,:) = xrr(1,1) + (i-1)*dlon ; ENDDO 
   DO j = 1, jm ; yrr(:,j) = yrr(1,1) + (j-1)*dlat ; ENDDO 

   Call Convert_LatLon( 2, im, jm, xrs, yrs, xrr, yrr, ctr_lon, ctr_lat ) 

!* Rotated grid's interval lengths (m) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   yrlen = dlat*deg2rad*r_earth ; xxx = dlon*deg2rad*r_earth 
   DO j = 1, jm ; xrlen(j) = xxx*cos( yrr(1,j)*deg2rad ) ; ENDDO 
   
!* Elements for converting vectors btw rotated & std latlon grids 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
!  uur =   wcos*uus + wsin*vvs  !  uus = wcos*uur - wsin*vvr 
!  vvr = - wsin*uus + wcos*vvs  !  vvs = wsin*uur + wcos*vvr 
   
   olamda = ctr_lon*deg2rad 
   otheta = ctr_lat*deg2rad 
   aaa = cos(otheta) ; bbb = sin(otheta) 
   DO j = 1, jm ; ccc = 1./cos(yrr(1,j)*deg2rad)
   DO i = 1, im         !* poles not allowed ... 
      xxx = xrs(i,j)*deg2rad - olamda ; yyy = yrs(i,j)*deg2rad 
      wcos(i,j)= ( aaa*cos(yyy) + bbb*sin(yyy)*cos(xxx) )*ccc 
      wsin(i,j)= - bbb*sin(xxx)*ccc 
   ENDDO 
   ENDDO 

!           *     *     *     *     *     *     *     *     *     * 
   
!* Fix initial 10m wind for quick tracker & wind swath 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF ( mfix_u10 == 1 .AND. fcst_time == 0. ) THEN !* - - - - - - - 
   
   aaa = .0185/g_earth !* GFDL ocean momentum roughness coeff 
   ddd = 60.0/1.943844 !* GFDL 10m wind correction threshold (m/s) 
   
   DO j = 1, jm ; DO i = 1, im 
      www = sqrt( u10(i,j)**2 + v10(i,j)**2 ) ; www = max(www,1.) 
      uur = u10(i,j)/www ; vvr = v10(i,j)/www !* given 33m wind 
      IF ( sm(i,j) == 0. ) THEN 
            zzz = z0t(i,j) 
      ELSE !* water surface 
         IF ( www < 12.5 ) THEN !* low wind 
            zzz = .001*aaa*((.024*www + .782)*www)**2 
         ELSE  !* high-wind empirical formula 
            zzz = .001*( .074*www - .58 )
         ENDIF 
      ENDIF 
      www = www*log( 10./zzz )/log( 33./zzz )  !* neutral 10m wind 
      IF ( gfdl_surface .AND. www > ddd ) THEN 
         www = www + .12*(www-ddd)             !* GFDL correction 
      ENDIF 
      u10(i,j) = www*uur ; v10(i,j) = www*vvr 
   ENDDO ; ENDDO 
   
   ENDIF !* ( mfix_u10 == 1 .AND. fcst_time == 0. ) - - - - - - - - 
   
!****************************************************! 
!  0d. Derive diagnostic variables on sigma-p levels ! 
!____________________________________________________! 

   IF ( derive_vars ) THEN  !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
!* Surface air temperature (K) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO j = 1, jm ; DO i = 1, im 
      ts(i,j) = ths(i,j)*( .001*ps(i,j) )**kd_air 
   ENDDO ; ENDDO 
   
!* Cut off SST & SSQ over land 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ssq(:,:) = qs(:,:) 
   DO j = 1, jm ; DO i = 1, im 
      IF ( sm(i,j) < 1. ) THEN 
         sst(i,j) = undef ; ssq(i,j) = undef 
      ENDIF 
   ENDDO ; ENDDO 

!* Fix signs of surface enthalpy fluxes 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   tw(:,:) = - tw(:,:) ; qw(:,:) = - qw(:,:) 

!* Column integrals for gh (m), qqh (mm), cwh (mm)  
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
!  rho_h2o = 1000.0 => rho*qq*dz = qq*dp/g [mm] 
   
   ccc = .5*rd_air/g_earth ; ddd = 100./g_earth 
   DO j = 1, jm ; DO i = 1, im 
         qqh(i,j) = ddd*qq(i,j,1)*dp(i,j,1) 
         cwh(i,j) = ddd*cw(i,j,1)*dp(i,j,1) 
         aaa = ccc*(1+.608*qq(i,j,1))*tt(i,j,1)*dp(i,j,1)/pp(i,j,1) 
         gh(i,j,1) = zs(i,j) + aaa 
      DO k = 2, km 
         qqh(i,j)  = qqh(i,j) + ddd*qq(i,j,k)*dp(i,j,k)
         cwh(i,j)  = cwh(i,j) + ddd*cw(i,j,k)*dp(i,j,k)
         bbb = ccc*(1+.608*qq(i,j,k))*tt(i,j,k)*dp(i,j,k)/pp(i,j,k) 
         gh(i,j,k) = gh(i,j,k-1) + aaa + bbb ; aaa = bbb 
      ENDDO 
   ENDDO ; ENDDO 
   
!* Recalculate mean-sea-level pressure (mb) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF ( calcu_mslp ) THEN !* ------------------------------------- 
   
   Allocate( wk(im,jm) ) 
   
   DO j = 1, jm ; DO i = 1, im 
        ttt = (1+.608*qq(i,j,1))*tt(i,j,1) + .5*lapsr*gh(i,j,1) 
      p0(i,j) = pp(i,j,1)*exp( gh(i,j,1)*g_earth/(rd_air*ttt) ) 
   ENDDO ; ENDDO 
   
   wk(:,:) = 0. 
   aaa = .25*overlax 
   DO n = 1, iterlax 
      do j = 2,jm-1 ; do i = 2,im-1
      IF ( sm(i,j) < 1. ) THEN 
         wk(i,j) = p0(i-1,j) + p0(i+1,j) + p0(i,j-1) + p0(i,j+1) - 4.*p0(i,j)
      ENDIF ! residual 
      enddo ; enddo 
!     - - - - - - - 
      do j = 2,jm-1 ; do i = 2,im-1
      IF ( sm(i,j) < 1. ) THEN 
         p0(i,j) = p0(i,j) + aaa*wk(i,j) 
      ENDIF ! smoothing 
      enddo ; enddo 
   ENDDO 
   
   DeAllocate( wk ) 
   
   ENDIF !* ------------------------------------------------------ 
   
!* Dewpoint temperature (K) on sigma-p levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   CALL Dewpoint_RH( pp, qq, tt, td, rh, im, jm, km ) 
   
!* Equivalent potential temperature (K) on sigma-p levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   tt0 = 273.15   !* ice-point temperature (K) 
   ee0 = 6.1121   !* ice-point vapor pressure (mb) 
   aa1 = 17.502   !*  factor  for water vapor pressure 
   bb1 = 240.97   !* constant for water vapor pressure 
   aa2 = 22.452   !*  factor  for  ice  vapor pressure 
   bb2 = 272.55   !* constant for  ice  vapor pressure 
   eps = .62198   !* ratio of vapor & air molecular weights
   tts_error = .1 !* error constraint for "tts" computation 
   
   iter_qt = 0 
   DO k = 1, km ; DO j = 1, jm ; DO i = 1, im 
      ttt =     tt(i,j,k) 
      rrr =     qq(i,j,k)/( 1. - qq(i,j,k) ) 
      eee = rrr*pp(i,j,k)/( eps + rrr  ) 
      ykappam =  1./( kd_air*(1.-.245*rrr) ) 
      ees = eee 
      ttx = ttt 
      DO n = 0, iterpot 
         IF ( ees == ee0 ) THEN 
            tts = tt0 
         ELSEIF ( ees > ee0 ) THEN 
            xxx = log(ees/ee0) 
            tts = bb1*xxx/(aa1-xxx) + tt0 
         ELSE  !( ees < ee0 ) 
            xxx = log(ees/ee0) 
            tts = bb2*xxx/(aa2-xxx) + tt0 
         ENDIF 
         IF ( tts > ttx - tts_error ) THEN 
            EXIT 
         ELSE 
            ees = eee*exp( log(tts/ttt)*ykappam ) 
            ttx = tts 
         ENDIF 
      ENDDO 
      iter_qt = max( iter_qt, n ) 
      aaa = ttt*( (1000.+1607.8*rrr)/pp(i,j,k) )**kd_air 
      bbb = aaa*exp( (3132.9-2.37*tts)*rrr/tts ) 
      qt(i,j,k) = bbb*exp( exp(1.62*log(bbb)+14.3*log(tts)-96.) ) 
   ENDDO ; ENDDO ; ENDDO 
   
!  PRINT*, '------------------------------------------------------------' 
!  PRINT*, 'Max Iterations for Eqv.Pot.Temp. =', iter_qt 
!  PRINT*, '------------------------------------------------------------' 
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* ===================================== 
!* GFDL surface-layer scheme diagnostics 
!* ===================================== 

   IF ( gfdl_surface ) THEN !* - - - - - - - - - - - - - - - - - - - - - 
   
   aaa = .0185/g_earth !* GFDL ocean momentum roughness coeff 
   ddd = 60.0/1.943844 !* GFDL 10m wind correction threshold (m/s) 
   
!* Rederive GFDL ocean momentum roughness 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   z0m(:,:) = z0t(:,:) !* copy roughness for land surface 
   DO j = 1, jm ; DO i = 1, im  !* 1st sigma-level wind 
      www = sqrt( uu(i,j,1)**2 + vv(i,j,1)**2 ) ; www = max(www,1.) 
      IF ( sm(i,j) == 1. ) THEN !* water sfc 
         IF ( www < 12.5 ) THEN !* low  wind 
            z0m(i,j) = .001*aaa*((.024*www + .782)*www)**2 
         ELSE  !* high-wind empirical formula 
            z0m(i,j) = .001*( .074*www - .58 )
         ENDIF 
      ENDIF 
   ENDDO ; ENDDO 
   
!* Basic GFDL sfc-layer variables for initial time 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   if ( fcst_time == 0. ) then 
   DO j = 1, jm ; DO i = 1, im 
      yyy = log( (gh(i,j,1)-zs(i,j))/z0m(i,j) ) !* assume neutral 
      www = sqrt( uu(i,j,1)**2 + vv(i,j,1)**2 ) ; www = max(www,1.) 
      uur = uu(i,j,1)/www ; vvr = vv(i,j,1)/www 
      IF ( sm(i,j) == 1. ) THEN 
         uf(i,j) = karman*www/yyy 
        z0t(i,j) = aaa*uf(i,j)**2  !* Charnock's z0m is used as z0t in HWRF 
!       z0t(i,j) = z0m(i,j)/exp(2.67*sqrt(sqrt(uf(i,j)*z0m(i,j)/viscos))-2.57) 
      ELSE  !^ Zeng'98 thermal roughness (U10 < 18 m/s) is not used in HWRF 
         uf(i,j) = karman*www/yyy
      ENDIF 
      rho = 100.*ps(i,j)/( rd_air*(1+.608*qs(i,j))*ts(i,j) ) 
      ttt = tt(i,j,1)*(1000./pp(i,j,1))**kd_air ! potential temp 
      tw(i,j) = rho*cp_air*uf(i,j)*karman*( ths(i,j) - ttt )/yyy 
      qw(i,j) = rho*Lv_0c *uf(i,j)*karman*( qs(i,j) - qq(i,j,1) )/yyy 
!     - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      www = uf(i,j)/karman*log( 10./z0m(i,j) ) !* neutral 10m wind 
      IF (www > ddd) www = www + .12*(www-ddd) !* GFDL correction 
      u10(i,j) = www*uur ; v10(i,j) = www*vvr 
   ENDDO ; ENDDO 
   endif 
   
!* GFDL thermodynamic variables for air-sea contrast  
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO j = 1, jm ; DO i = 1, im 
      www =  max( uf(i,j), .01 ) !* constrained ustar 
      rho =  100.*ps(i,j)/( rd_air*(1.+.608*qs(i,j))*ts(i,j) ) 
      rrr = -karman*g_earth*tw(i,j)/(rho*cp_air*ths(i,j)*www**3) !* 1/L 
      ttt = -tw(i,j)/(rho*cp_air*www) !* theta-star 
      qqq = -qw(i,j)/(rho*Lv_0c *www) !* q-star 
!     - - - - - - - 
      zzz = 10.*rrr 
      IF ( zzz < 0. ) THEN 
         yyy = 2.*log( .5*(1.+sqrt(1.-16.*zzz)) ) 
      ELSEIF ( zzz <= .5 ) THEN 
         yyy = - 5.*zzz 
      ELSEIF ( zzz <= 10 ) THEN 
         yyy = - 7.*log(zzz) - 4.25/zzz + .5/zzz**2 - .852  
      ELSE ! ( zzz  > 10 ) 
         yyy = log(zzz) - .7585*zzz - 12.11 
      ENDIF !* GFDL similarity function 
      y10 = ( log(10./z0t(i,j)) - yyy )/karman 
      t10(i,j) = ths(i,j) + ttt*y10 !* 10m potential temperature 
      q10(i,j) =  qs(i,j) + qqq*y10 !* 10m specific  humidity 
!     - - - - - - - 
      zzz = 2.*rrr 
      IF ( zzz < 0. ) THEN 
         yyy = 2.*log( .5*(1.+sqrt(1.-16.*zzz)) ) 
      ELSEIF ( zzz <= .5 ) THEN 
         yyy = - 5.*zzz 
      ELSEIF ( zzz <= 10 ) THEN 
         yyy = - 7.*log(zzz) - 4.25/zzz + .5/zzz**2 - .852  
      ELSE ! ( zzz  > 10 ) 
         yyy = log(zzz) - .7585*zzz - 12.11 
      ENDIF !* GFDL similarity function 
      y02 = ( log(2./z0t(i,j)) - yyy )/karman 
      t02(i,j) = ths(i,j) + ttt*y02 !* 2m potential temperature 
      q02(i,j) =  qs(i,j) + qqq*y02 !* 2m specific  humidity 
!     - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      zzz = gh(i,j,1) - zs(i,j) ; xxx = 2./zzz ; yyy = 10./zzz  !* convert temp 
      t02(i,j) = t02(i,j)*(.001*( (1.-xxx)*ps(i,j) + xxx*pp(i,j,1) ))**kd_air 
      t10(i,j) = t10(i,j)*(.001*( (1.-yyy)*ps(i,j) + yyy*pp(i,j,1) ))**kd_air 
   ENDDO ; ENDDO 
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* Delete bad boundary of surface variables 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   if ( .not. debug_data ) then !* --------------------------------- 

   DO j =  1, 1+jbdcut ;  uf(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ;  uf(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ;  uf(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ;  uf(i,:) = undef ; ENDDO 

   DO j =  1, 1+jbdcut ; z0m(:,j) = undef ; z0t(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ; z0m(:,j) = undef ; z0t(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ; z0m(i,:) = undef ; z0t(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ; z0m(i,:) = undef ; z0t(i,:) = undef ; ENDDO 

   DO j =  1, 1+jbdcut ;  tw(:,j) = undef ;  qw(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ;  tw(:,j) = undef ;  qw(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ;  tw(i,:) = undef ;  qw(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ;  tw(i,:) = undef ;  qw(i,:) = undef ; ENDDO 

   DO j =  1, 1+jbdcut ;  ts(:,j) = undef ;  qs(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ;  ts(:,j) = undef ;  qs(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ;  ts(i,:) = undef ;  qs(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ;  ts(i,:) = undef ;  qs(i,:) = undef ; ENDDO 

   DO j =  1, 1+jbdcut ; sst(:,j) = undef ; ssq(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ; sst(:,j) = undef ; ssq(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ; sst(i,:) = undef ; ssq(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ; sst(i,:) = undef ; ssq(i,:) = undef ; ENDDO 

   DO j =  1, 1+jbdcut ; t02(:,j) = undef ; q02(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ; t02(:,j) = undef ; q02(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ; t02(i,:) = undef ; q02(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ; t02(i,:) = undef ; q02(i,:) = undef ; ENDDO 

   DO j =  1, 1+jbdcut ; t10(:,j) = undef ; q10(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ; t10(:,j) = undef ; q10(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ; t10(i,:) = undef ; q10(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ; t10(i,:) = undef ; q10(i,:) = undef ; ENDDO 

!  DO j =  1, 1+jbdcut ; u10(:,j) = undef ; v10(:,j) = undef ; ENDDO 
!  DO j = jm-jbdcut,jm ; u10(:,j) = undef ; v10(:,j) = undef ; ENDDO 
!  DO i =  1, 1+ibdcut ; u10(i,:) = undef ; v10(i,:) = undef ; ENDDO 
!  DO i = im-ibdcut,im ; u10(i,:) = undef ; v10(i,:) = undef ; ENDDO 

   endif !* ( .not. debug_data ) ----------------------------------- 

   ENDIF !* ( gfdl_surface ) - - - - - - - - - - - - - - - - - - - - - - 
   
   ENDIF !* ( derive_vars ) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
   DeAllocate( dp ) 
   
!*****************************************************! 
!  1a. Calculate storm center and intensity (tracker) ! 
!_____________________________________________________! 
   
   ibd1 = 1+ibdcut ; ibd9 = im-ibdcut 
   jbd1 = 1+jbdcut ; jbd9 = jm-jbdcut 
   
!* Read first guess of storm center 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   OPEN(50,file='storm_center') 
   READ(50,*) stormlat, stormlon, previous_time ; CLOSE(50) 
   Call Convert_LatLon( 1, 1, 1, stormlon,stormlat, xxx,yyy, ctr_lon,ctr_lat ) 
   iloc = 1 + nint((xxx-xrr(1,1))/dlon) ; jloc = 1 + nint((yyy-yrr(1,1))/dlat) 
   
   IF ( fcst_time > 0 ) THEN 
      ttt = (fcst_time-previous_time)*60 
      rrr =  5.*ttt/111200. + 2.*r_inner  
      ioff = nint(rrr/dlon) ; joff = nint(rrr/dlat) 
      i1 = iloc - ioff ; i1 = max( ibd1, i1  ) 
      i2 = iloc + ioff ; i2 = min(  i2, ibd9 ) ; i2 = max(i1,i2)  
      j1 = jloc - joff ; j1 = max( jbd1, j1  ) 
      j2 = jloc + joff ; j2 = min(  j2, jbd9 ) ; j2 = max(j1,j2)  
      eee = ( i2 - i1 + 1 )*( j2 - j1 + 1 ) 
      uur = SUM(u10(i1:i2,j1:j2))/eee ; vvr = SUM(v10(i1:i2,j1:j2))/eee 
      iloc = iloc + nint(uur*ttt/xrlen(jloc)) ; iloc = max(ibd1,min(iloc,ibd9)) 
      jloc = jloc + nint(vvr*ttt/yrlen) ; jloc = max(jbd1,min(jloc,jbd9)) 
      stormlon = xrs(iloc,jloc) ; stormlat = yrs(iloc,jloc) 
   ENDIF 

   iguess = iloc ; xguess = stormlon 
   jguess = jloc ; yguess = stormlat 

!* Calculate storm center and min MSLP 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   ioff = int(r_outer/dlon) + 1 
   joff = int(r_outer/dlat) + 1 ; rho = r_outer**2 
   
   i1 = iloc - ioff ; i1 = max( ibd1, i1  ) 
   i2 = iloc + ioff ; i2 = min(  i2, ibd9 ) ; i2 = max(i1,i2)  
   j1 = jloc - joff ; j1 = max( jbd1, j1  ) 
   j2 = jloc + joff ; j2 = min(  j2, jbd9 ) ; j2 = max(j1,j2)  
   
   pmin = p0(iloc,jloc) 
   DO j = j1, j2 ; DO i = i1, i2 
      rrr = ((i-iloc)*dlon)**2 + ((j-jloc)*dlat)**2 
      IF ( rrr <= rho ) pmin = min( pmin, p0(i,j) )
   ENDDO ; ENDDO 
   pguess = pmin 
   
   IF ( fcst_time > 0 ) THEN !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
   DO m = 1, msearch !* -------------------------------------------- 
   
   ioff = int(rsearch(m)/dlon) + 1 
   joff = int(rsearch(m)/dlat) + 1 ; rho = rsearch(m)**2 
   
   DO n = 0, itermax  !* - - - - - - - - - - - - - - - - - - - - - - 
   
   i1 = iloc - ioff ; i1 = max( ibd1, i1  ) 
   i2 = iloc + ioff ; i2 = min(  i2, ibd9 ) ; i2 = max(i1,i2)  
   j1 = jloc - joff ; j1 = max( jbd1, j1  ) 
   j2 = jloc + joff ; j2 = min(  j2, jbd9 ) ; j2 = max(j1,j2)  
   
   pmax = p0(iloc,jloc) ; pmin = p0(iloc,jloc) 
   DO j = j1, j2 ; DO i = i1, i2 
      rrr = ((i-iloc)*dlon)**2 + ((j-jloc)*dlat)**2 
      IF ( rrr <= rho ) THEN 
         pmax = max( pmax, p0(i,j) ) ; pmin = min( pmin, p0(i,j) )
      ENDIF 
   ENDDO ; ENDDO
    xxx = max(.1,min(pmax-pmin,8.))*0.1 
   pcut = pmin + min(xxx*(pmax-pmin),8.) !* (mb) 
 
   aaa = 0 ; bbb = 0 ; ccc = 0 ; ddd = 0 ; eee = 0 
   DO j = j1, j2 ; DO i = i1, i2 
      rrr = ((i-iloc)*dlon)**2 + ((j-jloc)*dlat)**2 
      IF ( rrr <= rho ) THEN 
         www = max( pcut - p0(i,j), 0. )
         aaa = aaa + www*i ; bbb = bbb + www*j ; eee = eee + www 
         ccc = ccc + www*xrs(i,j) ; ddd = ddd + www*yrs(i,j) 
      ENDIF 
   ENDDO ; ENDDO
   
   IF ( eee == 0 ) EXIT !* Invalid range of MSLP 
   iold = iloc ; iloc = nint(aaa/eee) ; stormlon = ccc/eee 
   jold = jloc ; jloc = nint(bbb/eee) ; stormlat = ddd/eee 
   IF ( iold==iloc .AND. jold==jloc ) EXIT  !* Converged 
   
   ENDDO  !* ( itermax ) - - - - - - - - - - - - - - - - - - - - - - 
   
   IF ( m == 1 ) THEN 
      itmp = iloc ; alon = stormlon 
      jtmp = jloc ; alat = stormlat ; ptmp = pmin 
      itercen = n ; stormrad = rsearch(m) 
   ELSE !* Select the location near 1st guess 
      k0 = (itmp-iguess)**2 + (jtmp-jguess)**2 
      kk = (iloc-iguess)**2 + (jloc-jguess)**2 
      IF ( kk < k0 ) THEN 
      itmp = iloc ; alon = stormlon 
      jtmp = jloc ; alat = stormlat ; ptmp = pmin 
      itercen = n ; stormrad = rsearch(m) 
      ENDIF 
   ENDIF 

   ENDDO !* ( msearch ) -------------------------------------------- 

      iloc = itmp ; stormlon = alon 
      jloc = jtmp ; stormlat = alat ; pmin = ptmp 

   ENDIF !* ( fcst_time > 0 ) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
!* Check if the storm is well inside domain 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   imid = (im+1)/2 ; ioff = Abs(iloc-imid) ; ii = nint(r_inner/dlon) 
   jmid = (jm+1)/2 ; joff = Abs(jloc-jmid) ; jj = nint(r_inner/dlat) 
   valid_track = (ioff+ii <= ibd9-imid) .AND. (joff+jj <= jbd9-jmid)

   OPEN(50,file='storm_center') 
   WRITE(50,*) stormlat, stormlon, fcst_time ; CLOSE(50) 
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* Calculate max 10m wind & RMW 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   yyy = min(jloc-jbd1,jbd9-jloc)*dlat 
   xxx = min(iloc-ibd1,ibd9-iloc)*dlon 
   rho = min(xxx,yyy,9.);  eee = rho**2 
  ioff = nint(rho/dlon) ; joff = nint(rho/dlat) 
    i1 = iloc - ioff ; i2 = iloc + ioff 
    j1 = jloc - joff ; j2 = jloc + joff 
   
   vmax = 0. ; rmw = 0. 
   DO j = j1, j2 ; DO i = i1, i2 
      aaa = ((i-iloc)*dlon)**2 + ((j-jloc)*dlat)**2 
      IF ( aaa <= eee ) THEN 
         www = u10(i,j)**2 + v10(i,j)**2 
         IF ( vmax <= www ) THEN 
            xxlen = 0.5*( xrlen(jloc) + xrlen(j) ) 
            rrr = ((i-iloc)*xxlen)**2 + ((j-jloc)*yrlen)**2 
            IF ( vmax < www ) THEN 
               vmax = www ; rmw = rrr 
            ELSE !* ( vmax == www ) 
               rmw = max( rmw,rrr ) 
            ENDIF 
         ENDIF 
      ENDIF 
   ENDDO ; ENDDO

  vmax = sqrt(vmax) !* [m/s] 
   rrr = sqrt(rmw)  !* [m] 
   rmw = rrr*.001   !* [km] 

   ii  = nint(rrr/xrlen(jloc)) 
   IF ( ioff <= ii ) rmw =  0.  

  ipmin = nint(pmin)        !* [mb] 
  ivmax = nint(vmax*mps2kt) !* [kt] 
   irmw = nint(rmw/1.852)   !* [nm] 

!           *     *     *     *     *     *     *     *     *     * 
   
!* Define MSLP & 10m winds on cylindrical grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   drlen = 1852    !* radial resolution dr = 1 nm 
   rho = 900*1852  !* max search radius =  900 nm 
   jms = nint(rho/drlen)  !* max radial dimension 
   jmx = int( min(jloc-jbd1,jbd9-jloc)*yrlen/drlen ) 
   imx = int( min(iloc-ibd1,ibd9-iloc)*xrlen(jloc)/drlen ) 
   jmc = min( imx, jmx, jms ) !* radial dimension 
   imc = 360 ; dphi = 2*pi_cnst/imc  !* dphi = 1 degree 

   Allocate( p0x(0:imc,0:jmc), wnd(0:imc,0:jmc), wk1(0:imc,0:jmc) ) 

   xxx = wcos(iloc,jloc) 
   yyy = wsin(iloc,jloc) 
   sqrt2_ = 1./sqrt(2.) 
   IF (  xxx > sqrt2_ ) THEN 
      alfa = Asin(yyy) 
   ELSEIF (  xxx  < -sqrt2_ ) THEN 
      alfa = pi_cnst*Sign(1.,yyy) - Asin(yyy) 
   ELSE ! ( |xxx| <= sqrt2_ ) ! 
      alfa = Acos(xxx)*Sign(1.,yyy) 
   ENDIF  !* Angle of rotated east 
   
   DO i = 1, imc 
       phi = i*dphi - alfa 
      rcos = cos(phi) ; rsin = sin(phi) 
      DO j = 1, jmc  !* r = j*dr 
         yyy = j*rsin*drlen/yrlen ; jtmp = int(yyy) ; IF (yyy<0) jtmp = jtmp-1 
         ccc = yyy - jtmp ; ddd = 1 - ccc ; n = jloc + jtmp ; n1 = n + 1 
             xxlen = .5*( xrlen(jloc) + ddd*xrlen(n) + ccc*xrlen(n1) ) 
         xxx = j*rcos*drlen/xxlen ; itmp = int(xxx) ; IF (xxx<0) itmp = itmp-1 
         aaa = xxx - itmp ; bbb = 1 - aaa ; m = iloc + itmp ; m1 = m + 1 
         p0x(i,j) = bbb*( ddd* p0(m ,n) + ccc* p0(m ,n1) ) &
                  + aaa*( ddd* p0(m1,n) + ccc* p0(m1,n1) ) 
             uur  = bbb*( ddd*u10(m ,n) + ccc*u10(m ,n1) ) &
                  + aaa*( ddd*u10(m1,n) + ccc*u10(m1,n1) ) 
             vvr  = bbb*( ddd*v10(m ,n) + ccc*v10(m ,n1) ) &
                  + aaa*( ddd*v10(m1,n) + ccc*v10(m1,n1) ) 
         wnd(i,j) = sqrt( uur**2 + vvr**2 )*mps2kt !* wind speed [kt] 
         wk1(i,j) = - rsin*uur + rcos*vvr !* CCW tangential wind [m/s] 
      ENDDO 
   ENDDO 
   
   p0x(0,:) = p0x(imc,:) ; p0x(:,0) = p0(iloc,jloc) 
   www = sqrt(u10(iloc,jloc)**2+v10(iloc,jloc)**2)*mps2kt  
   wnd(0,:) = wnd(imc,:) ; wnd(:,0) = www 
   wk1(0,:) = wk1(imc,:) ; wk1(:,0) = 0. 

!* Radius of max avg tan wind (RMVt) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   jmr = min(jmc,2*jms/3) 
   jvt = 0 ; vtmax = 0. 

   DO j = 1, jmr 
      www = SUM(wk1(1:imc,j))/imc 
      IF ( vtmax <= www ) THEN 
         jvt = j ; vtmax = www 
      ENDIF 
   ENDDO 

   rmvt = jvt*drlen*0.001  !* [km] 
   IF (jvt==jmr) rmvt = 0  !* outside 

   valid_track = (valid_track) .AND. (v_close <= vtmax) 
   
!* Search R34, R50, R64 [nm] in quadrants 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   m0 = 0 ; m1 = imc/4 ; m2 = imc/2 ; m3 = 3*imc/4 ; m4 = imc 
   jmr = min(jmc,2*jms/3) 

   i34ne = 0 ; i50ne = 0 ; i64ne = 0 
   DO i = m0, m1 
      n1 = 0 ; n2 = 0 ; n3 = 0 
      DO j = 0, jmr 
      if ( wnd(i,j) >= 34 ) n1 = j 
      if ( wnd(i,j) >= 50 ) n2 = j 
      if ( wnd(i,j) >= 64 ) n3 = j 
      ENDDO 
      i34ne = max( i34ne, n1 ) 
      i50ne = max( i50ne, n2 ) 
      i64ne = max( i64ne, n3 ) 
   ENDDO 
   IF ( i34ne == jmr ) i34ne = 0   !* outside 
   IF ( i50ne == jmr ) i50ne = 0   !* outside 
   IF ( i64ne == jmr ) i64ne = 0   !* outside 
   i34ne = nint(i34ne*drlen/1852.) !* [nm] 
   i50ne = nint(i50ne*drlen/1852.) !* [nm] 
   i64ne = nint(i64ne*drlen/1852.) !* [nm] 

   i34nw = 0 ; i50nw = 0 ; i64nw = 0 
   DO i = m1, m2 
      n1 = 0 ; n2 = 0 ; n3 = 0 
      DO j = 0, jmr 
      if ( wnd(i,j) >= 34 ) n1 = j 
      if ( wnd(i,j) >= 50 ) n2 = j 
      if ( wnd(i,j) >= 64 ) n3 = j 
      ENDDO 
      i34nw = max( i34nw, n1 ) 
      i50nw = max( i50nw, n2 ) 
      i64nw = max( i64nw, n3 ) 
   ENDDO 
   IF ( i34nw == jmr ) i34nw = 0   !* outside 
   IF ( i50nw == jmr ) i50nw = 0   !* outside 
   IF ( i64nw == jmr ) i64nw = 0   !* outside 
   i34nw = nint(i34nw*drlen/1852.) !* [nm] 
   i50nw = nint(i50nw*drlen/1852.) !* [nm] 
   i64nw = nint(i64nw*drlen/1852.) !* [nm] 

   i34sw = 0 ; i50sw = 0 ; i64sw = 0 
   DO i = m2, m3 
      n1 = 0 ; n2 = 0 ; n3 = 0 
      DO j = 0, jmr 
      if ( wnd(i,j) >= 34 ) n1 = j 
      if ( wnd(i,j) >= 50 ) n2 = j 
      if ( wnd(i,j) >= 64 ) n3 = j 
      ENDDO 
      i34sw = max( i34sw, n1 ) 
      i50sw = max( i50sw, n2 ) 
      i64sw = max( i64sw, n3 ) 
   ENDDO 
   IF ( i34sw == jmr ) i34sw = 0   !* outside 
   IF ( i50sw == jmr ) i50sw = 0   !* outside 
   IF ( i64sw == jmr ) i64sw = 0   !* outside 
   i34sw = nint(i34sw*drlen/1852.) !* [nm] 
   i50sw = nint(i50sw*drlen/1852.) !* [nm] 
   i64sw = nint(i64sw*drlen/1852.) !* [nm] 

   i34se = 0 ; i50se = 0 ; i64se = 0 
   DO i = m3, m4 
      n1 = 0 ; n2 = 0 ; n3 = 0 
      DO j = 0, jmr 
      if ( wnd(i,j) >= 34 ) n1 = j 
      if ( wnd(i,j) >= 50 ) n2 = j 
      if ( wnd(i,j) >= 64 ) n3 = j 
      ENDDO 
      i34se = max( i34se, n1 ) 
      i50se = max( i50se, n2 ) 
      i64se = max( i64se, n3 ) 
   ENDDO 
   IF ( i34se == jmr ) i34se = 0   !* outside 
   IF ( i50se == jmr ) i50se = 0   !* outside 
   IF ( i64se == jmr ) i64se = 0   !* outside 
   i34se = nint(i34se*drlen/1852.) !* [nm] 
   i50se = nint(i50se*drlen/1852.) !* [nm] 
   i64se = nint(i64se*drlen/1852.) !* [nm] 

!* Search POCI [mb] and ROCI [nm] 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   kout = 1050 !* max outer pressure 
   ccc  = 1.75 !* aspect constraint 
   IF ( fcst_time == 0 ) ccc = 2.25 

   ipoci = kout ; iroci = 0 
   IF ( jmc == jms ) THEN !* - - - - - - - - - - - - - - - - - - 

   m1 = int(p0x(0,0))+1 ; m2 = kout 
   DO i = 1, imc
      m0 = MaxVal(p0x(i,:)) 
      m2 = min( m2, m0 ) 
   ENDDO 
   m2 = max( m1, m2 ) 

   DO m = m1, m2 
      jmax = 0 ; jmin = kout
      DO i = 1, imc
         DO j = 1, jmc 
         if ( m <= p0x(i,j) ) EXIT 
         ENDDO 
         IF ( j > jmc ) EXIT 
         jmax = max( jmax, j ) 
         jmin = min( jmin, j ) 
      ENDDO 
      IF ( j <= jmc .AND. jmax <= ccc*jmin ) THEN 
         ipoci = m ; iroci = jmax 
      ENDIF 
   ENDDO 

   iroci = nint(iroci*drlen/1852.) !* [nm] 

   ENDIF  !* - - - - - - - - - - - - - - - - - - - - - - - - - - 

   DeAllocate( p0x, wnd, wk1 ) 
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* Output domain and tracker information 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   i0 = (im+1)/2 ; j0 = (jm+1)/2 ; rrr = stormrad ; kk = itercen 
   n = nint(fcst_time) ; ifcst_hour = n/60 ; ifcst_min = Mod(n,60) 
   aa1 = stormlon   ; bb1 = stormlat   ; xxx = sqrt(dlon**2+dlat**2) 
   aaa = xrs(i0,j0) ; bbb = yrs(i0,j0) ; ccc = ctr_lon ; ddd = ctr_lat 

   WRITE(*,'(/, " Processing Domain: ",  I3)')  domain_num 
   WRITE(*,'(   " Forecast Hour Min: ", 2I3)')  ifcst_hour, ifcst_min 
!  IF (iter_qt > iterpot) WRITE(*,'(" Theta_e  has not converged yet !")') 
!  IF (itercen > itermax) WRITE(*,'(" Storm Center has not converged !")') 
   IF (.not. valid_track) WRITE(*,'(" Storm Center is NOT well defined !")') 
   WRITE(*,'(" Storm Search R (deg) &  Iterations =", F12.5,I6)') rrr, kk 
   WRITE(*,'(" Max 10m Wind (m/s) & Min MSLP (mb) =", 2F12.5)')  vmax, pmin 
   WRITE(*,'(" Radii of Max Wind, Max Avg Vt (km) =", 2F12.5)')   rmw, rmvt 
   WRITE(*,'(" Storm Center by Avg MSLP: lon, lat =", 2F12.5)')   aa1, bb1 
   WRITE(*,'(" Processed  Domain Center: lon, lat =", 2F12.5)')   aaa, bbb 
   WRITE(*,'(" Origin of  Rotated  Grid: lon, lat =", 2F12.5)')   ccc, ddd 
   WRITE(*,'(" Native E-Grid Resolution: lon, lat =", 2F12.5)')   xxx, xxx 
   WRITE(*,'(" Equiv. A-Grid Resolution: lon, lat =", 2F12.5)')  dlon, dlat
   WRITE(*,'(" Equiv. A-Grid Dimensions:  X, Y, Z =", 3I10)')    im, jm, km 
   
   WRITE(51,*) &
  'start_date   hr  min  bad   stormlon  stormlat  MSLP(mb)  Vmax(ms)', &
  '----------  ---  ---  ---  ---------  --------  --------  --------' 
   
   ibad = 0 ; IF ( .not. valid_track ) ibad = 1  
   WRITE(52,'(I11,I5,I4,I5,F12.5,F10.5,F10.3,F9.3)') &
   start_date,ifcst_hour,ifcst_min,ibad,stormlon,stormlat,pmin,vmax

   lat10 = nint(stormlat*10) ; lon10 = nint(-stormlon*10) 
   IF ( valid_track ) THEN !* - - - - - - - - - - - - - - - - - - - - - 
   
   WRITE(54,'(A2,A1,A3,A1,I11,A1,I3.2,A1,A5,A1,I4,A1,I4,A2,I5,A2,I4,A1,I5,A1, & 
              A3,A1, I4,A1, A4,A1, I5,A1,I5,A1,I5,A1,I5,A1, I5,A1,I5,A1,I4)') & 
              storm_basin,',',  storm_num,',', start_date,',', ifcst_min,',', & 
                atcf_code,',', ifcst_hour,',',     lat10,'N,',    lon10,'W,', & 
              ivmax,',', ipmin,',', ' XX',',', 34,',', ' NEQ',',', i34ne,',', & 
              i34se,',', i34sw,',', i34nw,',', ipoci,',', iroci,',', irmw 
   
   if ( 50 <= ivmax ) then 
   WRITE(54,'(A2,A1,A3,A1,I11,A1,I3.2,A1,A5,A1,I4,A1,I4,A2,I5,A2,I4,A1,I5,A1, & 
              A3,A1, I4,A1, A4,A1, I5,A1,I5,A1,I5,A1,I5,A1, I5,A1,I5,A1,I4)') & 
              storm_basin,',',  storm_num,',', start_date,',', ifcst_min,',', & 
                atcf_code,',', ifcst_hour,',',     lat10,'N,',    lon10,'W,', & 
              ivmax,',', ipmin,',', ' XX',',', 50,',', ' NEQ',',', i50ne,',', & 
              i50se,',', i50sw,',', i50nw,',', ipoci,',', iroci,',', irmw 
   endif 
   
   if ( 64 <= ivmax ) then 
   WRITE(54,'(A2,A1,A3,A1,I11,A1,I3.2,A1,A5,A1,I4,A1,I4,A2,I5,A2,I4,A1,I5,A1, & 
              A3,A1, I4,A1, A4,A1, I5,A1,I5,A1,I5,A1,I5,A1, I5,A1,I5,A1,I4)') & 
              storm_basin,',',  storm_num,',', start_date,',', ifcst_min,',', & 
                atcf_code,',', ifcst_hour,',',     lat10,'N,',    lon10,'W,', & 
              ivmax,',', ipmin,',', ' XX',',', 64,',', ' NEQ',',', i64ne,',', & 
              i64se,',', i64sw,',', i64nw,',', ipoci,',', iroci,',', irmw 
   endif 
   
   ENDIF !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   
   write(53,*) '                                                             '
   write(53,*) 'Column  1. Storm basin: AL = Atlantic.                       '
   write(53,*) 'Column  2. Annual cyclone number: 1-99.                      '
   write(53,*) 'Column  3. Model start date: YYYYMMDDHH.                     '
   write(53,*) 'Column  4. Forecast minute: 00-59.                           '
   write(53,*) 'Column  5. Technique: 4 characters.                          '
   write(53,*) 'Column  6. Forecast hour: 0-999.                             '
   write(53,*) 'Column  7. Latitude  (tenths of degree).                     '
   write(53,*) 'Column  8. Longitude (tenths of degree).                     '
   write(53,*) 'Column  9. Maximum 10-meter wind speed (kt).                 '
   write(53,*) 'Column 10. Minimum sea-level pressure  (mb).                 '
   write(53,*) 'Column 11. Storm type: XX = unknown.                         '
   write(53,*) 'Column 12. Wind speed (kt)  of storm radii being shown.      '
   write(53,*) 'Column 13. Starting quarter of storm radii being shown.      '
   write(53,*) 'Column 14. Radius (nm) of given wind speed in NE quarter.    '
   write(53,*) 'Column 15. Radius (nm) of given wind speed in SE quarter.    '
   write(53,*) 'Column 16. Radius (nm) of given wind speed in SW quarter.    '
   write(53,*) 'Column 17. Radius (nm) of given wind speed in NW quarter.    '
   write(53,*) 'Column 18. Pressure (mb) of outermost closed isobar.         '
   write(53,*) 'Column 19. Radius (nm) of outermost closed isobar.           '
   write(53,*) 'Column 20. Radius (nm) of max 10-meter wind speed.           '
   write(53,*) '                                                             '
   
   CLOSE(51) ; CLOSE(52) ; CLOSE(53) ; CLOSE(54) 
   
!*********************************************************! 
!  1b. Interpolate to static domain for wind & rain swath ! 
!_________________________________________________________! 

   IF ( wind_swath ) THEN !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

!* Dimensions of canvas for composite of moving nest 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   imx = (nint(d01cut/dlon)/2)*2+1 ; jmx = (nint(aspect*imx)/2)*2+1 
   Allocate( u10x(imx,jmx), v10x(imx,jmx), p0x(imx,jmx), prx(imx,jmx) ) 
   u10x(:,:) = undef ; v10x(:,:) = undef ; p0x(:,:) = undef ;  prx(:,:) = undef 

!* Standard grid that circumscribes the rotated grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   aaa = MinVal( xrs( 1,1:jm) )        ; ccc = MinVal( yrs(1:im,1 ) ) 
   bbb = MaxVal( xrs(im,1:jm) )        ; ddd = MaxVal( yrs(1:im,jm) ) 
   ims = (nint((bbb-aaa)/dlon)/2)*2+1  ; jms = (nint((ddd-ccc)/dlat)/2)*2+1
   Allocate( xsr(ims,jms), ysr(ims,jms), xss(ims,jms), yss(ims,jms) ) 
   
   xss(1,1) = ctr_lon + ( nint((.5*(aaa+bbb)-ctr_lon)/dlon) - (ims-1)/2 )*dlon 
   yss(1,1) = ctr_lat + ( nint((.5*(ccc+ddd)-ctr_lat)/dlat) - (jms-1)/2 )*dlat 
   DO i = 1, ims ; xss(i,:) = xss(1,1) + (i-1)*dlon ; ENDDO 
   DO j = 1, jms ; yss(:,j) = yss(1,1) + (j-1)*dlat ; ENDDO 

   Call Convert_LatLon( 1, ims, jms, xss, yss, xsr, ysr, ctr_lon, ctr_lat ) 
   
!* Convert rotated wind components to standard ones 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   urs(:,:) = wcos(:,:)*u10(:,:) - wsin(:,:)*v10(:,:) 
   vrs(:,:) = wsin(:,:)*u10(:,:) + wcos(:,:)*v10(:,:) 
   
!* Interpolate to the canvas domain on standard grid  
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xrr1 = xrr(1,1) ; xrr2 = xrr(2,1) ; xrr8 = xrr(im-1,1) 
   yrr1 = yrr(1,1) ; yrr2 = yrr(1,2) ; yrr8 = yrr(1,jm-1) 
    i0 = (imx-1)/2 + nint( (xss(1,1)-ctr_lon)/dlon ) 
    j0 = (jmx-1)/2 + nint( (yss(1,1)-ctr_lat)/dlat ) 
   
   IF ( iorder == 2 ) THEN !* -------------------------------------- 

   DO j = 1, jms ; j1 = j0 + j 
   if ( 1 <= j1 .AND. j1 <= jmx ) then 
      DO i = 1, ims ; i1 = i0 + i 
      if ( 1 <= i1 .AND. i1 <= imx ) then 
         xxx = xsr(i,j) ;  yyy = ysr(i,j) 
         IF ( yrr2 <= yyy .AND. yyy < yrr8 ) THEN 
         if ( xrr2 <= xxx .AND. xxx < xrr8 ) then 
             m  = 1 + int((xxx-xrr1)/dlon) ;  m1 = m + 1 
             n  = 1 + int((yyy-yrr1)/dlat) ;  n1 = n + 1 
            aaa = ( xxx - xrr(m,n) )/dlon  ; bbb = 1 - aaa 
            ccc = ( yyy - yrr(m,n) )/dlat  ; ddd = 1 - ccc 
          p0x(i1,j1) = bbb*( ddd* p0(m ,n) + ccc* p0(m ,n1) ) &
                     + aaa*( ddd* p0(m1,n) + ccc* p0(m1,n1) ) 
          prx(i1,j1) = bbb*( ddd*prc(m ,n) + ccc*prc(m ,n1) ) &
                     + aaa*( ddd*prc(m1,n) + ccc*prc(m1,n1) ) 
         u10x(i1,j1) = bbb*( ddd*urs(m ,n) + ccc*urs(m ,n1) ) &
                     + aaa*( ddd*urs(m1,n) + ccc*urs(m1,n1) ) 
         v10x(i1,j1) = bbb*( ddd*vrs(m ,n) + ccc*vrs(m ,n1) ) &
                     + aaa*( ddd*vrs(m1,n) + ccc*vrs(m1,n1) ) 
         endif 
         ENDIF 
      endif 
      ENDDO 
   endif 
   ENDDO 

   ELSEIF ( iorder == 4 ) THEN !* ---------------------------------- 

   DO j = 1, jms ; j1 = j0 + j 
   if ( 1 <= j1 .AND. j1 <= jmx ) then 
      DO i = 1, ims ; i1 = i0 + i 
      if ( 1 <= i1 .AND. i1 <= imx ) then 
         xxx = xsr(i,j) ;  yyy = ysr(i,j) 
         IF ( yrr2 <= yyy .AND. yyy < yrr8 ) THEN 
         if ( xrr2 <= xxx .AND. xxx < xrr8 ) then 
            m2 =  1 + int((xxx-xrr1)/dlon);    m1 = m2-1 ; m4 = m2+2 
            n2 =  1 + int((yyy-yrr1)/dlat);    n1 = n2-1 ; n4 = n2+2 
           aaa =  (xxx - xrr(m2,n2))/dlon ;   bbb = (yyy - yrr(m2,n2))/dlat 
         wtx(1)= -(aaa)*(aaa-1)*(aaa-2)/6 ; wtx(2)= (aaa+1)*(aaa-1)*(aaa-2)/2
         wtx(3)= -(aaa+1)*(aaa)*(aaa-2)/2 ; wtx(4)= (aaa+1)*(aaa)*(aaa-1)/6
         wty(1)= -(bbb)*(bbb-1)*(bbb-2)/6 ; wty(2)= (bbb+1)*(bbb-1)*(bbb-2)/2
         wty(3)= -(bbb+1)*(bbb)*(bbb-2)/2 ; wty(4)= (bbb+1)*(bbb)*(bbb-1)/6
!        - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
         ff= p0(m1:m4,n1:n4); CALL Cubic_Interpo( p0x(i1,j1),ff,wtx,wty,undef) 
         ff=prc(m1:m4,n1:n4); CALL Cubic_Interpo( prx(i1,j1),ff,wtx,wty,undef) 
         ff=urs(m1:m4,n1:n4); CALL Cubic_Interpo(u10x(i1,j1),ff,wtx,wty,undef) 
         ff=vrs(m1:m4,n1:n4); CALL Cubic_Interpo(v10x(i1,j1),ff,wtx,wty,undef) 
         endif 
         ENDIF 
      endif 
      ENDDO 
   endif 
   ENDDO 

   ENDIF !* ( iorder ) --------------------------------------------- 
   
   WRITE(62)  p0x ; WRITE(62)  prx  
   WRITE(62) u10x ; WRITE(62) v10x 
   nvars = 4 

!* GrADS control file for moving nest on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxs1 = .5*(1-imx)*dlon + ctr_lon ; xxs_ = dlon !* d01 west  bdry 
   yys1 = .5*(1-jmx)*dlat + ctr_lat ; yys_ = dlat !* d01 south bdry 
   
   WRITE(61,'("                                                      ")') 
   WRITE(61,'(" DSET    swath_std.dat                                ")') 
   WRITE(61,'(" TITLE   Moving nest data on standard latlon grid     ")') 
   WRITE(61,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(61,'(" UNDEF",E11.2)') undef 
   WRITE(61,'("                                                      ")') 
   WRITE(61,'(" XDEF", I7,"   LINEAR", 2F11.5 )') imx, xxs1, xxs_ 
   WRITE(61,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jmx, yys1, yys_ 
   WRITE(61,'(" ZDEF", I7,"   LINEAR          1          1 ")')   1 
   WRITE(61,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(61,'("                                                      ")') 
   WRITE(61,'(" VARS", I3)') nvars
   WRITE(61,'(" p0        0   0    mean sea-level pressure (mb)      ")') 
   if (accum_precip) then !*  - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(61,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(61,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(61,'(" u10       0   0    10m zonal wind comp (m/s)         ")') 
   WRITE(61,'(" v10       0   0    10m meridional wind (m/s)         ")') 
   WRITE(61,'(" ENDVARS                                              ")') 
   WRITE(61,'("                                                      ")') 
   
   CLOSE(61) ; CLOSE(62) 
   DeAllocate( u10x, v10x, p0x, prx, xsr, ysr, xss, yss ) 
   
   ENDIF !* ( wind_swath ) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

!*****************************************************! 
!  1c. Sigma-p-level variables on rotated latlon grid ! 
!_____________________________________________________! 

   IF ( sigma_level ) THEN !* ------------------------------------------ 
   
!* Output variables on sigma-p levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF ( .not. refine_rot ) THEN !* - - - - - - - - - - - - - - - - - 

   WRITE(100) xrs ; WRITE(100) yrs 
   WRITE(100) zs  ; WRITE(100) sm 
   WRITE(100) z0m ; WRITE(100) z0t 
   WRITE(100) uf  ; WRITE(100) prc 
   WRITE(100) cwh ; WRITE(100) qqh 
   WRITE(100) ps  ; WRITE(100) p0 
   WRITE(100) tw  ; WRITE(100) qw 
   WRITE(100) ts  ; WRITE(100) qs 
   WRITE(100) sst ; WRITE(100) ssq 
   WRITE(100) t10 ; WRITE(100) q10 
   WRITE(100) u10 ; WRITE(100) v10 
   do k =  1, km  ; WRITE(100) uu(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) vv(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) ww(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) tt(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) td(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) qt(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) gh(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) pp(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) qq(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) cw(:,:,k) ; enddo 
   do k =  1, km  ; WRITE(100) rh(:,:,k) ; enddo 
   
   nvars = 33 ; dxx = dlon ; dyy = dlat 
    imr  = im ; jmr =  jm  ; ims =  im ; jms = (nint(aspect*ims)/2)*2 + 1 
   
!* Refine the grid for low resolutions 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   ELSE !* Refine low resolution - - - - - - - - - - - - - - - - - - 

   im3 = 3*im - 2 ; jm3 = 3*jm - 2 ; k2d = 22 
   Allocate( uu3(im3,jm3,km), vv3(im3,jm3,km), ww3(im3,jm3,km) ) 
   Allocate( tt3(im3,jm3,km), td3(im3,jm3,km), qt3(im3,jm3,km) ) 
   Allocate( pp3(im3,jm3,km), qq3(im3,jm3,km), cw3(im3,jm3,km) ) 
   Allocate( gh3(im3,jm3,km), rh3(im3,jm3,km), wks(im3,jm3,k2d)) 
   
   CALL Refine_Grid3x( xrs(1,1), wks(1,1,1),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( yrs(1,1), wks(1,1,2),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  zs(1,1), wks(1,1,3),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  sm(1,1), wks(1,1,4),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( z0m(1,1), wks(1,1,5),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( z0t(1,1), wks(1,1,6),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  uf(1,1), wks(1,1,7),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( prc(1,1), wks(1,1,8),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( cwh(1,1), wks(1,1,9),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( qqh(1,1), wks(1,1,10), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  ps(1,1), wks(1,1,11), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  p0(1,1), wks(1,1,12), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  tw(1,1), wks(1,1,13), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  qw(1,1), wks(1,1,14), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  ts(1,1), wks(1,1,15), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  qs(1,1), wks(1,1,16), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( sst(1,1), wks(1,1,17), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( ssq(1,1), wks(1,1,18), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( t10(1,1), wks(1,1,19), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( q10(1,1), wks(1,1,20), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( u10(1,1), wks(1,1,21), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( v10(1,1), wks(1,1,22), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( uu, uu3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( vv, vv3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( ww, ww3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( tt, tt3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( td, td3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( qt, qt3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( gh, gh3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( pp, pp3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( qq, qq3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( cw, cw3, undef, im3, jm3, im, jm, km )
   CALL Refine_Grid3x( rh, rh3, undef, im3, jm3, im, jm, km )
   
!* Positive-definite constraints 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   wks(:,:,4)  = max( wks(:,:,4),  0. )
   wks(:,:,5)  = max( wks(:,:,5),  0. )
   wks(:,:,6)  = max( wks(:,:,6),  0. )
   wks(:,:,7)  = max( wks(:,:,7),  0. )
   wks(:,:,8)  = max( wks(:,:,8),  0. )
   wks(:,:,9)  = max( wks(:,:,9),  0. )
   wks(:,:,10) = max( wks(:,:,10), 0. )
   wks(:,:,16) = max( wks(:,:,16), 0. )
   wks(:,:,18) = max( wks(:,:,18), 0. )
   wks(:,:,20) = max( wks(:,:,20), 0. )
   qq3(:,:,:)  = max( qq3(:,:,:),  0. )
   cw3(:,:,:)  = max( cw3(:,:,:),  0. )
   rh3(:,:,:)  = min( rh3(:,:,:), 100.)
   td3(:,:,:)  = min( td3(:,:,:), tt3(:,:,:) )
   
   do k = 1, k2d ; WRITE(100) wks(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) uu3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) vv3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) ww3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) tt3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) td3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) qt3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) gh3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) pp3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) qq3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) cw3(:,:,k) ; enddo 
   do k = 1, km  ; WRITE(100) rh3(:,:,k) ; enddo 
   
   nvars = k2d + 11 ;  dxx = dlon/3. ; dyy = dlat/3. 
   imr = im3 ; jmr = jm3 ; ims = im3 ; jms = (nint(aspect*ims)/2)*2 + 1 
   
   DeAllocate( uu3, vv3, ww3, tt3, td3, qt3, gh3, pp3, qq3, cw3, rh3, wks ) 
   
   ENDIF !* ( refine_rot ) - - - - - - - - - - - - - - - - - - - - - 

!           *     *     *     *     *     *     *     *     *     * 
   
!* GrADS control file for sigma-p-level data on rotated grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxr1 = .5*(1-imr)*dxx ; xxr_ = dxx !* Lagrangian coord for moving nest 
   yyr1 = .5*(1-jmr)*dyy ; yyr_ = dyy !* and for fixed outer domain (d01) 
   
   WRITE(101,'("                                                      ")') 
   WRITE(101,'(" DSET    sigma_lev.dat                                ")') 
   WRITE(101,'(" TITLE   Sigma-p-level data on rotated latlon grid    ")') 
   WRITE(101,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(101,'(" UNDEF",E11.2)') undef 
   WRITE(101,'("                                                      ")') 
   WRITE(101,'(" XDEF", I7,"   LINEAR", 2F11.5 )') imr, xxr1, xxr_ 
   WRITE(101,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jmr, yyr1, yyr_ 
   WRITE(101,'(" ZDEF", I7,"   LINEAR          1          1 ")')  km 
   WRITE(101,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(101,'("                                                      ")') 
   WRITE(101,'(" VARS", I3)') nvars
   WRITE(101,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(101,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(101,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(101,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(101,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(101,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(101,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(101,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(101,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(101,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(101,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(101,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(101,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(101,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(101,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(101,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(101,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(101,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(101,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(101,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(101,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(101,'(" u10       0   0    rotated 10m x-wind (m/s)          ")') 
   WRITE(101,'(" v10       0   0    rotated 10m y-wind (m/s)          ")') 
   WRITE(101,'(" uu  ", I7,"   0    rotated x-wind (m/s)              ")') km 
   WRITE(101,'(" vv  ", I7,"   0    rotated y-wind (m/s)              ")') km 
   WRITE(101,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') km 
   WRITE(101,'(" tt  ", I7,"   0    air temperature (K)               ")') km 
   WRITE(101,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') km 
   WRITE(101,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') km 
   WRITE(101,'(" gh  ", I7,"   0    geopotential height (m)           ")') km 
   WRITE(101,'(" pp  ", I7,"   0    air  pressure  (mb)               ")') km 
   WRITE(101,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') km 
   WRITE(101,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') km 
   WRITE(101,'(" rh  ", I7,"   0    relative humidity (%)             ")') km 
   WRITE(101,'(" ENDVARS                                              ")') 
   WRITE(101,'("                                                      ")') 
   
!* GrADS control file for sigma-level data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxs1 = .5*(1-ims)*dxx + ctr_lon ; xxs_ = dxx !* d01 west  bdry 
   yys1 = .5*(1-jms)*dyy + ctr_lat ; yys_ = dyy !* d01 south bdry 
   
   WRITE(102,'("                                                      ")') 
   WRITE(102,'(" DSET    sigma_lev.dat                                ")') 
   WRITE(102,'(" TITLE   Sigma-p-level data on standard latlon grid   ")') 
   WRITE(102,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(102,'(" UNDEF",E11.2)') undef 
   WRITE(102,'("                                                      ")') 
   WRITE(102,'(" PDEF", I7,  I6,   A10,   4F11.5)') & 
                       imr, jmr, 'ETA.U', ctr_lon, ctr_lat, dxx,  dyy 
   WRITE(102,'(" XDEF", I7,"   LINEAR", 2F11.5 )') ims, xxs1, xxs_ 
   WRITE(102,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jms, yys1, yys_ 
   WRITE(102,'(" ZDEF", I7,"   LINEAR          1          1 ")')  km 
   WRITE(102,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(102,'("                                                      ")') 
   WRITE(102,'(" VECTORPAIRS   u10,v10  uu,vv                         ")') 
   WRITE(102,'(" VARS", I3)') nvars
   WRITE(102,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(102,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(102,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(102,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(102,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(102,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(102,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(102,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(102,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(102,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(102,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(102,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(102,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(102,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(102,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(102,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(102,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(102,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(102,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(102,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(102,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(102,'(" u10       0   0    10m zonal wind comp (m/s)         ")') 
   WRITE(102,'(" v10       0   0    10m meridional wind (m/s)         ")') 
   WRITE(102,'(" uu  ", I7,"   0    zonal wind comp (m/s)             ")') km 
   WRITE(102,'(" vv  ", I7,"   0    meridional wind (m/s)             ")') km 
   WRITE(102,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') km 
   WRITE(102,'(" tt  ", I7,"   0    air temperature (K)               ")') km 
   WRITE(102,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') km 
   WRITE(102,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') km 
   WRITE(102,'(" gh  ", I7,"   0    geopotential height (m)           ")') km 
   WRITE(102,'(" pp  ", I7,"   0    air  pressure  (mb)               ")') km 
   WRITE(102,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') km 
   WRITE(102,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') km 
   WRITE(102,'(" rh  ", I7,"   0    relative humidity (%)             ")') km 
   WRITE(102,'(" ENDVARS                                              ")') 
   WRITE(102,'("                                                      ")') 
   
   CLOSE(100) ; CLOSE(101) ; CLOSE(102) 
   
   ENDIF !* ( sigma_level ) -------------------------------------------- 
   
!***********************************************************! 
!  2a. 25mb-pressure-level variables on rotated latlon grid ! 
!___________________________________________________________! 

   IF ( press_vars ) THEN !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
!* Allocate work arrays for pressure levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate( uu_(im,jm,kmp), vv_(im,jm,kmp), ww_(im,jm,kmp) ) 
   Allocate( tt_(im,jm,kmp), td_(im,jm,kmp), qt_(im,jm,kmp) ) 
   Allocate( gh_(im,jm,kmp), qq_(im,jm,kmp), cw_(im,jm,kmp) ) 
   Allocate( rh_(im,jm,kmp) )
   
!* Interpolate from sigma to pressure levels (mb) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   uu_(:,:,:) = undef ; vv_(:,:,:) = undef ; ww_(:,:,:) = undef 
   tt_(:,:,:) = undef ; td_(:,:,:) = undef ; qt_(:,:,:) = undef 
   gh_(:,:,:) = undef ; qq_(:,:,:) = undef ; cw_(:,:,:) = undef 
   rh_(:,:,:) = undef 
   
   DO j = 1, jm 
   DO i = 1, im ; k0 = 1 
      DO m = 1, kmp ; xxx = plevel(m) 
      if ( pp(i,j,1) >= xxx .AND. xxx > pp(i,j,km) ) then 
         DO k = k0, km-1 ; IF ( xxx > pp(i,j,k+1) ) EXIT ; ENDDO 
         aaa = (xxx-pp(i,j,k+1))/(pp(i,j,k)-pp(i,j,k+1)) ; bbb = 1 - aaa 
         uu_(i,j,m) = aaa*uu(i,j,k) + bbb*uu(i,j,k+1) 
         vv_(i,j,m) = aaa*vv(i,j,k) + bbb*vv(i,j,k+1) 
         ww_(i,j,m) = aaa*ww(i,j,k) + bbb*ww(i,j,k+1) 
         tt_(i,j,m) = aaa*tt(i,j,k) + bbb*tt(i,j,k+1) 
         td_(i,j,m) = aaa*td(i,j,k) + bbb*td(i,j,k+1) 
         qt_(i,j,m) = aaa*qt(i,j,k) + bbb*qt(i,j,k+1) 
         gh_(i,j,m) = aaa*gh(i,j,k) + bbb*gh(i,j,k+1) 
         qq_(i,j,m) = aaa*qq(i,j,k) + bbb*qq(i,j,k+1) 
         cw_(i,j,m) = aaa*cw(i,j,k) + bbb*cw(i,j,k+1) 
         rh_(i,j,m) = aaa*rh(i,j,k) + bbb*rh(i,j,k+1) 
         k0 = k 
      endif 
      ENDDO 
   ENDDO 
   ENDDO 

   ENDIF !* ( press_vars ) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

!           *     *     *     *     *     *     *     *     *     * 
   
   IF ( press_level ) THEN !* ------------------------------------------ 
   
!* Output variables on pressure levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF ( .not. refine_rot ) THEN !* - - - - - - - - - - - - - - - - - 

   WRITE(200) xrs ; WRITE(200) yrs 
   WRITE(200) zs  ; WRITE(200) sm 
   WRITE(200) z0m ; WRITE(200) z0t 
   WRITE(200) uf  ; WRITE(200) prc 
   WRITE(200) cwh ; WRITE(200) qqh 
   WRITE(200) ps  ; WRITE(200) p0 
   WRITE(200) tw  ; WRITE(200) qw 
   WRITE(200) ts  ; WRITE(200) qs 
   WRITE(200) sst ; WRITE(200) ssq 
   WRITE(200) t10 ; WRITE(200) q10 
   WRITE(200) u10 ; WRITE(200) v10 
   do k =  1, kmp ; WRITE(200) uu_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) vv_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) ww_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) tt_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) td_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) qt_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) gh_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) qq_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) cw_(:,:,k) ; enddo 
   do k =  1, kmp ; WRITE(200) rh_(:,:,k) ; enddo 
   
   nvars = 32 ; dxx = dlon ; dyy = dlat 
    imr  = im ; jmr =  jm  ; ims =  im ; jms = (nint(aspect*ims)/2)*2 + 1 
   
!* Refine the grid for low resolutions 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   ELSE !* Refine low resolution - - - - - - - - - - - - - - - - - - 

   im3 = 3*im - 2 ; jm3 = 3*jm - 2 ; k2d = 22 
   Allocate( uu3(im3,jm3,kmp), vv3(im3,jm3,kmp), ww3(im3,jm3,kmp) ) 
   Allocate( tt3(im3,jm3,kmp), td3(im3,jm3,kmp), qt3(im3,jm3,kmp) ) 
   Allocate( gh3(im3,jm3,kmp), qq3(im3,jm3,kmp), cw3(im3,jm3,kmp) ) 
   Allocate( rh3(im3,jm3,kmp), wks(im3,jm3,k2d) ) 
   
   CALL Refine_Grid3x( xrs(1,1), wks(1,1,1),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( yrs(1,1), wks(1,1,2),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  zs(1,1), wks(1,1,3),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  sm(1,1), wks(1,1,4),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( z0m(1,1), wks(1,1,5),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( z0t(1,1), wks(1,1,6),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  uf(1,1), wks(1,1,7),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( prc(1,1), wks(1,1,8),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( cwh(1,1), wks(1,1,9),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( qqh(1,1), wks(1,1,10), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  ps(1,1), wks(1,1,11), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  p0(1,1), wks(1,1,12), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  tw(1,1), wks(1,1,13), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  qw(1,1), wks(1,1,14), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  ts(1,1), wks(1,1,15), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  qs(1,1), wks(1,1,16), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( sst(1,1), wks(1,1,17), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( ssq(1,1), wks(1,1,18), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( t10(1,1), wks(1,1,19), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( q10(1,1), wks(1,1,20), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( u10(1,1), wks(1,1,21), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( v10(1,1), wks(1,1,22), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( uu_, uu3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( vv_, vv3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( ww_, ww3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( tt_, tt3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( td_, td3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( qt_, qt3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( gh_, gh3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( qq_, qq3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( cw_, cw3, undef, im3, jm3, im, jm, kmp )
   CALL Refine_Grid3x( rh_, rh3, undef, im3, jm3, im, jm, kmp )
   
!* Positive-definite constraints 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   wks(:,:,4)  = max( wks(:,:,4),  0. )
   wks(:,:,5)  = max( wks(:,:,5),  0. )
   wks(:,:,6)  = max( wks(:,:,6),  0. )
   wks(:,:,7)  = max( wks(:,:,7),  0. )
   wks(:,:,8)  = max( wks(:,:,8),  0. )
   wks(:,:,9)  = max( wks(:,:,9),  0. )
   wks(:,:,10) = max( wks(:,:,10), 0. )
   wks(:,:,16) = max( wks(:,:,16), 0. )
   wks(:,:,18) = max( wks(:,:,18), 0. )
   wks(:,:,20) = max( wks(:,:,20), 0. )
   qq3(:,:,:)  = max( qq3(:,:,:),  0. )
   cw3(:,:,:)  = max( cw3(:,:,:),  0. )
   rh3(:,:,:)  = min( rh3(:,:,:), 100.)
   td3(:,:,:)  = min( td3(:,:,:), tt3(:,:,:) )
   
!* Output pressure-level data on refined rotated grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   do k = 1, k2d ; WRITE(200) wks(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) uu3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) vv3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) ww3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) tt3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) td3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) qt3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) gh3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) qq3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) cw3(:,:,k) ; enddo 
   do k = 1, kmp ; WRITE(200) rh3(:,:,k) ; enddo 
   
   nvars = k2d + 10 ;  dxx = dlon/3. ; dyy = dlat/3. 
   imr = im3 ; jmr = jm3 ; ims = im3 ; jms = (nint(aspect*ims)/2)*2 + 1 
   
   DeAllocate( uu3, vv3, ww3, tt3, td3, qt3, gh3, qq3, cw3, rh3, wks ) 
   
   ENDIF !* ( refine_rot ) - - - - - - - - - - - - - - - - - - - - - 

!           *     *     *     *     *     *     *     *     *     * 
   
!* GrADS control file for pressure-level data on rotated grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxr1 = .5*(1-imr)*dxx ; xxr_ = dxx !* Lagrangian coord for moving nest 
   yyr1 = .5*(1-jmr)*dyy ; yyr_ = dyy !* and for fixed outer domain (d01) 
   
   WRITE(201,'("                                                      ")') 
   WRITE(201,'(" DSET    press_lev.dat                                ")') 
   WRITE(201,'(" TITLE   Pressure-level data on rotated latlon grid   ")') 
   WRITE(201,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(201,'(" UNDEF",E11.2)') undef 
   WRITE(201,'("                                                      ")') 
   WRITE(201,'(" XDEF", I7,"   LINEAR", 2F11.5 )') imr, xxr1, xxr_ 
   WRITE(201,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jmr, yyr1, yyr_ 
   WRITE(201,'(" ZDEF", I7,"   LEVELS")') kmp 
   write(201,*)  plevel 
   WRITE(201,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(201,'("                                                      ")') 
   WRITE(201,'(" VARS", I3)') nvars
   WRITE(201,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(201,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(201,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(201,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(201,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(201,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(201,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(201,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(201,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(201,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(201,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(201,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(201,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(201,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(201,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(201,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(201,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(201,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(201,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(201,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(201,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(201,'(" u10       0   0    10m rotated x-wind (m/s)          ")') 
   WRITE(201,'(" v10       0   0    10m rotated y-wind (m/s)          ")') 
   WRITE(201,'(" uu  ", I7,"   0    rotated x-wind (m/s)              ")') kmp 
   WRITE(201,'(" vv  ", I7,"   0    rotated y-wind (m/s)              ")') kmp 
   WRITE(201,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') kmp 
   WRITE(201,'(" tt  ", I7,"   0    air temperature (K)               ")') kmp 
   WRITE(201,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') kmp 
   WRITE(201,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') kmp 
   WRITE(201,'(" gh  ", I7,"   0    geopotential height (m)           ")') kmp 
   WRITE(201,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') kmp 
   WRITE(201,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') kmp 
   WRITE(201,'(" rh  ", I7,"   0    relative humidity (%)             ")') kmp 
   WRITE(201,'(" ENDVARS                                              ")') 
   WRITE(201,'("                                                      ")') 
   
!* GrADS control file for pressure-level data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxs1 = .5*(1-ims)*dxx + ctr_lon ; xxs_ = dxx !* d01 west  bdry 
   yys1 = .5*(1-jms)*dyy + ctr_lat ; yys_ = dyy !* d01 south bdry 
   
   WRITE(202,'("                                                      ")') 
   WRITE(202,'(" DSET    press_lev.dat                                ")') 
   WRITE(202,'(" TITLE   Pressure-level data on standard latlon grid  ")') 
   WRITE(202,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(202,'(" UNDEF",E11.2)') undef 
   WRITE(202,'("                                                      ")') 
   WRITE(202,'(" PDEF", I7,  I6,   A10,   4F11.5)') & 
                       imr, jmr, 'ETA.U', ctr_lon, ctr_lat, dxx,  dyy 
   WRITE(202,'(" XDEF", I7,"   LINEAR", 2F11.5 )') ims, xxs1, xxs_ 
   WRITE(202,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jms, yys1, yys_ 
   WRITE(202,'(" ZDEF", I7,"   LEVELS")') kmp 
   write(202,*)  plevel 
   WRITE(202,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(202,'("                                                      ")') 
   WRITE(202,'(" VECTORPAIRS   u10,v10  uu,vv                         ")') 
   WRITE(202,'(" VARS", I3)') nvars
   WRITE(202,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(202,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(202,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(202,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(202,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(202,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(202,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(202,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(202,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(202,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(202,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(202,'(" ps        0   0    surface air   pressure (mb)       ")') 
   WRITE(202,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(202,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(202,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(202,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(202,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(202,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(202,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(202,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(202,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(202,'(" u10       0   0    10m zonal wind comp (m/s)         ")') 
   WRITE(202,'(" v10       0   0    10m meridional wind (m/s)         ")') 
   WRITE(202,'(" uu  ", I7,"   0    zonal wind comp (m/s)             ")') kmp 
   WRITE(202,'(" vv  ", I7,"   0    meridional wind (m/s)             ")') kmp 
   WRITE(202,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') kmp 
   WRITE(202,'(" tt  ", I7,"   0    air temperature (K)               ")') kmp 
   WRITE(202,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') kmp 
   WRITE(202,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') kmp 
   WRITE(202,'(" gh  ", I7,"   0    geopotential height (m)           ")') kmp 
   WRITE(202,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') kmp 
   WRITE(202,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') kmp 
   WRITE(202,'(" rh  ", I7,"   0    relative humidity (%)             ")') kmp 
   WRITE(202,'(" ENDVARS                                              ")') 
   WRITE(202,'("                                                      ")') 
   
   CLOSE(200) ; CLOSE(201) ; CLOSE(202) 
   
   ENDIF !* ( press_level ) -------------------------------------------- 

   IF ( press_vars ) THEN 
   DeAllocate( uu_, vv_, ww_, tt_, td_, qt_, gh_, qq_, cw_, rh_ ) 
   ENDIF 
   
!************************************************************! 
!  2b. 50mb-pressure-level variables on standard latlon grid ! 
!____________________________________________________________! 

   IF ( basic_vars ) THEN !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

!* Allocate work arrays for basic pressure levels (dp = 50mb) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate( uu_(im,jm,kmb), vv_(im,jm,kmb), ww_(im,jm,kmb) ) 
   Allocate( tt_(im,jm,kmb), td_(im,jm,kmb), qt_(im,jm,kmb) ) 
   Allocate( gh_(im,jm,kmb), qq_(im,jm,kmb), cw_(im,jm,kmb) ) 
   Allocate( rh_(im,jm,kmb), uus(im,jm,kmb), vvs(im,jm,kmb) )
   Allocate( p02(im,jm), td2(im,jm), wk(im,jm) ) 
   
!* Interpolate from sigma to basic pressure levels (mb) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   uu_(:,:,:) = undef ; vv_(:,:,:) = undef ; ww_(:,:,:) = undef 
   tt_(:,:,:) = undef ; td_(:,:,:) = undef ; qt_(:,:,:) = undef 
   gh_(:,:,:) = undef ; qq_(:,:,:) = undef ; cw_(:,:,:) = undef 
   rh_(:,:,:) = undef ; uus(:,:,:) = undef ; vvs(:,:,:) = undef 
   
   DO j = 1, jm 
   DO i = 1, im ; k0 = 1 
      DO m = 1, kmb ; xxx = pbasic(m) 
      if ( pp(i,j,1) >= xxx .AND. xxx > pp(i,j,km) ) then 
         DO k = k0, km-1 ; IF ( xxx > pp(i,j,k+1) ) EXIT ; ENDDO 
         aaa = (xxx-pp(i,j,k+1))/(pp(i,j,k)-pp(i,j,k+1)) ; bbb = 1 - aaa 
         uu_(i,j,m) = aaa*uu(i,j,k) + bbb*uu(i,j,k+1) 
         vv_(i,j,m) = aaa*vv(i,j,k) + bbb*vv(i,j,k+1) 
         ww_(i,j,m) = aaa*ww(i,j,k) + bbb*ww(i,j,k+1) 
         tt_(i,j,m) = aaa*tt(i,j,k) + bbb*tt(i,j,k+1) 
         td_(i,j,m) = aaa*td(i,j,k) + bbb*td(i,j,k+1) 
         qt_(i,j,m) = aaa*qt(i,j,k) + bbb*qt(i,j,k+1) 
         gh_(i,j,m) = aaa*gh(i,j,k) + bbb*gh(i,j,k+1) 
         qq_(i,j,m) = aaa*qq(i,j,k) + bbb*qq(i,j,k+1) 
         cw_(i,j,m) = aaa*cw(i,j,k) + bbb*cw(i,j,k+1) 
         rh_(i,j,m) = aaa*rh(i,j,k) + bbb*rh(i,j,k+1) 
         k0 = k 
      endif 
      ENDDO 
   ENDDO 
   ENDDO 

!* 10m winds in Earth coordinates (urs, vrs) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   urs(:,:) = wcos(:,:)*u10(:,:) - wsin(:,:)*v10(:,:) 
   vrs(:,:) = wsin(:,:)*u10(:,:) + wcos(:,:)*v10(:,:) 
   
!* 3D winds in Earth coordinates (uus, vvs) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO k = 1, kmb ; DO j = 1, jm ; DO i = 1, im  
   if ( max(uu_(i,j,k),vv_(i,j,k)) < undef ) then 
      uus(i,j,k) = wcos(i,j)*uu_(i,j,k) - wsin(i,j)*vv_(i,j,k) 
      vvs(i,j,k) = wsin(i,j)*uu_(i,j,k) + wcos(i,j)*vv_(i,j,k) 
   endif 
   ENDDO ; ENDDO ; ENDDO 
   
!* 2m Dewpoint temperature (td2) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO j = 1, jm ; DO i = 1, im 
      zzz = gh(i,j,1)-zs(i,j) ; aaa = (zzz-2)/zzz 
      p02(i,j) =  aaa*ps(i,j) + (1-aaa)*pp(i,j,1)
   ENDDO ; ENDDO  !* 2m air pressure *! 
   CALL Dewpoint_RH( p02(1,1), q02(1,1), t02(1,1), td2(1,1), wk(1,1), im,jm,1 ) 

!* Delete bad boundary for GFDL surface scheme 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   if ( gfdl_surface .AND. .not. debug_data ) then !* -------------- 
   DO j =  1, 1+jbdcut ; t02(:,j) = undef ; td2(:,j) = undef ; ENDDO 
   DO j = jm-jbdcut,jm ; t02(:,j) = undef ; td2(:,j) = undef ; ENDDO 
   DO i =  1, 1+ibdcut ; t02(i,:) = undef ; td2(i,:) = undef ; ENDDO 
   DO i = im-ibdcut,im ; t02(i,:) = undef ; td2(i,:) = undef ; ENDDO 
   endif !* ( .not. debug_data ) ----------------------------------- 

   ENDIF !* ( basic_vars ) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   IF ( basic_level ) THEN !* ========================================== 
   
!* Dimensions of default unrefined standard latlon grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxx = d02cut ; IF ( domain_num == 1 ) xxx = d01cut 
   imx = (nint(xxx/dlon)/2)*2+1 ; jmx = (nint(aspect*imx)/2)*2+1 
   
   IF ( .not. refine_std ) THEN !* - - - - - - - - - - - - - - - - - 

!* Dimensions of unrefined rotated & standard grids 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   dxx = dlon ; dyy = dlat ; k2d = 24 
   imr = im ; jmr = jm ; ims = imx ; jms = jmx 
   
!* Allocate work arrays for unrefined rotated grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate( xxr(imr,jmr), yyr(imr,jmr), xxs(imr,jmr), yys(imr,jmr) ) 
   Allocate( uu3(imr,jmr,kmb), vv3(imr,jmr,kmb), ww3(imr,jmr,kmb) ) 
   Allocate( tt3(imr,jmr,kmb), td3(imr,jmr,kmb), qt3(imr,jmr,kmb) ) 
   Allocate( gh3(imr,jmr,kmb), qq3(imr,jmr,kmb), cw3(imr,jmr,kmb) ) 
   Allocate( rh3(imr,jmr,kmb), pp3(imr,jmr,k2d) ) 
   
!* Copy unrefined rotated-grid coord (degree) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxr(:,:) = xrr(:,:) ; yyr(:,:) = yrr(:,:) 
   xxs(:,:) = xrs(:,:) ; yys(:,:) = yrs(:,:) 
   
!* Copy unrefined rotated-grid variables 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   pp3(:,:, 1) = xxs(:,:)  ; pp3(:,:, 2) = yys(:,:) 
   pp3(:,:, 3) =  zs(:,:)  ; pp3(:,:, 4) =  sm(:,:) 
   pp3(:,:, 5) = z0m(:,:)  ; pp3(:,:, 6) = z0t(:,:) 
   pp3(:,:, 7) =  uf(:,:)  ; pp3(:,:, 8) = prc(:,:) 
   pp3(:,:, 9) = cwh(:,:)  ; pp3(:,:,10) = qqh(:,:) 
   pp3(:,:,11) =  ps(:,:)  ; pp3(:,:,12) =  p0(:,:) 
   pp3(:,:,13) =  tw(:,:)  ; pp3(:,:,14) =  qw(:,:) 
   pp3(:,:,15) =  ts(:,:)  ; pp3(:,:,16) =  qs(:,:) 
   pp3(:,:,17) = sst(:,:)  ; pp3(:,:,18) = ssq(:,:) 
   pp3(:,:,19) = t02(:,:)  ; pp3(:,:,20) = td2(:,:) 
   pp3(:,:,21) = t10(:,:)  ; pp3(:,:,22) = q10(:,:) 
   pp3(:,:,23) = urs(:,:)  ; pp3(:,:,24) = vrs(:,:) !* std 10m wind 
   uu3(:,:,:) = uus(:,:,:) ; vv3(:,:,:) = vvs(:,:,:) ; ww3(:,:,:) = ww_(:,:,:) 
   tt3(:,:,:) = tt_(:,:,:) ; td3(:,:,:) = td_(:,:,:) ; qt3(:,:,:) = qt_(:,:,:) 
   gh3(:,:,:) = gh_(:,:,:) ; qq3(:,:,:) = qq_(:,:,:) ; cw3(:,:,:) = cw_(:,:,:) 
   rh3(:,:,:) = rh_(:,:,:) 
   
   ELSE !* Refine low resolution - - - - - - - - - - - - - - - - - - 

!* Dimensions of refined rotated & standard grids 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   dxx = dlon/3. ; dyy = dlat/3. ; k2d = 24
   imr = 3*im-2  ; jmr = 3*jm-2  ; ims = 3*imx-2 ; jms = 3*jmx-2 
   
!* Allocate work arrays for refined rotated grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate( xxr(imr,jmr), yyr(imr,jmr), xxs(imr,jmr), yys(imr,jmr) ) 
   Allocate( uu3(imr,jmr,kmb), vv3(imr,jmr,kmb), ww3(imr,jmr,kmb) ) 
   Allocate( tt3(imr,jmr,kmb), td3(imr,jmr,kmb), qt3(imr,jmr,kmb) ) 
   Allocate( gh3(imr,jmr,kmb), qq3(imr,jmr,kmb), cw3(imr,jmr,kmb) ) 
   Allocate( rh3(imr,jmr,kmb), pp3(imr,jmr,k2d) ) 
   
!* Refined rotated-grid coordinates (degree) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO i = 1, imr ; xxr(i,:) = xrr(1,1) + (i-1)*dxx ; ENDDO 
   DO j = 1, jmr ; yyr(:,j) = yrr(1,1) + (j-1)*dyy ; ENDDO 
   Call Convert_LatLon( 2, imr, jmr, xxs, yys, xxr, yyr, ctr_lon, ctr_lat ) 

!* Define refined rotated-grid variables 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   pp3(:,:, 1) = xxs(:,:)  ; pp3(:,:, 2) = yys(:,:) 
   CALL Refine_Grid3x(  zs(1,1), pp3(1,1, 3), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  sm(1,1), pp3(1,1, 4), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( z0m(1,1), pp3(1,1, 5), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( z0t(1,1), pp3(1,1, 6), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  uf(1,1), pp3(1,1, 7), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( prc(1,1), pp3(1,1, 8), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( cwh(1,1), pp3(1,1, 9), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( qqh(1,1), pp3(1,1,10), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  ps(1,1), pp3(1,1,11), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  p0(1,1), pp3(1,1,12), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  tw(1,1), pp3(1,1,13), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  qw(1,1), pp3(1,1,14), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  ts(1,1), pp3(1,1,15), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x(  qs(1,1), pp3(1,1,16), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( sst(1,1), pp3(1,1,17), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( ssq(1,1), pp3(1,1,18), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( t02(1,1), pp3(1,1,19), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( td2(1,1), pp3(1,1,20), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( t10(1,1), pp3(1,1,21), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( q10(1,1), pp3(1,1,22), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( urs(1,1), pp3(1,1,23), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( vrs(1,1), pp3(1,1,24), undef, imr, jmr, im, jm, 1 )
   CALL Refine_Grid3x( uus, uu3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( vvs, vv3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( ww_, ww3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( tt_, tt3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( td_, td3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( qt_, qt3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( gh_, gh3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( qq_, qq3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( cw_, cw3, undef, imr, jmr, im, jm, kmb )
   CALL Refine_Grid3x( rh_, rh3, undef, imr, jmr, im, jm, kmb )
   
!* Positive-definite constraints 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   pp3(:,:,4)  = max( pp3(:,:,4),  0. )
   pp3(:,:,5)  = max( pp3(:,:,5),  0. )
   pp3(:,:,6)  = max( pp3(:,:,6),  0. )
   pp3(:,:,7)  = max( pp3(:,:,7),  0. )
   pp3(:,:,8)  = max( pp3(:,:,8),  0. )
   pp3(:,:,9)  = max( pp3(:,:,9),  0. )
   pp3(:,:,10) = max( pp3(:,:,10), 0. )
   pp3(:,:,16) = max( pp3(:,:,16), 0. )
   pp3(:,:,18) = max( pp3(:,:,18), 0. )
   pp3(:,:,22) = max( pp3(:,:,22), 0. )
   qq3(:,:,:)  = max( qq3(:,:,:),  0. )
   cw3(:,:,:)  = max( cw3(:,:,:),  0. )
   rh3(:,:,:)  = min( rh3(:,:,:), 100.)
   td3(:,:,:)  = min( td3(:,:,:), tt3(:,:,:) )
   pp3(:,:,20) = min( pp3(:,:,20),pp3(:,:,19) )
   
   ENDIF !* ( refine_std ) - - - - - - - - - - - - - - - - - - - - - 

!* Modify standard output grid for requested resolution 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxx = d02cut ; IF ( domain_num == 1 ) xxx = d01cut 
   dxo = dxx ; IF ( basic_dlon > 0. ) dxo = basic_dlon ; dyo = dxo 
   ims = (nint(xxx/dxo)/2)*2+1 ; jms = (nint(aspect*ims)/2)*2+1 
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* Allocate work arrays for standard latlon grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate(  xsr(ims,jms), ysr(ims,jms), xss(ims,jms), yss(ims,jms) ) 
   Allocate( uu3_(ims,jms,kmb), vv3_(ims,jms,kmb), ww3_(ims,jms,kmb) ) 
   Allocate( tt3_(ims,jms,kmb), td3_(ims,jms,kmb), qt3_(ims,jms,kmb) ) 
   Allocate( gh3_(ims,jms,kmb), qq3_(ims,jms,kmb), cw3_(ims,jms,kmb) ) 
   Allocate( rh3_(ims,jms,kmb), pp3_(ims,jms,k2d) ) 
   
!* Standard output grid coordinates (degree) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   i0 = (im+1)/2 ; j0 = (jm+1)/2 !* output domain center = model domain center 
   IF (mbasic_cen == 1) THEN ; i0 = iloc ; j0 = jloc ; ENDIF !* storm-centered 
   xss(1,1) = ctr_lon + ( nint((xrs(i0,j0)-ctr_lon)/dxo) - (ims-1)/2 )*dxo
   yss(1,1) = ctr_lat + ( nint((yrs(i0,j0)-ctr_lat)/dyo) - (jms-1)/2 )*dyo
   DO i = 1, ims ; xss(i,:) = xss(1,1) + (i-1)*dxo ; ENDDO 
   DO j = 1, jms ; yss(:,j) = yss(1,1) + (j-1)*dyo ; ENDDO 
   
   Call Convert_LatLon( 1, ims, jms, xss, yss, xsr, ysr, ctr_lon, ctr_lat ) 
   
!* Interpolate refined data to standard output grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   ibd = 4 ; IF ( refine_std ) ibd = 12 !* bad boundary points 
   xrr1 = xxr(1,1) ; xrr2 = xxr(1+ibd,1) ; xrr8 = xxr(imr-ibd,1) 
   yrr1 = yyr(1,1) ; yrr2 = yyr(1,1+ibd) ; yrr8 = yyr(1,jmr-ibd) 
   
   uu3_(:,:,:) = undef ; vv3_(:,:,:) = undef ; ww3_(:,:,:) = undef 
   tt3_(:,:,:) = undef ; td3_(:,:,:) = undef ; qt3_(:,:,:) = undef 
   gh3_(:,:,:) = undef ; qq3_(:,:,:) = undef ; cw3_(:,:,:) = undef  
   rh3_(:,:,:) = undef ; pp3_(:,:,:) = undef 
   
   DO j = 1, jms ; DO i = 1, ims !* -------------------------------> 
   
      xxx = xsr(i,j) ;  yyy = ysr(i,j) 
   IF ( yrr2 <= yyy .AND. yyy < yrr8 ) THEN 
   if ( xrr2 <= xxx .AND. xxx < xrr8 ) then !* - - - - - - - - - - - 
   
      m2  =  1 + int((xxx-xrr1)/dxx) ;    m1  = m2-1 ; m4 = m2+2 
      n2  =  1 + int((yyy-yrr1)/dyy) ;    n1  = n2-1 ; n4 = n2+2 
     aaa  = ( xxx - xxr(m2,n2) )/dxx ;   bbb  = ( yyy - yyr(m2,n2) )/dyy 
   wtx(1) = -(aaa)*(aaa-1)*(aaa-2)/6 ; wtx(2) = (aaa+1)*(aaa-1)*(aaa-2)/2 
   wtx(3) = -(aaa+1)*(aaa)*(aaa-2)/2 ; wtx(4) = (aaa+1)*(aaa)*(aaa-1)/6 
   wty(1) = -(bbb)*(bbb-1)*(bbb-2)/6 ; wty(2) = (bbb+1)*(bbb-1)*(bbb-2)/2 
   wty(3) = -(bbb+1)*(bbb)*(bbb-2)/2 ; wty(4) = (bbb+1)*(bbb)*(bbb-1)/6 
   
   DO k = 1, k2d 
   ff = pp3(m1:m4,n1:n4,k); CALL Cubic_Interpo( pp3_(i,j,k),ff,wtx,wty,undef ) 
   ENDDO 
   
   DO k = 1, kmb 
   ff = uu3(m1:m4,n1:n4,k); CALL Cubic_Interpo( uu3_(i,j,k),ff,wtx,wty,undef ) 
   ff = vv3(m1:m4,n1:n4,k); CALL Cubic_Interpo( vv3_(i,j,k),ff,wtx,wty,undef ) 
   ff = ww3(m1:m4,n1:n4,k); CALL Cubic_Interpo( ww3_(i,j,k),ff,wtx,wty,undef ) 
   ff = tt3(m1:m4,n1:n4,k); CALL Cubic_Interpo( tt3_(i,j,k),ff,wtx,wty,undef ) 
   ff = td3(m1:m4,n1:n4,k); CALL Cubic_Interpo( td3_(i,j,k),ff,wtx,wty,undef ) 
   ff = qt3(m1:m4,n1:n4,k); CALL Cubic_Interpo( qt3_(i,j,k),ff,wtx,wty,undef ) 
   ff = gh3(m1:m4,n1:n4,k); CALL Cubic_Interpo( gh3_(i,j,k),ff,wtx,wty,undef ) 
   ff = qq3(m1:m4,n1:n4,k); CALL Cubic_Interpo( qq3_(i,j,k),ff,wtx,wty,undef ) 
   ff = cw3(m1:m4,n1:n4,k); CALL Cubic_Interpo( cw3_(i,j,k),ff,wtx,wty,undef ) 
   ff = rh3(m1:m4,n1:n4,k); CALL Cubic_Interpo( rh3_(i,j,k),ff,wtx,wty,undef ) 
   ENDDO 
   
   endif  !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   ENDIF 
   
   ENDDO ; ENDDO !* <----------------------------------------------- 

!* Positive-definite constraints 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   pp3_(:,:,4)  = max( pp3_(:,:,4),  0. )
   pp3_(:,:,5)  = max( pp3_(:,:,5),  0. )
   pp3_(:,:,6)  = max( pp3_(:,:,6),  0. )
   pp3_(:,:,7)  = max( pp3_(:,:,7),  0. )
   pp3_(:,:,8)  = max( pp3_(:,:,8),  0. )
   pp3_(:,:,9)  = max( pp3_(:,:,9),  0. )
   pp3_(:,:,10) = max( pp3_(:,:,10), 0. )
   pp3_(:,:,16) = max( pp3_(:,:,16), 0. )
   pp3_(:,:,18) = max( pp3_(:,:,18), 0. )
   pp3_(:,:,22) = max( pp3_(:,:,22), 0. )
   qq3_(:,:,:)  = max( qq3_(:,:,:),  0. )
   cw3_(:,:,:)  = max( cw3_(:,:,:),  0. )
   rh3_(:,:,:)  = min( rh3_(:,:,:), 100.)
   td3_(:,:,:)  = min( td3_(:,:,:), tt3_(:,:,:) )
   pp3_(:,:,20) = min( pp3_(:,:,20),pp3_(:,:,19) )
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* Output basic-level data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   do k = 1, k2d ; WRITE(252) pp3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) uu3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) vv3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) ww3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) tt3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) td3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) qt3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) gh3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) qq3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) cw3_(:,:,k) ; enddo 
   do k = 1, kmb ; WRITE(252) rh3_(:,:,k) ; enddo 
   nvars = k2d + 10 

!* GrADS control file for basic-level data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxs_ = dxo ; yys_ = dyo 
   IF ( domain_num == 1 .AND. mbasic_cen /= 1 ) THEN 
      xxs1 = xss(1,1) ; yys1 = yss(1,1) 
   ELSE !* Lagrangian coordinates for moving output domain 
      xxs1 = .5*(1-ims)*dxo ; yys1 = .5*(1-jms)*dyo 
   ENDIF 
   
   WRITE(251,'("                                                      ")') 
   WRITE(251,'(" DSET    basic_std.dat                                ")') 
   WRITE(251,'(" TITLE   Basic p-level data on standard latlon grid   ")') 
   WRITE(251,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(251,'(" UNDEF",E11.2)') undef 
   WRITE(251,'("                                                      ")') 
   WRITE(251,'(" XDEF", I7,"   LINEAR", 2F11.5 )') ims, xxs1, xxs_ 
   WRITE(251,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jms, yys1, yys_ 
   WRITE(251,'(" ZDEF", I7,"   LEVELS")') kmb 
   WRITE(251,*)  pbasic 
   WRITE(251,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(251,'("                                                      ")') 
   WRITE(251,'(" VARS", I3)') nvars
   WRITE(251,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(251,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(251,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(251,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(251,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(251,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(251,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(251,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(251,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(251,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(251,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(251,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(251,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(251,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(251,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(251,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(251,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(251,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(251,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(251,'(" t02       0   0    2-m air temperature (K)           ")') 
   WRITE(251,'(" td2       0   0    2-m dewpoint temperature (K)      ")') 
   WRITE(251,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(251,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(251,'(" u10       0   0    10m zonal wind comp (m/s)         ")') 
   WRITE(251,'(" v10       0   0    10m meridional wind (m/s)         ")') 
   WRITE(251,'(" uu  ", I7,"   0    zonal wind comp (m/s)             ")') kmb 
   WRITE(251,'(" vv  ", I7,"   0    meridional wind (m/s)             ")') kmb 
   WRITE(251,'(" ww  ", I7,"   0    vertical   wind (m/s)             ")') kmb 
   WRITE(251,'(" tt  ", I7,"   0    air temperature (K)               ")') kmb 
   WRITE(251,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') kmb 
   WRITE(251,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') kmb 
   WRITE(251,'(" gh  ", I7,"   0    geopotential height (m)           ")') kmb 
   WRITE(251,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') kmb 
   WRITE(251,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') kmb 
   WRITE(251,'(" rh  ", I7,"   0    relative humidity (%)             ")') kmb 
   WRITE(251,'(" ENDVARS                                              ")') 
   WRITE(251,'("                                                      ")') 
   
!* Dimensions of basic-level data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   WRITE(254,'(I10,I5,I3,I3,1X,2I5,2F9.4,2F10.4,4F8.4)')    & 
   start_date, ifcst_hour, ifcst_min, domain_num, ims, jms, & 
   yss(1,jms), yss(1,1), xss(1,1), xss(ims,1), dxo, dyo, dlon, dlat 

!* Message file for dimensions of basic-level data 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   write(253,*) '                                                         '
   write(253,*) 'Column  Description of the Data                   Example'
   write(253,*) '------  -------------------------------------  ----------'
   write(253,*) '   1    Starting date:                         2008090800'
   write(253,*) '   2    Forecast hour:                                120'
   write(253,*) '   3    Forecast minute:                               30'
   write(253,*) '   4    Domain number:                                  1'
   write(253,*) '   5    Number of grid points in X-direction:         997'
   write(253,*) '   6    Number of grid points in Y-direction:         721'
   write(253,*) '   7    Northern  most  latitude   (degree):      40.2000'
   write(253,*) '   8    Southern  most  latitude   (degree):      -3.0000'
   write(253,*) '   9    Western   most  longitude  (degree):    -110.0000'
   write(253,*) '  10    Eastern   most  longitude  (degree):     -50.0000'
   write(253,*) '  11    Output resolution in X-dir (degree):       0.0600'
   write(253,*) '  12    Output resolution in Y-dir (degree):       0.0600'
   write(253,*) '  13    Model  resolution in X-dir (degree):       0.1800'
   write(253,*) '  14    Model  resolution in Y-dir (degree):       0.1800'
   write(253,*) '                                                         '
   
   CLOSE(251) ; CLOSE(252) ; CLOSE(253) ; CLOSE(254) 
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* Output basic 2D data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   do k = 1, k2d ; WRITE(262) pp3_(:,:,k) ; enddo 
   nvars = k2d 

!* GrADS control file for basic 2D data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxs_ = dxo ; yys_ = dyo 
   IF ( domain_num == 1 ) THEN 
      xxs1 = xss(1,1) ; yys1 = yss(1,1) 
   ELSE !* Lagrangian coordinates for moving nest 
      xxs1 = .5*(1-ims)*dxo ; yys1 = .5*(1-jms)*dyo 
   ENDIF 
   
   WRITE(261,'("                                                      ")') 
   WRITE(261,'(" DSET    bas2d_std.dat                                ")') 
   WRITE(261,'(" TITLE   Surface-level data on standard latlon grid   ")') 
   WRITE(261,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(261,'(" UNDEF",E11.2)') undef 
   WRITE(261,'("                                                      ")') 
   WRITE(261,'(" XDEF", I7,"   LINEAR", 2F11.5 )') ims, xxs1, xxs_ 
   WRITE(261,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jms, yys1, yys_ 
   WRITE(261,'(" ZDEF", I7,"   LINEAR          1          1 ")')   1 
   WRITE(261,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(261,'("                                                      ")') 
   WRITE(261,'(" VARS", I3)') nvars
   WRITE(261,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(261,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(261,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(261,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(261,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(261,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(261,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(261,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(261,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(261,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(261,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(261,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(261,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(261,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(261,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(261,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(261,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(261,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(261,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(261,'(" t02       0   0    2-m air temperature (K)           ")') 
   WRITE(261,'(" td2       0   0    2-m dewpoint temperature (K)      ")') 
   WRITE(261,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(261,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(261,'(" u10       0   0    10m zonal wind comp (m/s)         ")') 
   WRITE(261,'(" v10       0   0    10m meridional wind (m/s)         ")') 
   WRITE(261,'(" ENDVARS                                              ")') 
   WRITE(261,'("                                                      ")') 
   
   CLOSE(261) ; CLOSE(262) 
   
   DeAllocate( uu3 , vv3 , ww3 , tt3 , td3 , qt3  ) 
   DeAllocate( gh3 , qq3 , cw3 , rh3 , pp3 , xxr, yyr, xxs, yys ) 
   DeAllocate( uu3_, vv3_, ww3_, tt3_, td3_, qt3_ ) 
   DeAllocate( gh3_, qq3_, cw3_, rh3_, pp3_, xsr, ysr, xss, yss ) 
   
   ENDIF !* ( basic_level ) ============================================ 
   
   IF ( basic_vars ) THEN 
   DeAllocate( uu_, vv_, ww_, tt_, td_, qt_, gh_, qq_, cw_, rh_ ) 
   DeAllocate( uus, vvs, p02, td2, wk ) 
   ENDIF 
   
!*****************************************************************! 
!  3a. Height-level variables on rotated latlon grid (dp = ~25mb) ! 
!_________________________________________________________________! 

   IF ( height_vars ) THEN !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

!* Allocate work arrays for height levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   Allocate( uu_(im,jm,kmz), vv_(im,jm,kmz), ww_(im,jm,kmz) ) 
   Allocate( tt_(im,jm,kmz), td_(im,jm,kmz), qt_(im,jm,kmz) ) 
   Allocate( pp_(im,jm,kmz), qq_(im,jm,kmz), cw_(im,jm,kmz) ) 
   Allocate( ux_(im,jm,kmz), vx_(im,jm,kmz), rh_(im,jm,kmz) ) 
   
   uu_(:,:,:) = undef ; vv_(:,:,:) = undef ; ww_(:,:,:) = undef 
   tt_(:,:,:) = undef ; td_(:,:,:) = undef ; qt_(:,:,:) = undef 
   pp_(:,:,:) = undef ; qq_(:,:,:) = undef ; cw_(:,:,:) = undef 
   ux_(:,:,:) = undef ; vx_(:,:,:) = undef ; rh_(:,:,:) = undef 

!* Vertical interpolation to height levels (m) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO j = 1, jm 
   DO i = 1, im ; k0 = 1 
      DO m = 1, kmz ; xxx = zlevel(m) 
      if ( gh(i,j,1) <= xxx .AND. xxx < gh(i,j,km) ) then 
         DO k = k0, km-1 ; IF ( xxx < gh(i,j,k+1) ) EXIT ; ENDDO 
         aaa = (gh(i,j,k+1)-xxx)/(gh(i,j,k+1)-gh(i,j,k)) ; bbb = 1 - aaa 
         uu_(i,j,m) = aaa*uu(i,j,k) + bbb*uu(i,j,k+1) 
         vv_(i,j,m) = aaa*vv(i,j,k) + bbb*vv(i,j,k+1) 
         ww_(i,j,m) = aaa*ww(i,j,k) + bbb*ww(i,j,k+1) 
         tt_(i,j,m) = aaa*tt(i,j,k) + bbb*tt(i,j,k+1) 
         td_(i,j,m) = aaa*td(i,j,k) + bbb*td(i,j,k+1) 
         qt_(i,j,m) = aaa*qt(i,j,k) + bbb*qt(i,j,k+1) 
         pp_(i,j,m) = aaa*pp(i,j,k) + bbb*pp(i,j,k+1) 
         qq_(i,j,m) = aaa*qq(i,j,k) + bbb*qq(i,j,k+1) 
         cw_(i,j,m) = aaa*cw(i,j,k) + bbb*cw(i,j,k+1) 
         rh_(i,j,m) = aaa*rh(i,j,k) + bbb*rh(i,j,k+1) 
         k0 = k 
      endif 
      ENDDO 
   ENDDO 
   ENDDO 

!* Divergence & vorticity for cylindrical grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO k = 1, kmz ; DO j = 2, jm-1 ; DO i = 2, im-1 
      aaa = max( uu_(i+1,j,k), uu_(i-1,j,k), vv_(i,j+1,k), vv_(i,j-1,k) )  
      IF ( aaa < undef ) THEN 
         ux_(i,j,k) = .5*( uu_(i+1,j,k) - uu_(i-1,j,k) )/xrlen(j) & 
                    + .5*( vv_(i,j+1,k) - vv_(i,j-1,k) )/yrlen 
      ENDIF 
      bbb = max( vv_(i+1,j,k), vv_(i-1,j,k), uu_(i,j-1,k), uu_(i,j+1,k) ) 
      IF ( bbb < undef ) THEN 
         vx_(i,j,k) = .5*( vv_(i+1,j,k) - vv_(i-1,j,k) )/xrlen(j) & 
                    + .5*( uu_(i,j-1,k) - uu_(i,j+1,k) )/yrlen 
      ENDIF 
   ENDDO ; ENDDO ; ENDDO 

   ENDIF !* ( height_vars ) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

!           *     *     *     *     *     *     *     *     *     * 
   
   IF ( height_level ) THEN !* ----------------------------------------- 

!* Output variables on height levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF ( .not. refine_rot ) THEN !* - - - - - - - - - - - - - - - - - 

   WRITE(300) xrs ; WRITE(300) yrs 
   WRITE(300) zs  ; WRITE(300) sm 
   WRITE(300) z0m ; WRITE(300) z0t 
   WRITE(300) uf  ; WRITE(300) prc 
   WRITE(300) cwh ; WRITE(300) qqh 
   WRITE(300) ps  ; WRITE(300) p0 
   WRITE(300) tw  ; WRITE(300) qw 
   WRITE(300) ts  ; WRITE(300) qs 
   WRITE(300) sst ; WRITE(300) ssq 
   WRITE(300) t10 ; WRITE(300) q10 
   WRITE(300) u10 ; WRITE(300) v10 
   do k =  1, kmz ; WRITE(300) uu_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) vv_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) ww_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) tt_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) td_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) qt_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) pp_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) qq_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) cw_(:,:,k) ; enddo 
   do k =  1, kmz ; WRITE(300) rh_(:,:,k) ; enddo 
   
   nvars = 32 ; dxx = dlon ; dyy = dlat 
    imr  = im ; jmr =  jm  ; ims =  im ; jms = (nint(aspect*ims)/2)*2 + 1 
   
!* Refine the grid for low resolutions 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   ELSE !* Refine low resolution - - - - - - - - - - - - - - - - - - 

   im3 = 3*im - 2 ; jm3 = 3*jm - 2 ; k2d = 22 
   Allocate( uu3(im3,jm3,kmz), vv3(im3,jm3,kmz), ww3(im3,jm3,kmz) ) 
   Allocate( tt3(im3,jm3,kmz), td3(im3,jm3,kmz), qt3(im3,jm3,kmz) ) 
   Allocate( pp3(im3,jm3,kmz), qq3(im3,jm3,kmz), cw3(im3,jm3,kmz) ) 
   Allocate( rh3(im3,jm3,kmz), wks(im3,jm3,k2d) ) 
   
   CALL Refine_Grid3x( xrs(1,1), wks(1,1,1),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( yrs(1,1), wks(1,1,2),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  zs(1,1), wks(1,1,3),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  sm(1,1), wks(1,1,4),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( z0m(1,1), wks(1,1,5),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( z0t(1,1), wks(1,1,6),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  uf(1,1), wks(1,1,7),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( prc(1,1), wks(1,1,8),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( cwh(1,1), wks(1,1,9),  undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( qqh(1,1), wks(1,1,10), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  ps(1,1), wks(1,1,11), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  p0(1,1), wks(1,1,12), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  tw(1,1), wks(1,1,13), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  qw(1,1), wks(1,1,14), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  ts(1,1), wks(1,1,15), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x(  qs(1,1), wks(1,1,16), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( sst(1,1), wks(1,1,17), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( ssq(1,1), wks(1,1,18), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( t10(1,1), wks(1,1,19), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( q10(1,1), wks(1,1,20), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( u10(1,1), wks(1,1,21), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( v10(1,1), wks(1,1,22), undef, im3, jm3, im, jm, 1 )
   CALL Refine_Grid3x( uu_, uu3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( vv_, vv3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( ww_, ww3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( tt_, tt3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( td_, td3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( qt_, qt3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( pp_, pp3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( qq_, qq3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( cw_, cw3, undef, im3, jm3, im, jm, kmz )
   CALL Refine_Grid3x( rh_, rh3, undef, im3, jm3, im, jm, kmz )
   
!* Positive-definite constraints 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   wks(:,:,4)  = max( wks(:,:,4),  0. )
   wks(:,:,5)  = max( wks(:,:,5),  0. )
   wks(:,:,6)  = max( wks(:,:,6),  0. )
   wks(:,:,7)  = max( wks(:,:,7),  0. )
   wks(:,:,8)  = max( wks(:,:,8),  0. )
   wks(:,:,9)  = max( wks(:,:,9),  0. )
   wks(:,:,10) = max( wks(:,:,10), 0. )
   wks(:,:,16) = max( wks(:,:,16), 0. )
   wks(:,:,18) = max( wks(:,:,18), 0. )
   wks(:,:,20) = max( wks(:,:,20), 0. )
   qq3(:,:,:)  = max( qq3(:,:,:),  0. )
   cw3(:,:,:)  = max( cw3(:,:,:),  0. )
   rh3(:,:,:)  = min( rh3(:,:,:), 100.)
   td3(:,:,:)  = min( td3(:,:,:), tt3(:,:,:) )
   
   do k = 1, k2d ; WRITE(300) wks(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) uu3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) vv3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) ww3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) tt3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) td3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) qt3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) pp3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) qq3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) cw3(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(300) rh3(:,:,k) ; enddo 
   
   nvars = k2d + 10 ;  dxx = dlon/3. ; dyy = dlat/3. 
   imr = im3 ; jmr = jm3 ; ims = im3 ; jms = (nint(aspect*ims)/2)*2 + 1 
   
   DeAllocate( uu3, vv3, ww3, tt3, td3, qt3, pp3, qq3, cw3, rh3, wks ) 
   
   ENDIF !* ( refine_rot ) - - - - - - - - - - - - - - - - - - - - - 

!           *     *     *     *     *     *     *     *     *     * 
   
!* GrADS control file for height-level data on rotated grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxr1 = .5*(1-imr)*dxx ; xxr_ = dxx !* Lagrangian coord for moving nest 
   yyr1 = .5*(1-jmr)*dyy ; yyr_ = dyy !* and for fixed outer domain (d01) 
   
   WRITE(301,'("                                                      ")') 
   WRITE(301,'(" DSET    height_lev.dat                               ")') 
   WRITE(301,'(" TITLE   Height-level data on rotated latlon grid     ")') 
   WRITE(301,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(301,'(" UNDEF",E11.2)') undef 
   WRITE(301,'("                                                      ")') 
   WRITE(301,'(" XDEF", I7,"   LINEAR", 2F11.5 )') imr, xxr1, xxr_ 
   WRITE(301,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jmr, yyr1, yyr_ 
   WRITE(301,'(" ZDEF", I7,"   LEVELS")') kmz 
   WRITE(301,*)  zlevel 
   WRITE(301,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(301,'("                                                      ")') 
   WRITE(301,'(" VARS", I3)') nvars
   WRITE(301,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(301,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(301,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(301,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(301,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(301,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(301,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(301,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(301,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(301,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(301,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(301,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(301,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(301,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(301,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(301,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(301,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(301,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(301,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(301,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(301,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(301,'(" u10       0   0    rotated 10m x-wind (m/s)          ")') 
   WRITE(301,'(" v10       0   0    rotated 10m y-wind (m/s)          ")') 
   WRITE(301,'(" uu  ", I7,"   0    rotated x-wind (m/s)              ")') kmz 
   WRITE(301,'(" vv  ", I7,"   0    rotated y-wind (m/s)              ")') kmz 
   WRITE(301,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') kmz 
   WRITE(301,'(" tt  ", I7,"   0    air temperature (K)               ")') kmz 
   WRITE(301,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') kmz 
   WRITE(301,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') kmz 
   WRITE(301,'(" pp  ", I7,"   0    air  pressure  (mb)               ")') kmz 
   WRITE(301,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') kmz 
   WRITE(301,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') kmz 
   WRITE(301,'(" rh  ", I7,"   0    relative humidity (%)             ")') kmz 
   WRITE(301,'(" ENDVARS                                              ")') 
   WRITE(301,'("                                                      ")') 
   
!* GrADS control file for height-level data on standard grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   xxs1 = .5*(1-ims)*dxx + ctr_lon ; xxs_ = dxx !* d01 west  bdry 
   yys1 = .5*(1-jms)*dyy + ctr_lat ; yys_ = dyy !* d01 south bdry 
   
   WRITE(302,'("                                                      ")') 
   WRITE(302,'(" DSET    height_lev.dat                               ")') 
   WRITE(302,'(" TITLE   Height-level data on standard latlon grid    ")') 
   WRITE(302,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(302,'(" UNDEF",E11.2)') undef 
   WRITE(302,'("                                                      ")') 
   WRITE(302,'(" PDEF", I7,  I6,   A10,   4F11.5)') & 
                       imr, jmr, 'ETA.U', ctr_lon, ctr_lat, dxx,  dyy 
   WRITE(302,'(" XDEF", I7,"   LINEAR", 2F11.5 )') ims, xxs1, xxs_ 
   WRITE(302,'(" YDEF", I7,"   LINEAR", 2F11.5 )') jms, yys1, yys_ 
   WRITE(302,'(" ZDEF", I7,"   LEVELS")') kmz 
   WRITE(302,*)  zlevel 
   WRITE(302,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(302,'("                                                      ")') 
   WRITE(302,'(" VECTORPAIRS   u10,v10  uu,vv                         ")') 
   WRITE(302,'(" VARS", I3)') nvars
   WRITE(302,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(302,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(302,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(302,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(302,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(302,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(302,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(302,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(302,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(302,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(302,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(302,'(" ps        0   0    surface air   pressure (mb)       ")') 
   WRITE(302,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(302,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(302,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(302,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(302,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(302,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(302,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(302,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(302,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(302,'(" u10       0   0    10m zonal wind comp (m/s)         ")') 
   WRITE(302,'(" v10       0   0    10m meridional wind (m/s)         ")') 
   WRITE(302,'(" uu  ", I7,"   0    rotated x-wind (m/s)              ")') kmz 
   WRITE(302,'(" vv  ", I7,"   0    rotated y-wind (m/s)              ")') kmz 
   WRITE(302,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') kmz 
   WRITE(302,'(" tt  ", I7,"   0    air temperature (K)               ")') kmz 
   WRITE(302,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') kmz 
   WRITE(302,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') kmz 
   WRITE(302,'(" pp  ", I7,"   0    air  pressure  (mb)               ")') kmz 
   WRITE(302,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') kmz 
   WRITE(302,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') kmz 
   WRITE(302,'(" rh  ", I7,"   0    relative humidity (%)             ")') kmz 
   WRITE(302,'(" ENDVARS                                              ")') 
   WRITE(302,'("                                                      ")') 
   
   CLOSE(300) ; CLOSE(301) ; CLOSE(302) 
   
   ENDIF !* ( height_level ) ------------------------------------------- 

!**************************************************************! 
!  3b. Height-level variables on cylindrical grid (dp = ~25mb) ! 
!______________________________________________________________! 

   ibd1 = 1+ibdcut ; ibd9 = im-ibdcut 
   jbd1 = 1+jbdcut ; jbd9 = jm-jbdcut 
   
   IF ( height_cylin ) THEN !* ----------------------------------------- 

!* Variables on cylindrical grid with south-polar projection 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
!*   radial  dimension: j = 0 -> center of cylinder 
!* azimuthal dimension: i = 0 -> south; + clockwise 
   
   IF ( dr_cylin ==0 ) dr_cylin = nint(dlat*100) 
   jmc = nint(r_cylin/dr_cylin) ; drlen = dr_cylin*1000. 
   
   IF ( dphi_deg > 0 ) THEN 
      imc = nint(360/dphi_deg)  
   ELSE  !* Calculate default azimuthal dimension 
      aaa = jmc*pi_cnst ; aaa = max( aaa, philev(1) ) 
      kk = int(log(aaa/philev(1))/log(10.)) ; xxx = aaa/10**kk 
      DO k = 1, kphi-1 ; IF ( xxx < philev(k+1) ) EXIT ; ENDDO 
      imc = int(philev(k))*10**kk 
   ENDIF ; dphi = 2*pi_cnst/imc 

   Allocate( icen(0:kmz), jcen(0:kmz) ) ; k2d = 22  !* # of 2D fields 
   Allocate( ppc(0:imc,0:jmc,kmz), qqc(0:imc,0:jmc,kmz), cwc(0:imc,0:jmc,kmz) ) 
   Allocate( ttc(0:imc,0:jmc,kmz), tdc(0:imc,0:jmc,kmz), qtc(0:imc,0:jmc,kmz) ) 
   Allocate( uuc(0:imc,0:jmc,kmz), vvc(0:imc,0:jmc,kmz), wwc(0:imc,0:jmc,kmz) ) 
   Allocate( uxc(0:imc,0:jmc,kmz), vxc(0:imc,0:jmc,kmz), mwc(0:imc,0:jmc,kmz) ) 
   Allocate( rhc(0:imc,0:jmc,kmz), wks(0:imc,0:jmc,k2d) ) 
   
   ppc(:,:,:) = undef ; qqc(:,:,:) = undef ; cwc(:,:,:) = undef 
   ttc(:,:,:) = undef ; tdc(:,:,:) = undef ; qtc(:,:,:) = undef 
   uuc(:,:,:) = undef ; vvc(:,:,:) = undef ; wwc(:,:,:) = undef 
   uxc(:,:,:) = undef ; vxc(:,:,:) = undef ; mwc(:,:,:) = undef 
   rhc(:,:,:) = undef ; wks(:,:,:) = undef 

!           *     *     *     *     *     *     *     *     *     * 
   
!* 3D Storm center for tilted cylinder axis 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   icen(:) = iloc ; jcen(:) = jloc !* Storm center by MSLP 
   
   IF ( tilt_cylin ) THEN  !* 3D Storm center by vorticity - - - - - 

   ioff = int(r_inner/dlon) + 1 
   joff = int(r_inner/dlat) + 1 ; rho = r_inner**2 
   iold = iloc ; jold = jloc ; zzz = 0. 
 
   DO k = 1, kmz 
      i1 = iold - ioff ; i1 = max( ibd1, i1  ) 
      i2 = iold + ioff ; i2 = min(  i2, ibd9 ) ; i2 = max(i1,i2)  
      j1 = jold - joff ; j1 = max( jbd1, j1  ) 
      j2 = jold + joff ; j2 = min(  j2, jbd9 ) ; j2 = max(j1,j2)  
      aaa = 0 ; bbb = 0 ; eee = 0 
      DO j = j1, j2 ; DO i = i1, i2 
         rrr = ((i-iold)*dlon)**2 + ((j-jold)*dlat)**2 
         IF ( rrr <= rho ) THEN 
            www = max( vx_(i,j,k), 0. ) ; IF ( www == undef ) www = 0. 
            aaa = aaa + www*i ; bbb = bbb + www*j ; eee = eee + www 
         ENDIF 
      ENDDO ; ENDDO
      IF ( eee == 0. ) THEN 
         icen(k) = iold ; jcen(k) = jold 
      ELSE 
          m0  = int( (zlevel(k)-zzz)*r_slope/yrlen ) 
         itmp = nint(aaa/eee) ; jtmp = nint(bbb/eee) 
         icen(k) = max( iold-m0, min(itmp, iold+m0) ) 
         jcen(k) = max( jold-m0, min(jtmp, jold+m0) ) 
         IF ( icen(k)/=iold .OR. jcen(k)/=jold ) THEN 
            iold = icen(k) ; jold = jcen(k) ; zzz = zlevel(k) 
         ENDIF
      ENDIF
   ENDDO 
 
!  PRINT*, '------------------------------------------------------------' 
!  DO k = 0, kmz ; PRINT*, 'k, icen, jcen =', k, icen(k), jcen(k) ; ENDDO 
!  PRINT*, '------------------------------------------------------------' 
 
   ENDIF !* ( tilt_cylin ) - - - - - - - - - - - - - - - - - - - - - 

!* Interpolate variables to cylindrical grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
!  outward radial wind: uuc =   rcos*uur + rsin*vvr 
!  CCW tangential wind: vvc = - rsin*uur + rcos*vvr 
   
   kout = km+1  ;  xxlen = xrlen(jcen(0)) 
   pi3_2 = 1.5*pi_cnst ; sqrt2_ = 1./sqrt(2.) 
   
   DO j = 1, jmc 
   DO i = 1, imc ; k = 0 !* ---------------------------------------> 
   
         xxx = wcos(icen(k),jcen(k)) 
         yyy = wsin(icen(k),jcen(k)) 
         IF (  xxx > sqrt2_ ) THEN 
            alfa = Asin(yyy) 
         ELSEIF (  xxx  < -sqrt2_ ) THEN 
            alfa = pi_cnst*Sign(1.,yyy) - Asin(yyy) 
         ELSE ! ( |xxx| <= sqrt2_ ) ! 
            alfa = Acos(xxx)*Sign(1.,yyy) 
         ENDIF !* CW SOUTH-POLAR COORD
          phi = pi3_2 - i*dphi - alfa 
         rcos = cos(phi) ; rsin = sin(phi) 
!        - - - - - - - - - - - - - - - - - 
         yyy = j*drlen*rsin/yrlen ; jloc = int(yyy) ; IF (yyy<0) jloc = jloc-1 
         ccc = yyy-jloc ; ddd = 1 - ccc ; n = jcen(k) + jloc ; n1 = n + 1 
         defined = ( jbd1 <= n .AND. n1 <= jbd9 ) 
         IF (defined) xxlen = .5*( xrlen(jcen(k))+ddd*xrlen(n)+ccc*xrlen(n1) ) 
         xxx = j*drlen*rcos/xxlen ; iloc = int(xxx) ; IF (xxx<0) iloc = iloc-1 
         aaa = xxx-iloc ; bbb = 1 - aaa ; m = icen(k) + iloc ; m1 = m + 1 
         defined = ( defined ) .AND. ( ibd1 <= m .AND. m1 <= ibd9 )
   
         IF ( defined ) THEN !* Interpolate  2D fields ~~~~~~~~~~~~~ 
            wks(i,j, 1) = bbb*( ddd*xrs(m ,n) + ccc*xrs(m ,n1) ) &
                        + aaa*( ddd*xrs(m1,n) + ccc*xrs(m1,n1) ) 
            wks(i,j, 2) = bbb*( ddd*yrs(m ,n) + ccc*yrs(m ,n1) ) &
                        + aaa*( ddd*yrs(m1,n) + ccc*yrs(m1,n1) ) 
            wks(i,j, 3) = bbb*( ddd* zs(m ,n) + ccc* zs(m ,n1) ) &
                        + aaa*( ddd* zs(m1,n) + ccc* zs(m1,n1) ) 
            wks(i,j, 4) = bbb*( ddd* sm(m ,n) + ccc* sm(m ,n1) ) &
                        + aaa*( ddd* sm(m1,n) + ccc* sm(m1,n1) ) 
            wks(i,j, 5) = bbb*( ddd*z0m(m ,n) + ccc*z0m(m ,n1) ) &
                        + aaa*( ddd*z0m(m1,n) + ccc*z0m(m1,n1) ) 
            wks(i,j, 6) = bbb*( ddd*z0t(m ,n) + ccc*z0t(m ,n1) ) &
                        + aaa*( ddd*z0t(m1,n) + ccc*z0t(m1,n1) ) 
            wks(i,j, 7) = bbb*( ddd* uf(m ,n) + ccc* uf(m ,n1) ) &
                        + aaa*( ddd* uf(m1,n) + ccc* uf(m1,n1) ) 
            wks(i,j, 8) = bbb*( ddd*prc(m ,n) + ccc*prc(m ,n1) ) &
                        + aaa*( ddd*prc(m1,n) + ccc*prc(m1,n1) ) 
            wks(i,j, 9) = bbb*( ddd*cwh(m ,n) + ccc*cwh(m ,n1) ) &
                        + aaa*( ddd*cwh(m1,n) + ccc*cwh(m1,n1) ) 
            wks(i,j,10) = bbb*( ddd*qqh(m ,n) + ccc*qqh(m ,n1) ) &
                        + aaa*( ddd*qqh(m1,n) + ccc*qqh(m1,n1) ) 
            wks(i,j,11) = bbb*( ddd* ps(m ,n) + ccc* ps(m ,n1) ) &
                        + aaa*( ddd* ps(m1,n) + ccc* ps(m1,n1) ) 
            wks(i,j,12) = bbb*( ddd* p0(m ,n) + ccc* p0(m ,n1) ) &
                        + aaa*( ddd* p0(m1,n) + ccc* p0(m1,n1) ) 
            wks(i,j,13) = bbb*( ddd* tw(m ,n) + ccc* tw(m ,n1) ) &
                        + aaa*( ddd* tw(m1,n) + ccc* tw(m1,n1) ) 
            wks(i,j,14) = bbb*( ddd* qw(m ,n) + ccc* qw(m ,n1) ) &
                        + aaa*( ddd* qw(m1,n) + ccc* qw(m1,n1) ) 
            wks(i,j,15) = bbb*( ddd* ts(m ,n) + ccc* ts(m ,n1) ) &
                        + aaa*( ddd* ts(m1,n) + ccc* ts(m1,n1) ) 
            wks(i,j,16) = bbb*( ddd* qs(m ,n) + ccc* qs(m ,n1) ) &
                        + aaa*( ddd* qs(m1,n) + ccc* qs(m1,n1) ) 
            IF ( max(sst(m,n),sst(m,n1),sst(m1,n),sst(m1,n1)) < undef ) THEN 
            wks(i,j,17) = bbb*( ddd*sst(m ,n) + ccc*sst(m ,n1) ) &
                        + aaa*( ddd*sst(m1,n) + ccc*sst(m1,n1) ) 
            wks(i,j,18) = bbb*( ddd*ssq(m ,n) + ccc*ssq(m ,n1) ) &
                        + aaa*( ddd*ssq(m1,n) + ccc*ssq(m1,n1) ) 
            ENDIF 
            wks(i,j,19) = bbb*( ddd*t10(m ,n) + ccc*t10(m ,n1) ) &
                        + aaa*( ddd*t10(m1,n) + ccc*t10(m1,n1) ) 
            wks(i,j,20) = bbb*( ddd*q10(m ,n) + ccc*q10(m ,n1) ) &
                        + aaa*( ddd*q10(m1,n) + ccc*q10(m1,n1) ) 
                   uur  = bbb*( ddd*u10(m ,n) + ccc*u10(m ,n1) ) &
                        + aaa*( ddd*u10(m1,n) + ccc*u10(m1,n1) ) 
                   vvr  = bbb*( ddd*v10(m ,n) + ccc*v10(m ,n1) ) &
                        + aaa*( ddd*v10(m1,n) + ccc*v10(m1,n1) ) 
            wks(i,j,21) =   rcos*uur + rsin*vvr !*   radial   wind 
            wks(i,j,22) = - rsin*uur + rcos*vvr !* tangential wind 
         ENDIF  !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
      DO k = 1, kmz  !* - - - - - - - - - - - - - - - - - - - - - ->
   
         IF ( tilt_cylin ) THEN !* ----------------------------- 
         xxx = wcos(icen(k),jcen(k)) 
         yyy = wsin(icen(k),jcen(k)) 
         IF (  xxx > sqrt2_ ) THEN 
            alfa = Asin(yyy) 
         ELSEIF (  xxx  < -sqrt2_ ) THEN 
            alfa = pi_cnst*Sign(1.,yyy) - Asin(yyy) 
         ELSE ! ( |xxx| <= sqrt2_ ) ! 
            alfa = Acos(xxx)*Sign(1.,yyy) 
         ENDIF !* SOUTH-POLAR PROJECTION 
          phi = pi3_2 - alfa - i*dphi  
         rcos = cos(phi) ; rsin = sin(phi) 
!        - - - - - - - - - - - - - - - - - 
         yyy = j*drlen*rsin/yrlen ; jloc = int(yyy) ; IF (yyy<0) jloc = jloc-1 
         ccc = yyy-jloc ; ddd = 1 - ccc ; n = jcen(k) + jloc ; n1 = n + 1 
         defined = defined .AND. ( jbd1 <= n .AND. n1 <= jbd9 ) 
         IF (defined) xxlen = .5*( xrlen(jcen(k))+ddd*xrlen(n)+ccc*xrlen(n1) ) 
         xxx = j*drlen*rcos/xxlen ; iloc = int(xxx) ; IF (xxx<0) iloc = iloc-1 
         aaa = xxx-iloc ; bbb = 1 - aaa ; m = icen(k) + iloc ; m1 = m + 1 
         defined = ( defined ) .AND. ( ibd1 <= m .AND. m1 <= ibd9 )
         ENDIF !* ( tilt_cylin ) ------------------------------- 
   
         IF ( .not. defined ) THEN 
            bad_cylin = .true. ; kout = min(k,kout) ; EXIT 
         ENDIF 
   
         eee = max( pp_(m,n,k), pp_(m,n1,k), pp_(m1,n,k), pp_(m1,n1,k) ) 
         eee = max( ux_(m,n,k), ux_(m,n1,k), ux_(m1,n,k), ux_(m1,n1,k), eee ) 
         eee = max( vx_(m,n,k), vx_(m,n1,k), vx_(m1,n,k), vx_(m1,n1,k), eee ) 
   
         IF ( eee < undef ) THEN !* Interpolate 3D fields ~~~~~~~~~~ 
            ppc(i,j,k) = bbb*( ddd*pp_(m ,n,k) + ccc*pp_(m ,n1,k) ) & 
                       + aaa*( ddd*pp_(m1,n,k) + ccc*pp_(m1,n1,k) ) 
            qqc(i,j,k) = bbb*( ddd*qq_(m ,n,k) + ccc*qq_(m ,n1,k) ) & 
                       + aaa*( ddd*qq_(m1,n,k) + ccc*qq_(m1,n1,k) ) 
            cwc(i,j,k) = bbb*( ddd*cw_(m ,n,k) + ccc*cw_(m ,n1,k) ) & 
                       + aaa*( ddd*cw_(m1,n,k) + ccc*cw_(m1,n1,k) ) 
            ttc(i,j,k) = bbb*( ddd*tt_(m ,n,k) + ccc*tt_(m ,n1,k) ) & 
                       + aaa*( ddd*tt_(m1,n,k) + ccc*tt_(m1,n1,k) ) 
            tdc(i,j,k) = bbb*( ddd*td_(m ,n,k) + ccc*td_(m ,n1,k) ) & 
                       + aaa*( ddd*td_(m1,n,k) + ccc*td_(m1,n1,k) ) 
            rhc(i,j,k) = bbb*( ddd*rh_(m ,n,k) + ccc*rh_(m ,n1,k) ) & 
                       + aaa*( ddd*rh_(m1,n,k) + ccc*rh_(m1,n1,k) ) 
            qtc(i,j,k) = bbb*( ddd*qt_(m ,n,k) + ccc*qt_(m ,n1,k) ) & 
                       + aaa*( ddd*qt_(m1,n,k) + ccc*qt_(m1,n1,k) ) 
            uxc(i,j,k) = bbb*( ddd*ux_(m ,n,k) + ccc*ux_(m ,n1,k) ) & 
                       + aaa*( ddd*ux_(m1,n,k) + ccc*ux_(m1,n1,k) ) 
            vxc(i,j,k) = bbb*( ddd*vx_(m ,n,k) + ccc*vx_(m ,n1,k) ) & 
                       + aaa*( ddd*vx_(m1,n,k) + ccc*vx_(m1,n1,k) ) 
            wwc(i,j,k) = bbb*( ddd*ww_(m ,n,k) + ccc*ww_(m ,n1,k) ) & 
                       + aaa*( ddd*ww_(m1,n,k) + ccc*ww_(m1,n1,k) ) 
                  uur  = bbb*( ddd*uu_(m ,n,k) + ccc*uu_(m ,n1,k) ) & 
                       + aaa*( ddd*uu_(m1,n,k) + ccc*uu_(m1,n1,k) ) 
                  vvr  = bbb*( ddd*vv_(m ,n,k) + ccc*vv_(m ,n1,k) ) & 
                       + aaa*( ddd*vv_(m1,n,k) + ccc*vv_(m1,n1,k) ) 
            uuc(i,j,k) =   rcos*uur + rsin*vvr !*   radial   wind 
            vvc(i,j,k) = - rsin*uur + rcos*vvr !* tangential wind 
!           - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
            mwc(i,j,k) =   wwc(i,j,k)*ppc(i,j,k)*100. & 
                       / ( rd_air*(1.+.608*qqc(i,j,k))*ttc(i,j,k) ) 
         ENDIF !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
      ENDDO  !* <- - - - - - - - - - - - - - - - - - - - - - - - - - 
   
   ENDDO !* <------------------------------------------------------- 
   ENDDO 
   
!* Cylindrical Boundary conditions 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   wks(0,:, :) = wks(imc,:,:) ; ii= icen(0); jj= jcen(0)
   wks(:,0, 1) = xrs(ii,jj)   ; wks(:,0, 2) = yrs(ii,jj)
   wks(:,0, 3) =  zs(ii,jj)   ; wks(:,0, 4) =  sm(ii,jj)
   wks(:,0, 5) = z0m(ii,jj)   ; wks(:,0, 6) = z0t(ii,jj)
   wks(:,0, 7) =  uf(ii,jj)   ; wks(:,0, 8) = prc(ii,jj)
   wks(:,0, 9) = cwh(ii,jj)   ; wks(:,0,10) = qqh(ii,jj)
   wks(:,0,11) =  ps(ii,jj)   ; wks(:,0,12) =  p0(ii,jj)
   wks(:,0,13) =  tw(ii,jj)   ; wks(:,0,14) =  qw(ii,jj)
   wks(:,0,15) =  ts(ii,jj)   ; wks(:,0,16) =  qs(ii,jj)
   wks(:,0,17) = sst(ii,jj)   ; wks(:,0,18) = ssq(ii,jj)
   wks(:,0,19) = t10(ii,jj)   ; wks(:,0,20) = q10(ii,jj)
   
   DO j = 0, jmc ; DO i = 0, imc 
   IF ( wks(i,j,4) < .99 ) THEN ; wks(i,j,4) = 0. 
                           ELSE ; wks(i,j,4) = 1. ; ENDIF 
   ENDDO ; ENDDO  !* sea mask *! 
   
   DO k = 1, kmz ; ii = icen(k) ; jj = jcen(k) 
      ppc(0,:,k) = ppc(imc,:,k) ; ppc(:,0,k) = pp_(ii,jj,k) 
      qqc(0,:,k) = qqc(imc,:,k) ; qqc(:,0,k) = qq_(ii,jj,k) 
      cwc(0,:,k) = cwc(imc,:,k) ; cwc(:,0,k) = cw_(ii,jj,k) 
      ttc(0,:,k) = ttc(imc,:,k) ; ttc(:,0,k) = tt_(ii,jj,k) 
      tdc(0,:,k) = tdc(imc,:,k) ; tdc(:,0,k) = td_(ii,jj,k) 
      rhc(0,:,k) = rhc(imc,:,k) ; rhc(:,0,k) = rh_(ii,jj,k) 
      qtc(0,:,k) = qtc(imc,:,k) ; qtc(:,0,k) = qt_(ii,jj,k) 
      uxc(0,:,k) = uxc(imc,:,k) ; uxc(:,0,k) = ux_(ii,jj,k) 
      vxc(0,:,k) = vxc(imc,:,k) ; vxc(:,0,k) = vx_(ii,jj,k) 
      wwc(0,:,k) = wwc(imc,:,k) ; wwc(:,0,k) = ww_(ii,jj,k) 
      uuc(0,:,k) = uuc(imc,:,k) 
      vvc(0,:,k) = vvc(imc,:,k) 
      mwc(0,:,k) = mwc(imc,:,k) 
   ENDDO 
   
!* Output height-level data on cylindrical grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   do k = 1, k2d ; WRITE(310) wks(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) ppc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) qqc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) cwc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) ttc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) tdc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) qtc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) uuc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) vvc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) wwc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) uxc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) vxc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) mwc(:,:,k) ; enddo 
   do k = 1, kmz ; WRITE(310) rhc(:,:,k) ; enddo 
   nvars = k2d + 13 
   
   DeAllocate( icen, jcen, rhc, wks )
   DeAllocate( ppc, qqc, cwc, ttc, tdc, qtc, uuc, vvc, wwc, uxc, vxc, mwc )
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* GrADS control file for height-cylinder data on rectangular grid 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   dxx = 360./float(imc) !* resolution in azimuthal dir (deg) 
   dyy =  drlen/1000.    !* resolution in  radial   dir (km) 
   
   WRITE(311,'("                                                      ")') 
   WRITE(311,'(" DSET    height_cyl.dat                               ")') 
   WRITE(311,'(" TITLE   Height-cylinder data on rectangular grid     ")') 
   WRITE(311,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(311,'(" UNDEF",E11.2)') undef 
   WRITE(311,'("                                                      ")') 
   WRITE(311,'(" XDEF", I7,"   LINEAR", 2F10.3)') imc+1, -180., dxx 
   WRITE(311,'(" YDEF", I7,"   LINEAR", 2F10.3)') jmc+1,   0. , dyy 
   WRITE(311,'(" ZDEF", I7,"   LEVELS")') kmz 
   WRITE(311,*)  zlevel 
   WRITE(311,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(311,'("                                                      ")') 
   WRITE(311,'(" VARS", I3)') nvars
   WRITE(311,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(311,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(311,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(311,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(311,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(311,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(311,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(311,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(311,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(311,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(311,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(311,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(311,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(311,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(311,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(311,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(311,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(311,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(311,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(311,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(311,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(311,'(" u10       0   0    10m   radial   wind (m/s)         ")') 
   WRITE(311,'(" v10       0   0    10m tangential wind (m/s)         ")') 
   WRITE(311,'(" pp  ", I7,"   0    air  pressure  (mb)               ")') kmz 
   WRITE(311,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') kmz 
   WRITE(311,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') kmz 
   WRITE(311,'(" tt  ", I7,"   0    air  temperature  (K)             ")') kmz 
   WRITE(311,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') kmz 
   WRITE(311,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') kmz 
   WRITE(311,'(" uu  ", I7,"   0    outward radial wind (m/s)         ")') kmz 
   WRITE(311,'(" vv  ", I7,"   0    CCW tangential wind (m/s)         ")') kmz 
   WRITE(311,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') kmz 
   WRITE(311,'(" ux  ", I7,"   0    horizontal divergence (/s)        ")') kmz 
   WRITE(311,'(" vx  ", I7,"   0    relative vorticity (/s)           ")') kmz 
   WRITE(311,'(" mw  ", I7,"   0    vertical mass flux (kg/m^2/s)     ")') kmz 
   WRITE(311,'(" rh  ", I7,"   0    relative humidity (%)             ")') kmz 
   WRITE(311,'(" ENDVARS                                              ")') 
   WRITE(311,'("                                                      ")') 
   
!* GrADS control file for height-cylinder data on polar projection 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   dxx = 360./float(imc) !* resolution in azimuthal dir (deg) 
   dyy =  10./float(jmc) !* resolution in  radial   dir (deg) 
   
   WRITE(312,'("                                                      ")') 
   WRITE(312,'(" DSET    height_cyl.dat                               ")') 
   WRITE(312,'(" TITLE   Height-cylinder data on polar projection     ")') 
   WRITE(312,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(312,'(" UNDEF",E11.2)') undef 
   WRITE(312,'("                                                      ")') 
   WRITE(312,'(" XDEF", I7,"   LINEAR", F10.3, F13.6)') imc+1, -180., dxx 
   WRITE(312,'(" YDEF", I7,"   LINEAR", F10.3, F13.6)') jmc+1,  -90., dyy 
   WRITE(312,'(" ZDEF", I7,"   LEVELS")') kmz 
   WRITE(312,*)  zlevel 
   WRITE(312,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(312,'("                                                      ")') 
   WRITE(312,'(" VARS", I3)') nvars
   WRITE(312,'(" xx        0   0    Earth longitude (deg)             ")') 
   WRITE(312,'(" yy        0   0    Earth latitude  (deg)             ")') 
   WRITE(312,'(" zs        0   0    Earth topography (m)              ")') 
   WRITE(312,'(" sm        0   0    Sea mask (0/1: land/sea)          ")') 
   WRITE(312,'(" z0m       0   0    momentum roughness height (m)     ")') 
   WRITE(312,'(" z0t       0   0    thermal  roughness height (m)     ")') 
   WRITE(312,'(" uf        0   0    friction velocity  (m/s)          ")') 
   if (accum_precip) then  !* - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(312,'(" prc       0   0    accumulated precipitation (mm)    ")') 
   else  !* average precip rate - - - - - - - - - - - - - - - - - - - - - 
   WRITE(312,'(" prc       0   0    precipitation rate (mm/hr)        ")') 
   endif !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   WRITE(312,'(" cwh       0   0    column water condensate (mm)      ")') 
   WRITE(312,'(" qqh       0   0    column precipitable water (mm)    ")') 
   WRITE(312,'(" ps        0   0    surface air pressure (mb)         ")') 
   WRITE(312,'(" p0        0   0    mean-sea-level pressure (mb)      ")') 
   WRITE(312,'(" tw        0   0    sensible heat flux (W/m^2)        ")') 
   WRITE(312,'(" qw        0   0    latent   heat flux (W/m^2)        ")') 
   WRITE(312,'(" ts        0   0    surface air temperature (K)       ")') 
   WRITE(312,'(" qs        0   0    surface specific humidity (kg/kg) ")') 
   WRITE(312,'(" sst       0   0    sea-surface temperature (K)       ")') 
   WRITE(312,'(" ssq       0   0    sea-sfc specific humidity (kg/kg) ")') 
   WRITE(312,'(" t10       0   0    10m air temperature (K)           ")') 
   WRITE(312,'(" q10       0   0    10m specific humidity (kg/kg)     ")') 
   WRITE(312,'(" u10       0   0    10m   radial   wind (m/s)         ")') 
   WRITE(312,'(" v10       0   0    10m tangential wind (m/s)         ")') 
   WRITE(312,'(" pp  ", I7,"   0    air  pressure  (mb)               ")') kmz 
   WRITE(312,'(" qq  ", I7,"   0    specific humidity (kg/kg)         ")') kmz 
   WRITE(312,'(" cw  ", I7,"   0    total condensate  (kg/kg)         ")') kmz 
   WRITE(312,'(" tt  ", I7,"   0    air  temperature  (K)             ")') kmz 
   WRITE(312,'(" td  ", I7,"   0    dewpoint temperature (K)          ")') kmz 
   WRITE(312,'(" qt  ", I7,"   0    equiv. pot. temperature (K)       ")') kmz 
   WRITE(312,'(" uu  ", I7,"   0    outward radial wind (m/s)         ")') kmz 
   WRITE(312,'(" vv  ", I7,"   0    CCW tangential wind (m/s)         ")') kmz 
   WRITE(312,'(" ww  ", I7,"   0    vertical  wind (m/s)              ")') kmz 
   WRITE(312,'(" ux  ", I7,"   0    horizontal divergence (/s)        ")') kmz 
   WRITE(312,'(" vx  ", I7,"   0    relative vorticity (/s)           ")') kmz 
   WRITE(312,'(" mw  ", I7,"   0    vertical mass flux (kg/m^2/s)     ")') kmz 
   WRITE(312,'(" rh  ", I7,"   0    relative humidity (%)             ")') kmz 
   WRITE(312,'(" ENDVARS                                              ")') 
   WRITE(312,'("                                                      ")') 
   
   CLOSE(310) ; CLOSE(311) ; CLOSE(312) 
   
   ENDIF !* ( height_cylin ) ------------------------------------------- 

!**************************************************************! 
!  3c. Diagnostics with height-level variables on rotated grid ! 
!______________________________________________________________! 

   IF ( diagnostics ) THEN  !* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

!* Constant Frequency by Altitude Diagrams (CFADs) 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   vrange = 20. ; binsize = .5 
    mbins = nint(vrange/binsize) 
   Allocate( wwfr(-mbins:mbins,1:km) ) 

   wwfr(:,:) = 0. 
   percent = 100./(im*jm) 
   DO k = 1, km 
      DO j = 1, jm ; DO i = 1, im 
         n = nint( ww(i,j,k)/binsize ) 
         m = max( -mbins, min(n,mbins) ) 
         wwfr(m,k) = wwfr(m,k) + 1. 
      ENDDO ; ENDDO 
      DO m = -mbins, mbins 
         wwfr(m,k) = wwfr(m,k)*percent 
      ENDDO 
   ENDDO 
   
   nvars = 1 
   WRITE(332) wwfr 
   DeAllocate(wwfr) 
   
!* GrADS control file for CFADs 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   imax = 2*mbins + 1 ; jmax = km 
   xxr1 = -vrange ; dxx = binsize 
   
   WRITE(331,'("                                                      ")') 
   WRITE(331,'(" DSET    cfads_demo.dat                               ")') 
   WRITE(331,'(" TITLE   Constant Frequency by Altitude Diagrams      ")') 
   WRITE(331,'(" OPTIONS sequential big_endian                        ")') 
   WRITE(331,'(" UNDEF",E11.2)') undef 
   WRITE(331,'("                                                      ")') 
   WRITE(331,'(" XDEF", I7,"   LINEAR", 2F11.5)') imax,  xxr1, dxx 
   WRITE(331,'(" YDEF", I7,"   LINEAR          1          1 ")') jmax 
   WRITE(331,'(" ZDEF", I7,"   LINEAR          1          1 ")')   1 
   WRITE(331,'(" TDEF", I7,"   LINEAR   00:00Z01JAN0001  1yr")') ntime 
   WRITE(331,'("                                                      ")') 
   WRITE(331,'(" VARS", I3)') nvars
   WRITE(331,'(" wwfr      0   0    vertical velocity distribution    ")') 
   WRITE(331,'(" ENDVARS                                              ")') 
   WRITE(331,'("                                                      ")') 
   
   CLOSE(331) ; CLOSE(332) 
   
   ENDIF  !* ( diagnostics ) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   
   IF ( height_vars ) THEN 
   DeAllocate( uu_, vv_, ww_, tt_, td_, qt_, pp_, qq_, cw_, ux_, vx_, rh_ ) 
   ENDIF 
   
   DeAllocate( xrr, yrr, xrs, yrs, wcos, wsin, urs, vrs, xrlen ) 
   CALL ext_ncd_get_next_time( dh0, DateStr, Status_next_time ) 

   ENDDO !<======== next_time ============================================== 

   DeAllocate( zs, sm, p0, prc, ps, ths, ts, qs, u10, v10, t10, q10 )
   DeAllocate( t02, q02, sst, ssq, tw, qw, cwh, qqh, uf, z0m, z0t ) 
   DeAllocate( uu, vv, ww, tt, td, qt, pp, qq, cw, gh, rh ) 
   CALL ext_ncd_ioclose(dh0, Status)
   
   cpu_time_b = ETIME(btime) 
   aaa = btime(1) - atime(1) ; bbb = cpu_time_b - cpu_time_a 
   WRITE(*,'(" Elapsed  CPU  Time (sec): usr, tot =", 2F12.5)') aaa, bbb 
   if ( bad_cylin ) &  
   WRITE(*,'(" Cylinder falls OUT domain at level =", I9)') kout 
   WRITE(*,'(" Normal Termination  -- ",/)') 
   
END PROGRAM diapost !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

!==============================================================================

!******************************************************************************
! 
!  Buck's dewpoint temperature & relative humidity 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
!  Calculate dewpoint temperature & relative humidity from pressure and
!  specific humidity with Buck's equation for saturation vapor pressure of 
!  water (Buck 1981). The approximated dewpoint is contrained by air temp. 
!  
!______________________________________________________________________________
   
   SUBROUTINE Dewpoint_RH( pp, qq, tt, td, rh, im, jm, km ) 
   
   Implicit None 
!  -------------------------------------------------------------------- [input] 
   Integer  im, jm, km       !* x, y, z dimension 
   Real(4)  pp(im,jm,km)     !* air pressure (mb) 
   Real(4)  qq(im,jm,km)     !* specific humidity (kg/kg) 
   Real(4)  tt(im,jm,km)     !* air temperature (K) 
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - [output] 
   Real(4)  td(im,jm,km)     !* dewpoint temperature (K) 
   Real(4)  rh(im,jm,km)     !* relative humidity (%) 
!  -------------------------------------------------------------------- [local] 
   Real(4)  t0c, e0c, eps, aaa, bbb, eee, ees, rrr, xxx, yyy, ttt, ttd 
   Integer  i, j, k 
   
!           *     *     *     *     *     *     *     *     *     * 
   
!* Dewpoint temperature (K) on sigma-p levels 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   t0c = 273.15  !* Kelvin temperature (K) at 0'C 
   e0c = 6.1121  !* water vapor pressure (mb) at 0'C 
   aaa = 17.502  !* factor for water vapor pressure 
   bbb = 240.97  !* constant for water vapor pressure 
   eps = .62198  !* ratio of vapor/air molecular weights
   
   DO k = 1, km !* ----------------------------------------------------> 
   
   DO j = 1, jm ; DO i = 1, im 
   
      rrr = qq(i,j,k)/( 1. - qq(i,j,k) ) !* mixing ratio 
   
      eee = rrr*pp(i,j,k)/( eps + rrr  ) ; xxx = log(eee/e0c) 
   
      ttd = bbb*xxx/( aaa - xxx ) + t0c  !* Buck's dewpoint (K) 
   
      td(i,j,k) = min( ttd, tt(i,j,k) )  !* constrained dewpoint 
   
      ttt = tt(i,j,k) - t0c ; ees = e0c*exp( aaa*ttt/(ttt+bbb) ) 
   
      rh(i,j,k) = min(100.*eee/ees,100.) !* relative humidity 
   
   ENDDO ; ENDDO 

   ENDDO !* ( k = 1, km ) <---------------------------------------------
   
   END SUBROUTINE Dewpoint_RH 
   
!==============================================================================

!******************************************************************************
! 
!  Convert between standard & rotated latlon coordinates 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
!  This subroutine converts latlon coordinates between two coordinate systems, 
!  the standard geographic system {S} & the rotated spherical coord system {R} 
!  obtained by rotating the origin of {S} to (olat,olon) as the new origin: 
!  (0,0) -> (olat,olon) = (0,0)'. The poles are not allowed for conversion. 
!  
!  mode = 1 : convert standard coord (slat,slon) to rotated  coord (rlat,rlon) 
!  mode = 2 : convert rotated  coord (rlat,rlon) to standard coord (slat,slon) 
!  
!______________________________________________________________________________
   
   SUBROUTINE Convert_LatLon(mode, im, jm, slon, slat, rlon, rlat, olon, olat)
   
   Implicit None 
!  -------------------------------------------------------------------- [input] 
   Integer  mode, im, jm      !* mode= 1/2: Std -> Rot / Rot -> Std coord 
   Real(4)  olon, olat        !* origin of rotated grid in standard coord 
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - [in/out] 
   Real(4)  slon(im,jm), slat(im,jm) !* standard latlon coordinates (deg) 
   Real(4)  rlon(im,jm), rlat(im,jm) !* rotated  latlon coordinates (deg) 
!  -------------------------------------------------------------------- [local] 
   Real(8):: pi_cnst = 3.141592653589793238462643383279502884 
   Real(8):: deg2rad = 3.141592653589793238462643383279502884/180. 
   Real(8)  alon(im,jm), alat(im,jm) !* standard latlon coordinates (deg) 
   Real(8)  blon(im,jm), blat(im,jm) !* rotated  latlon coordinates (deg) 
   Real(8)  olamda, otheta, alamda, atheta, blamda, btheta 
   Real(8)  eps1, sqrt2_,  xcos, xsin,  ycos, ysin 
   Real(8)  aaa, bbb, ccc, ddd, eee, xxx, yyy, zzz 
   Integer   i, j 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   eps1 = 1. - 1.D-6 ; sqrt2_ = 1./sqrt(2.) 
   
!* Enhance precision for trigonometric functions 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   alon(:,:) = slon(:,:) ; alat(:,:) = slat(:,:) 
   blon(:,:) = rlon(:,:) ; blat(:,:) = rlat(:,:) 
   olamda = olon*deg2rad ; otheta = olat*deg2rad 
      aaa = Dcos(otheta) ; bbb = Dsin(otheta) 
   
!* Convert standard Earth latlon to rotated Model latlon 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   IF ( mode == 1 ) THEN  !* - - - - - - - - - - - - - - - - - - - - 
   
   DO j = 1, jm 
   DO i = 1, im 
      yyy = alat(i,j)*deg2rad 
      xxx = alon(i,j)*deg2rad - olamda 
     xcos = Dcos(xxx) ; xsin = Dsin(xxx) 
     ycos = Dcos(yyy) ; ysin = Dsin(yyy) 
!    - - - - - - - - - - - - - - - - - - 
      zzz = aaa*ysin - bbb*ycos*xcos 
      zzz = max(-eps1, min(zzz, eps1)) !* Poles undefined 
     btheta = DAsin(zzz) ; eee = 1./Dcos(btheta) 
      ccc = ( bbb*ysin + aaa*ycos*xcos )*eee 
      ddd = (                ycos*xsin )*eee 
      IF (  ccc > sqrt2_ ) THEN 
         blamda = DAsin(ddd) 
      ELSEIF (  ccc  < -sqrt2_ ) THEN 
         blamda = pi_cnst*Sign(1.D+0,ddd) - DAsin(ddd) 
      ELSE ! ( |ccc| <= sqrt2_ ) ! 
         blamda = DAcos(ccc)*Sign(1.D+0,ddd) 
      ENDIF 
      blat(i,j) = btheta/deg2rad 
      blon(i,j) = blamda/deg2rad 
   ENDDO 
   ENDDO 
   
!* Convert rotated Model latlon to standard Earth latlon
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   ELSE !* ( mode == 2 ) - - - - - - - - - - - - - - - - - - - - - - 
   
   DO j = 1, jm 
   DO i = 1, im 
      yyy = blat(i,j)*deg2rad 
      xxx = blon(i,j)*deg2rad 
     xcos = Dcos(xxx) ; xsin = Dsin(xxx) 
     ycos = Dcos(yyy) ; ysin = Dsin(yyy) 
!    - - - - - - - - - - - - - - - - - - 
      zzz = aaa*ysin + bbb*ycos*xcos 
      zzz = max(-eps1, min(zzz, eps1)) !* Poles undefined 
     atheta = DAsin(zzz) ; eee = 1./Dcos(atheta) 
      ccc = (-bbb*ysin + aaa*ycos*xcos )*eee 
      ddd = (                ycos*xsin )*eee 
      IF (  ccc > sqrt2_ ) THEN 
         alamda = DAsin(ddd) + olamda 
      ELSEIF (  ccc  < -sqrt2_ ) THEN 
         alamda = pi_cnst*Sign(1.D+0,ddd) - DAsin(ddd) + olamda 
      ELSE ! ( |ccc| <= sqrt2_ ) ! 
         alamda = DAcos(ccc)*Sign(1.D+0,ddd) + olamda 
      ENDIF 
      alat(i,j) = atheta/deg2rad 
      alon(i,j) = alamda/deg2rad 
   ENDDO 
   ENDDO 
   
   ENDIF  !* - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
   
!* Recover single precision for output 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   slon(:,:) = alon(:,:) ; slat(:,:) = alat(:,:) 
   rlon(:,:) = blon(:,:) ; rlat(:,:) = blat(:,:) 
   
   END SUBROUTINE Convert_LatLon 
   
!==============================================================================
   
!******************************************************************************
! 
!  Monotonic bi-cubic interpolation to the mid interval 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  
!  -----(1)----------(2)---x------(3)----------(4)----- 
!  
!______________________________________________________________________________
   
   SUBROUTINE Cubic_Interpo( hhh, ff, wtx, wty, undef )
   
   Implicit None 
!  -------------------------------------------------------------------- [input] 
   Real(4):: ff(4,4), undef  !* given values on grid points 
   Real(4):: wtx(4), wty(4)  !* cubic interpolation weights 
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - [output] 
   Real(4):: hhh             !* interpolated value 
!  -------------------------------------------------------------------- [local] 
   Real(4):: gg(4), aaa, bbb, xxx, yyy 
   Integer:: i, j 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   DO i = 1, 4 
      IF ( MaxVal(ff(i,:)) == undef ) THEN 
         gg(i) = undef 
      ELSE
         aaa = min( ff(i,2), ff(i,3) ) ;  bbb  = max( ff(i,2), ff(i,3) ) 
         yyy = SUM(wty(1:4)*ff(i,1:4)) ; gg(i) = max( aaa, min(yyy, bbb) ) 
      ENDIF 
   ENDDO 
   
   IF ( MaxVal(gg(:)) == undef ) THEN 
      hhh = undef 
   ELSE
      aaa = min( gg(2), gg(3) )   ; bbb = max( gg(2), gg(3) ) 
      xxx = SUM(wtx(1:4)*gg(1:4)) ; hhh = max( aaa, min(xxx, bbb) ) 
   ENDIF 
   
   END SUBROUTINE Cubic_Interpo 
   
!==============================================================================
   
!******************************************************************************
! 
!  Monotonic cubic grid refinement with triple resolution 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  
!  ----(1)-----------(2)---|---|---(3)-----------(4)----- 
!  
!______________________________________________________________________________
   
   SUBROUTINE Refine_Grid3x( pp, pp3, undef, im3, jm3, im, jm, km )
   
   Implicit  None 
!  -------------------------------------------------------------------- [input] 
   Integer:: iorder = 4    !* 2/4 
   Integer:: im3, jm3, im, jm, km 
   Real(4):: pp(im,jm,km), pp3(im3,jm3,km), undef 
!  -------------------------------------------------------------------- [local] 
   Real(4):: aaa, bbb, xxx, yyy, a1_3, b2_3, awt(4), bwt(4), ff(4) 
   Integer:: i, j, k, m, n, j1, j2, j3, j4, n0, n1, n2 
   Integer:: m0, m1, m2, mm1, mm2, mm3, mm4 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   SELECT CASE (iorder)  !================================================ 

!> ====================================
   CASE (2)  !* linear interpolation *! 
!> ====================================

!> ====================================
   CASE (4)  !* cubic  interpolation *! 
!> ====================================

    a1_3  =  1./3.  ;  b2_3  =  2./3. 
   awt(1) = -5./81. ; awt(2) = 20./27. ; awt(3) = 10./27. ; awt(4) = -4./81.  
   bwt(1) = -4./81. ; bwt(2) = 10./27. ; bwt(3) = 20./27. ; bwt(4) = -5./81.  
   
   DO k = 1, km !* ----------------------------------------------------> 
   
!* Refine the grid in Y-direction 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO i = 1, im ; m0 = 3*(i-1) + 1 
   
      j = jm ; n0 = 3*(j-1) + 1 
      pp3(m0,n0,k) = pp(i,j,k) 
   
      DO j = 1, jm-1, jm-2 ; j2 = j ; j3 = j+1 
         n0 = 3*(j-1) + 1  ; n1 = n0+1 ; n2 = n0+2 
         ff(2:3) = pp(i,j2:j3,k) ; pp3(m0,n0,k) = ff(2) 
         IF ( max(ff(2), ff(3)) == undef ) THEN 
            pp3(m0,n1,k) = undef ; pp3(m0,n2,k) = undef 
	 ELSE
            pp3(m0,n1,k) = b2_3*ff(2) + a1_3*ff(3) 
            pp3(m0,n2,k) = a1_3*ff(2) + b2_3*ff(3) 
         ENDIF 
      ENDDO 
   
      DO j = 2, jm-2  ; j1 = j-1 ; j4 = j+2 
         n0 = 3*(j-1) + 1 ; n1 = n0+1 ; n2 = n0+2 
         ff(1:4) = pp(i,j1:j4,k) ; pp3(m0,n0,k) = ff(2) 
	 aaa = min(ff(2), ff(3)) ; bbb = max(ff(2), ff(3)) 
         IF ( MaxVal(ff(:)) == undef ) THEN 
            pp3(m0,n1,k) = undef ; pp3(m0,n2,k) = undef 
	 ELSE
            xxx = SUM(awt(:)*ff(:)) ; pp3(m0,n1,k) = max( aaa, min(xxx, bbb) ) 
            yyy = SUM(bwt(:)*ff(:)) ; pp3(m0,n2,k) = max( aaa, min(yyy, bbb) ) 
         ENDIF 
      ENDDO 
   
   ENDDO 
   
!* Refine the grid in X-direction 
!* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   DO n = 1, jm3 
   
      DO i = 1, im-1, im-2 ; m0 = 3*(i-1) + 1 
	 m1 = m0+1 ; m2 = m0+2 ; mm2 = m0 ; mm3 = m0+3 
         ff(2) = pp3(mm2,n,k) ; ff(3) = pp3(mm3,n,k) 
         IF ( max(ff(2),ff(3)) == undef ) THEN 
            pp3(m1,n,k) = undef ; pp3(m2,n,k) = undef 
	 ELSE
            pp3(m1,n,k) = b2_3*ff(2) + a1_3*ff(3) 
            pp3(m2,n,k) = a1_3*ff(2) + b2_3*ff(3) 
         ENDIF 
      ENDDO 
   
      DO i = 2, im-2 
         m0 = 3*(i-1) + 1 ; m1 = m0+1 ; m2 = m0+2 
         mm1 = m0-3 ; mm2 = m0 ; mm3 = m0+3 ; mm4 = m0+6 
         ff(1) =  pp3(mm1,n,k) ; ff(2) =  pp3(mm2,n,k) 
         ff(3) =  pp3(mm3,n,k) ; ff(4) =  pp3(mm4,n,k) 
	 aaa = min(ff(2), ff(3)) ; bbb = max(ff(2), ff(3)) 
         IF ( MaxVal(ff(:)) == undef ) THEN 
            pp3(m1,n,k) = undef ; pp3(m2,n,k) = undef 
	 ELSE
	    xxx = SUM(awt(:)*ff(:)) ; pp3(m1,n,k) = max( aaa, min(xxx, bbb) ) 
	    yyy = SUM(bwt(:)*ff(:)) ; pp3(m2,n,k) = max( aaa, min(yyy, bbb) ) 
         ENDIF 
      ENDDO 
   
   ENDDO 
   
   ENDDO !* <-----------------------------------------------------------
   
   END SELECT  !========================================================== 

   END SUBROUTINE Refine_Grid3x 
   
!==============================================================================
   
!******************************************************************************
! 
!  Convert a scalar variable from E to A-grid 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  This subroutine converts a scalar variable from E to A-grid with doubled 
!  grid points assuming that im = 2*ime-1, jm = jme (ime, jme = odd numbers). 
!  
!______________________________________________________________________________
   
   SUBROUTINE E2A_Grid_Scalar( hh, im, jm, km, ee, ime, jme, kme ) 
   
   Implicit None 
!  -------------------------------------------------------------------- [input] 
   Integer  im , jm , km     !* x, y, z-dimension of target A-grid 
   Integer  ime, jme, kme    !* x, y, z-dimension of source E-grid 
   Real(4)  ee(ime,jme,kme)  !* original data on the source E-grid 
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - [output] 
   Real(4)  hh(im,jm,km)     !* extended data on the target A-grid 
!  -------------------------------------------------------------------- [local] 
   Integer  ii, i, j, k 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   DO k = 1, km !* ----------------------------------------------------> 
   
   DO j = 2, jm-1, 2 
      DO i = 2, im-1, 2 ; ii = i/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      i = 1  ; ii = (i+1)/2 
      hh(i,j,k) = .50*( ee(ii,j-1,k) + ee(ii,j+1,k) ) 
      DO i = 3, im-2, 2 ; ii = (i+1)/2 
         hh(i,j,k) = .25*( ee(ii-1,j,k)+ee(ii,j,k)+ee(ii,j-1,k)+ee(ii,j+1,k) ) 
      ENDDO 
      i = im ; ii = (i+1)/2 
      hh(i,j,k) = .50*( ee(ii,j-1,k) + ee(ii,j+1,k) ) 
   ENDDO 
   
      j = 1 
      DO i = 1, im, 2 ; ii = (i+1)/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      DO i = 2, im-1, 2 ; ii = i/2 
         hh(i,j,k) = .50*( ee(ii+1,j,k) + ee(ii,j,k) ) 
      ENDDO 
   
   DO j = 3, jm-2, 2 
      DO i = 1, im, 2 ; ii = (i+1)/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      DO i = 2, im-1, 2 ; ii = i/2 
         hh(i,j,k) = .25*( ee(ii+1,j,k)+ee(ii,j,k)+ee(ii,j-1,k)+ee(ii,j+1,k) ) 
      ENDDO 
   ENDDO 
   
      j = jm 
      DO i = 1, im, 2 ; ii = (i+1)/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      DO i = 2, im-1, 2 ; ii = i/2 
         hh(i,j,k) = .50*( ee(ii+1,j,k) + ee(ii,j,k) ) 
      ENDDO 

   ENDDO !* ( k = 1, km ) <---------------------------------------------
   
   END SUBROUTINE E2A_Grid_Scalar 
   
!==============================================================================

!******************************************************************************
! 
!  Convert a vector component from E to A-grid 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  This subroutine converts a vector component from E to A-grid with doubled 
!  grid points assuming that im = 2*ime-1, jm = jme (ime, jme = odd numbers). 
!  
!______________________________________________________________________________
   
   SUBROUTINE E2A_Grid_Vector( uu, im, jm, km, ee, ime, jme, kme ) 
   
   Implicit None 
!  -------------------------------------------------------------------- [input] 
   Integer  im , jm , km     !* x, y, z-dimension of target A-grid 
   Integer  ime, jme, kme    !* x, y, z-dimension of source E-grid 
   Real(4)  ee(ime,jme,kme)  !* original data on the source E-grid 
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - [output] 
   Real(4)  uu(im,jm,km)     !* extended data on the target A-grid 
!  -------------------------------------------------------------------- [local] 
   Integer  ii, i, j, k 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   DO k = 1, km !* ----------------------------------------------------> 
   
   DO j = 2, jm-1, 2 
      DO i = 1, im, 2 ; ii = (i+1)/2 
         uu(i,j,k) = ee(ii,j,k) 
      ENDDO 
      DO i = 2, im-1, 2 ; ii = i/2 
         uu(i,j,k) = .25*( ee(ii+1,j,k)+ee(ii,j,k)+ee(ii,j-1,k)+ee(ii,j+1,k) ) 
      ENDDO 
   ENDDO 
   
      j = 1 
      DO i = 2, im-1, 2 ; ii = i/2 
         uu(i,j,k) = ee(ii,j,k) 
      ENDDO 
      i = 1  ; ii = (i+1)/2 
      uu(i,j,k) = .25*( 3*ee(ii,j,k)-ee(ii+1,j,k)+3*ee(ii,j+1,k)-ee(ii,j+3,k) ) 
      DO i = 3, im-2, 2 ; ii = (i+1)/2 
         uu(i,j,k) = .50*( ee(ii-1,j,k) + ee(ii,j,k) ) 
      ENDDO 
      i = im ; ii = (i+1)/2 
      uu(i,j,k) = .25*(3*ee(ii-1,j,k)-ee(ii-2,j,k)+3*ee(ii,j+1,k)-ee(ii,j+3,k)) 
   
   DO j = 3, jm-2, 2 
      DO i = 2, im-1, 2 ; ii = i/2 
         uu(i,j,k) = ee(ii,j,k) 
      ENDDO 
      i = 1  ; ii = (i+1)/2 
      uu(i,j,k) = .50*( ee(ii,j-1,k) + ee(ii,j+1,k) ) 
      DO i = 3, im-2, 2 ; ii = (i+1)/2 
         uu(i,j,k) = .25*( ee(ii-1,j,k)+ee(ii,j,k)+ee(ii,j-1,k)+ee(ii,j+1,k) ) 
      ENDDO 
      i = im ; ii = (i+1)/2 
      uu(i,j,k) = .50*( ee(ii,j-1,k) + ee(ii,j+1,k) ) 
   ENDDO 
   
      j = jm
      DO i = 2, im-1, 2 ; ii = i/2 
         uu(i,j,k) = ee(ii,j,k) 
      ENDDO 
      i = 1  ; ii = (i+1)/2 
      uu(i,j,k) = .25*( 3*ee(ii,j,k)-ee(ii+1,j,k)+3*ee(ii,j-1,k)-ee(ii,j-3,k) ) 
      DO i = 3, im-2, 2 ; ii = (i+1)/2 
         uu(i,j,k) = .50*( ee(ii-1,j,k) + ee(ii,j,k) ) 
      ENDDO 
      i = im ; ii = (i+1)/2 
      uu(i,j,k) = .25*(3*ee(ii-1,j,k)-ee(ii-2,j,k)+3*ee(ii,j-1,k)-ee(ii,j-3,k)) 
   
   ENDDO !* ( k = 1, km ) <---------------------------------------------
   
   END SUBROUTINE E2A_Grid_Vector 
   
!==============================================================================
   
!******************************************************************************
! 
!  Convert a mis-located vector component from E to A-grid 
!  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  This subroutine converts a mis-located vector component from E to A-grid 
!  with doubled grid points assuming im = 2*ime-1, jm = jme (ime, jme = odd). 
!  
!______________________________________________________________________________
   
   SUBROUTINE E2A_Debug_Vector( hh, im, jm, km, ee, ime, jme, kme ) 
   
   Implicit None 
!  -------------------------------------------------------------------- [input] 
   Integer  im , jm , km     !* x, y, z-dimension of target A-grid 
   Integer  ime, jme, kme    !* x, y, z-dimension of source E-grid 
   Real(4)  ee(ime,jme,kme)  !* original data on the source E-grid 
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - [output] 
   Real(4)  hh(im,jm,km)     !* extended data on the target A-grid 
!  -------------------------------------------------------------------- [local] 
   Integer  ii, i, j, k 
   
!           *     *     *     *     *     *     *     *     *     * 
   
   DO k = 1, km !* ----------------------------------------------------> 
   
   DO j = 2, jm-1, 2 
      DO i = 2, im-1, 2 ; ii = i/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      DO i = 3, im-2, 2 ; ii = (i+1)/2 
         hh(i,j,k) = .25*( ee(ii-1,j,k)+ee(ii,j,k)+ee(ii,j-1,k)+ee(ii,j+1,k) ) 
      ENDDO 
      i = im ; ii = (i+1)/2 
      hh(i,j,k) = .50*( ee(ii,j-1,k) + ee(ii,j+1,k) ) 
   ENDDO 
   
      j = 1 
      DO i = 3, im, 2 ; ii = (i+1)/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      i = 2 ; ii = i/2 
      hh(i,j,k) = .25*(3*ee(ii+1,j,k)-ee(ii+2,j,k)+3*ee(ii,j+1,k)-ee(ii,j+3,k)) 
      DO i = 4, im-1, 2 ; ii = i/2 
         hh(i,j,k) = .50*( ee(ii+1,j,k) + ee(ii,j,k) ) 
      ENDDO 
   
   DO j = 3, jm-2, 2 
      DO i = 3, im, 2 ; ii = (i+1)/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      i = 2 ; ii = i/2 
      hh(i,j,k) = .50*( ee(ii,j-1,k) + ee(ii,j+1,k) ) 
      DO i = 4, im-1, 2 ; ii = i/2 
         hh(i,j,k) = .25*( ee(ii+1,j,k)+ee(ii,j,k)+ee(ii,j-1,k)+ee(ii,j+1,k) ) 
      ENDDO 
   ENDDO 
   
      j = jm 
      DO i = 3, im, 2 ; ii = (i+1)/2 
         hh(i,j,k) = ee(ii,j,k) 
      ENDDO 
      i = 2 ; ii = i/2 
      hh(i,j,k) = .25*(3*ee(ii+1,j,k)-ee(ii+2,j,k)+3*ee(ii,j-1,k)-ee(ii,j-3,k)) 
      DO i = 4, im-1, 2 ; ii = i/2 
         hh(i,j,k) = .50*( ee(ii+1,j,k) + ee(ii,j,k) ) 
      ENDDO 
   
      DO j = 1, jm 
         hh(1,j,k) = 2*hh(2,j,k) - hh(3,j,k) 
      ENDDO 

   ENDDO !* ( k = 1, km ) <---------------------------------------------
   
   END SUBROUTINE E2A_Debug_Vector 
   
!==============================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ 

MODULE read_util_module

#ifdef crayx1
#define iargc ipxfargc
#endif

CONTAINS !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

#ifdef crayx1
   SUBROUTINE getarg(i, harg)
      implicit none
      character(len=*) :: harg
      integer :: ierr, ilen, i
      CALL pxfgetarg(i, harg, ilen, ierr)
      return
   END SUBROUTINE getarg
#endif

!------------------------------------------------------------------------------

   SUBROUTINE arguments(v2file, lmore)

   implicit none
   character(len=*) :: v2file
   character(len=120) :: harg
   logical :: lmore
   integer :: ierr, i, numarg
   integer, external :: iargc

   numarg = iargc()

   i = 1
   lmore = .false.

   DO while ( i < numarg)
      CALL getarg(i, harg)
      print*, 'harg = ', trim(harg)
      IF (harg == "-v") THEN
         i = i + 1
         lmore = .true.
      ELSEIF (harg == "-h") THEN
         CALL help
      ENDIF
   ENDDO

   CALL getarg(i,harg)
   v2file = harg

   END SUBROUTINE arguments

!------------------------------------------------------------------------------

   SUBROUTINE help

   implicit none
   character(len=120) :: cmd
   call getarg(0, cmd)

   write(*,'(/,"Usage: ", A, " [-v] v2file ")') trim(cmd)
   write(*,'(8x, "-v     : Print extra info")')
   write(*,'(8x, "v3file : MM5v3 file name to read.")')
   write(*,'(8x, "-h     : print this help message and exit.",/)')
   STOP

   END SUBROUTINE help

END MODULE read_util_module  !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

!==============================================================================

LOGICAL FUNCTION iveceq( a, b, n )
   implicit none
   integer n
   integer a(n), b(n)
   integer i
   iveceq = .true.
   do i = 1,n
      if ( a(i) .ne. b(i) ) iveceq = .false.
   enddo
   return
END FUNCTION iveceq

!* Stop for routine called by module_wrf_error
!* (used by netcdf implementation of IO api)

SUBROUTINE wrf_abort
   STOP
END SUBROUTINE wrf_abort

SUBROUTINE get_current_time_string( time_str )
   CHARACTER(LEN=*), INTENT(OUT) :: time_str
   time_str = ''
END SUBROUTINE get_current_time_string

SUBROUTINE get_current_grid_name( grid_str )
   CHARACTER(LEN=*), INTENT(OUT) :: grid_str
   grid_str = ''
END SUBROUTINE get_current_grid_name

!============================================================================== 
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ 
